@@ -1,26 +1,32 @@
-class JobInfo:
+from typing import Generic, TypeVar
+
+JobType = TypeVar("JobType")
+JobKey = TypeVar("JobKey")
+
+
+class JobInfo(Generic[JobType, JobKey]):
     """
     Stores information of each job.
     """
-    def __init__(self, key, job):
+
+    def __init__(self, key: JobKey, job: JobType):
         self.key = key
-        self.jobs = [(job, '')]
+        self.jobs = [(job, "")]
 
         self.narrowing_count = 0  # not used
 
     def __hash__(self):
-        return hash(self.key)
+        return hash((self.key,) + tuple(self.jobs))
 
     def __eq__(self, o):
-        return type(self) == type(o) and \
-               self.key == o.key
+        return type(self) == type(o) and self.key == o.key and self.job == o.job
 
     def __repr__(self):
         s = "<JobInfo %s>" % (str(self.key))
         return s
 
     @property
-    def job(self):
+    def job(self) -> JobType:
         """
         Get the latest available job.
 
@@ -33,13 +39,13 @@ class JobInfo:
     @property
     def merged_jobs(self):
         for job, job_type in self.jobs:
-            if job_type == 'merged':
+            if job_type == "merged":
                 yield job
 
     @property
     def widened_jobs(self):
         for job, job_type in self.jobs:
-            if job_type == 'widened':
+            if job_type == "widened":
                 yield job
 
     def add_job(self, job, merged=False, widened=False):
@@ -50,9 +56,9 @@ class JobInfo:
         :param bool widened: Whether it is a widened job or not.
         """
 
-        job_type = ''
+        job_type = ""
         if merged:
-            job_type = 'merged'
+            job_type = "merged"
         elif widened:
-            job_type = 'widened'
+            job_type = "widened"
         self.jobs.append((job, job_type))
