@@ -13,7 +13,7 @@ class strcpy(Func):
     non_null = list(range(1, 256))
 
     def __init__(self):
-        super(strcpy, self).__init__() #pylint disable=useless-super-delegation
+        super().__init__()  # pylint disable=useless-super-delegation
 
     def get_name(self):
         return "strcpy"
@@ -21,7 +21,7 @@ class strcpy(Func):
     def num_args(self):
         return 2
 
-    def args(self): #pylint disable=no-self-use
+    def args(self):  # pylint disable=no-self-use
         return ["dst", "src"]
 
     def can_call_other_funcs(self):
@@ -31,7 +31,7 @@ class strcpy(Func):
         # TODO we don't check the return val, some cases I saw char * strcpy, some size_t strcpy
         strlen = random.randint(1, 80)
         buf = rand_str(strlen, byte_list=strcpy.non_null) + b"\x00"
-        result_buf = rand_str(strlen+1)
+        result_buf = rand_str(strlen + 1)
         test_input = [result_buf, buf]
         test_output = [buf, buf]
         max_steps = 20
