@@ -1,4 +1,3 @@
-
 import logging
 
 from .base import SimSootStmt
@@ -10,5 +9,4 @@ l = logging.getLogger(name=__name__)
 class SimSootStmt_Throw(SimSootStmt):
     def _execute(self):
         # TODO: implement simprocedure to throw exception
-        self._add_jmp_target(target=SootAddressTerminator(),
-                             condition=self.state.solver.true)
+        self._add_jmp_target(target=SootAddressTerminator(), condition=self.state.solver.true)
