@@ -1,13 +1,20 @@
+from typing import List, Optional, Union, overload, Type
 import logging
-from typing import List, Optional, Union, overload
+import archinfo
 from archinfo.arch_soot import ArchSoot, SootAddressDescriptor
 
-
 from .sim_state import SimState
-from .calling_conventions import DEFAULT_CC, SimRegArg, SimStackArg, PointerWrapper
+from .calling_conventions import default_cc, SimRegArg, SimStackArg, PointerWrapper, SimCCUnknown
 from .callable import Callable
 from .errors import AngrAssemblyError
-from .engines import UberEngine, ProcedureEngine, SimEngineConcrete
+from .engines import UberEngine, ProcedureEngine, SimEngineConcrete, SimEngine
+from .sim_type import SimTypeFunction, SimTypeInt
+
+try:
+    from .engines import UberEnginePcode
+    from .engines.pcode import register_pcode_arch_default_cc
+except ImportError:
+    UberEnginePcode = None
 
 
 l = logging.getLogger(name=__name__)
@@ -17,13 +24,25 @@ class AngrObjectFactory:
     """
     This factory provides access to important analysis elements.
     """
-    def __init__(self, project, default_engine=None):
+
+    def __init__(self, project, default_engine: Optional[Type[SimEngine]] = None):
         if default_engine is None:
-            default_engine = UberEngine
+            if isinstance(project.arch, archinfo.ArchPcode) and UberEnginePcode is not None:
+                l.warning("Creating project with the experimental 'UberEnginePcode' engine")
+                default_engine_n = UberEnginePcode
+            else:
+                default_engine_n = UberEngine
+        else:
+            default_engine_n = default_engine
+
+        if isinstance(project.arch, archinfo.ArchPcode):
+            register_pcode_arch_default_cc(project.arch)
 
         self.project = project
-        self._default_cc = DEFAULT_CC[project.arch.name]
-        self.default_engine = default_engine(project)
+        self._default_cc = default_cc(
+            project.arch.name, platform=project.simos.name if project.simos is not None else None, default=SimCCUnknown
+        )
+        self.default_engine = default_engine_n(project)
         self.procedure_engine = ProcedureEngine(project)
 
         if project.concrete_target:
@@ -32,16 +51,16 @@ class AngrObjectFactory:
             self.concrete_engine = None
 
     def snippet(self, addr, jumpkind=None, **block_opts):
-        if self.project.is_hooked(addr) and jumpkind != 'Ijk_NoHook':
+        if self.project.is_hooked(addr) and jumpkind != "Ijk_NoHook":
             hook = self.project._sim_procedures[addr]
-            size = hook.kwargs.get('length', 0)
+            size = hook.kwargs.get("length", 0)
             return HookNode(addr, size, self.project.hooked_by(addr))
         elif self.project.simos.is_syscall_addr(addr):
             syscall = self.project.simos.syscall_from_addr(addr)
-            size = syscall.kwargs.get('length', 0)
+            size = syscall.kwargs.get("length", 0)
             return SyscallNode(addr, size, syscall)
         else:
-            return self.block(addr, **block_opts).codenode # pylint: disable=no-member
+            return self.block(addr, **block_opts).codenode  # pylint: disable=no-member
 
     def successors(self, *args, engine=None, **kwargs):
         """
@@ -104,20 +123,7 @@ class AngrObjectFactory:
         execution begins at a special SimProcedure that plays the role of the dynamic loader, calling each of the
         initializer functions that should be called before execution reaches the entry point.
 
-        :param addr:            The address the state should start at instead of the entry point.
-        :param initial_prefix:  If this is provided, all symbolic registers will hold symbolic values with names
-                                prefixed by this string.
-        :param fs:              a dictionary of file names with associated preset SimFile objects.
-        :param concrete_fs:     boolean describing whether the host filesystem should be consulted when opening files.
-        :param chroot:          a path to use as a fake root directory, behaves similar to a real chroot. used only when
-                                concrete_fs is set to True.
-        :param argc:            a custom value to use for the program's argc. May be either an int or a bitvector. If
-                                not provided, defaults to the length of args.
-        :param args:            a list of values to use as arguments to the program. May be mixed strings and bitvectors.
-        :param env:             a dictionary to use as the environment for the program. Both keys and values may be
-                                mixed strings and bitvectors.
-        :return:                The fully initialized state.
-        :rtype:                 SimState
+        It can take any of the arguments that can be provided to ``entry_state``, except for ``addr``.
         """
         return self.project.simos.state_full_init(**kwargs)
 
@@ -128,7 +134,7 @@ class AngrObjectFactory:
         :param addr:            The address the state should start at instead of the entry point.
         :param args:            Any additional positional arguments will be used as arguments to the function call.
 
-        The following parametrs are optional.
+        The following parameters are optional.
 
         :param base_state:      Use this SimState as the base for the new state instead of a blank state.
         :param cc:              Optionally provide a SimCC object to use a specific calling convention.
@@ -165,11 +171,14 @@ class AngrObjectFactory:
         """
         return self.project.simos.state_call(addr, *args, **kwargs)
 
-    def simulation_manager(self, thing: Optional[Union[List[SimState], SimState]]=None, **kwargs) -> 'SimulationManager':
+    def simulation_manager(
+        self, thing: Optional[Union[List[SimState], SimState]] = None, **kwargs
+    ) -> "SimulationManager":
         """
         Constructs a new simulation manager.
 
-        :param thing:           Optional - What to put in the new SimulationManager's active stash (either a SimState or a list of SimStates).
+        :param thing:           What to put in the new SimulationManager's active stash (either a SimState or a list of
+                                SimStates).
         :param kwargs:          Any additional keyword arguments will be passed to the SimulationManager constructor
         :returns:               The new SimulationManager
         :rtype:                 angr.sim_manager.SimulationManager
@@ -179,15 +188,16 @@ class AngrObjectFactory:
         * If nothing is passed in, the SimulationManager is seeded with a state initialized for the program
           entry point, i.e. :meth:`entry_state()`.
         * If a :class:`SimState` is passed in, the SimulationManager is seeded with that state.
-        * If a list is passed in, the list must contain only SimStates and the whole list will be used to seed the SimulationManager.
+        * If a list is passed in, the list must contain only SimStates and the whole list will be used to seed the
+          SimulationManager.
         """
         if thing is None:
-            thing = [ self.entry_state() ]
+            thing = [self.entry_state()]
         elif isinstance(thing, (list, tuple)):
             if any(not isinstance(val, SimState) for val in thing):
                 raise AngrError("Bad type to initialize SimulationManager")
         elif isinstance(thing, SimState):
-            thing = [ thing ]
+            thing = [thing]
         else:
             raise AngrError("BadType to initialze SimulationManager: %s" % repr(thing))
 
@@ -199,12 +209,24 @@ class AngrObjectFactory:
         """
         return self.simulation_manager(*args, **kwargs)
 
-    def callable(self, addr, concrete_only=False, perform_merge=True, base_state=None, toc=None, cc=None):
+    def callable(
+        self,
+        addr,
+        prototype=None,
+        concrete_only=False,
+        perform_merge=True,
+        base_state=None,
+        toc=None,
+        cc=None,
+        add_options=None,
+        remove_options=None,
+    ):
         """
         A Callable is a representation of a function in the binary that can be interacted with like a native python
         function.
 
         :param addr:            The address of the function to use
+        :param prototype:         The prototype of the call to use, as a string or a SimTypeFunction
         :param concrete_only:   Throw an exception if the execution splits into multiple states
         :param perform_merge:   Merge all result states into one at the end (only relevant if concrete_only=False)
         :param base_state:      The state from which to do these runs
@@ -214,28 +236,22 @@ class AngrObjectFactory:
                                 python function.
         :rtype:                 angr.callable.Callable
         """
-        return Callable(self.project,
-                        addr=addr,
-                        concrete_only=concrete_only,
-                        perform_merge=perform_merge,
-                        base_state=base_state,
-                        toc=toc,
-                        cc=cc)
+        return Callable(
+            self.project,
+            addr=addr,
+            prototype=prototype,
+            concrete_only=concrete_only,
+            perform_merge=perform_merge,
+            base_state=base_state,
+            toc=toc,
+            cc=cc,
+            add_options=add_options,
+            remove_options=remove_options,
+        )
 
-    def cc(self, args=None, ret_val=None, sp_delta=None, func_ty=None):
+    def cc(self):
         """
-        Return a SimCC (calling convention) parametrized for this project and, optionally, a given function.
-
-        :param args:        A list of argument storage locations, as SimFunctionArguments.
-        :param ret_val:     The return value storage location, as a SimFunctionArgument.
-        :param sp_delta:    Does this even matter??
-        :param func_ty:     The prototype for the given function, as a SimType or a C-style function declaration that
-                            can be parsed into a SimTypeFunction instance.
-
-        Example func_ty strings:
-        >>> "int func(char*, int)"
-        >>> "int f(int, int, int*);"
-        Function names are ignored.
+        Return a SimCC (calling convention) parameterized for this project.
 
         Relevant subclasses of SimFunctionArgument are SimRegArg and SimStackArg, and shortcuts to them can be found on
         this `cc` object.
@@ -243,61 +259,83 @@ class AngrObjectFactory:
         For stack arguments, offsets are relative to the stack pointer on function entry.
         """
 
-        return self._default_cc(arch=self.project.arch,
-                                  args=args,
-                                  ret_val=ret_val,
-                                  sp_delta=sp_delta,
-                                  func_ty=func_ty)
+        return self._default_cc(arch=self.project.arch)
 
-    def cc_from_arg_kinds(self, fp_args, ret_fp=None, sizes=None, sp_delta=None, func_ty=None):
+    def function_prototype(self):
         """
-        Get a SimCC (calling convention) that will extract floating-point/integral args correctly.
-
-        :param arch:        The Archinfo arch for this CC
-        :param fp_args:     A list, with one entry for each argument the function can take. True if the argument is fp,
-                            false if it is integral.
-        :param ret_fp:      True if the return value for the function is fp.
-        :param sizes:       Optional: A list, with one entry for each argument the function can take. Each entry is the
-                            size of the corresponding argument in bytes.
-        :param sp_delta:    The amount the stack pointer changes over the course of this function - CURRENTLY UNUSED
-        :param func_ty:     A SimType for the function itself or a C-style function declaration that can be parsed into
-                            a SimTypeFunction instance.
-
-        Example func_ty strings:
-        >>> "int func(char*, int)"
-        >>> "int f(int, int, int*);"
-        Function names are ignored.
-
+        Return a default function prototype parameterized for this project and SimOS.
         """
-        return self._default_cc.from_arg_kinds(arch=self.project.arch,
-                fp_args=fp_args,
-                ret_fp=ret_fp,
-                sizes=sizes,
-                sp_delta=sp_delta,
-                func_ty=func_ty)
+        return SimTypeFunction(tuple(), SimTypeInt()).with_arch(self.project.arch)
 
-    #pylint: disable=unused-argument, no-self-use, function-redefined
+    # pylint: disable=unused-argument, no-self-use, function-redefined
     @overload
-    def block(self, addr: int, size=None, max_size=None, byte_string=None, vex=None, thumb=False, backup_state=None,
-              extra_stop_points=None, opt_level=None, num_inst=None, traceflags=0,
-              insn_bytes=None, insn_text=None,  # backward compatibility
-              strict_block_end=None, collect_data_refs=False, cross_insn_opt=True,
-              ) -> 'Block': ...
+    def block(
+        self,
+        addr: int,
+        size=None,
+        max_size=None,
+        byte_string=None,
+        vex=None,
+        thumb=False,
+        backup_state=None,
+        extra_stop_points=None,
+        opt_level=None,
+        num_inst=None,
+        traceflags=0,
+        insn_bytes=None,
+        insn_text=None,  # backward compatibility
+        strict_block_end=None,
+        collect_data_refs=False,
+        cross_insn_opt=True,
+        load_from_ro_regions=False,
+        initial_regs=None,
+    ) -> "Block":
+        ...
 
-    #pylint: disable=unused-argument, no-self-use, function-redefined
+    # pylint: disable=unused-argument, no-self-use, function-redefined
     @overload
-    def block(self, addr: SootAddressDescriptor, size=None, max_size=None, byte_string=None, vex=None, thumb=False, backup_state=None,
-              extra_stop_points=None, opt_level=None, num_inst=None, traceflags=0,
-              insn_bytes=None, insn_text=None,  # backward compatibility
-              strict_block_end=None, collect_data_refs=False, cross_insn_opt=True,
-              ) -> 'SootBlock': ...
-
-    def block(self, addr, size=None, max_size=None, byte_string=None, vex=None, thumb=False, backup_state=None,
-              extra_stop_points=None, opt_level=None, num_inst=None, traceflags=0,
-              insn_bytes=None, insn_text=None,  # backward compatibility
-              strict_block_end=None, collect_data_refs=False, cross_insn_opt=True,
-              ):
+    def block(
+        self,
+        addr: SootAddressDescriptor,
+        size=None,
+        max_size=None,
+        byte_string=None,
+        vex=None,
+        thumb=False,
+        backup_state=None,
+        extra_stop_points=None,
+        opt_level=None,
+        num_inst=None,
+        traceflags=0,
+        insn_bytes=None,
+        insn_text=None,  # backward compatibility
+        strict_block_end=None,
+        collect_data_refs=False,
+        cross_insn_opt=True,
+    ) -> "SootBlock":
+        ...
 
+    def block(
+        self,
+        addr,
+        size=None,
+        max_size=None,
+        byte_string=None,
+        vex=None,
+        thumb=False,
+        backup_state=None,
+        extra_stop_points=None,
+        opt_level=None,
+        num_inst=None,
+        traceflags=0,
+        insn_bytes=None,
+        insn_text=None,  # backward compatibility
+        strict_block_end=None,
+        collect_data_refs=False,
+        cross_insn_opt=True,
+        load_from_ro_regions=False,
+        initial_regs=None,
+    ):
         if isinstance(self.project.arch, ArchSoot) and isinstance(addr, SootAddressDescriptor):
             return SootBlock(addr, arch=self.project.arch, project=self.project)
 
@@ -311,25 +349,37 @@ class AngrObjectFactory:
             byte_string = self.project.arch.asm(insn_text, addr=addr, as_bytes=True, thumb=thumb)
             if byte_string is None:
                 # assembly failed
-                raise AngrAssemblyError("Assembling failed. Please make sure keystone is installed, and the assembly"
-                                        " string is correct.")
+                raise AngrAssemblyError(
+                    "Assembling failed. Please make sure keystone is installed, and the assembly string is correct."
+                )
 
         if max_size is not None:
             l.warning('Keyword argument "max_size" has been deprecated for block(). Please use "size" instead.')
             size = max_size
-        return Block(addr, project=self.project, size=size, byte_string=byte_string, vex=vex,
-                     extra_stop_points=extra_stop_points, thumb=thumb, backup_state=backup_state,
-                     opt_level=opt_level, num_inst=num_inst, traceflags=traceflags,
-                     strict_block_end=strict_block_end, collect_data_refs=collect_data_refs,
-                     cross_insn_opt=cross_insn_opt,
-         )
+        return Block(
+            addr,
+            project=self.project,
+            size=size,
+            byte_string=byte_string,
+            vex=vex,
+            extra_stop_points=extra_stop_points,
+            thumb=thumb,
+            backup_state=backup_state,
+            opt_level=opt_level,
+            num_inst=num_inst,
+            traceflags=traceflags,
+            strict_block_end=strict_block_end,
+            collect_data_refs=collect_data_refs,
+            cross_insn_opt=cross_insn_opt,
+            load_from_ro_regions=load_from_ro_regions,
+            initial_regs=initial_regs,
+        )
 
     def fresh_block(self, addr, size, backup_state=None):
         return Block(addr, project=self.project, size=size, backup_state=backup_state)
 
     cc.SimRegArg = SimRegArg
     cc.SimStackArg = SimStackArg
-    _default_cc = None
     callable.PointerWrapper = PointerWrapper
     call_state.PointerWrapper = PointerWrapper
 
