@@ -7,12 +7,9 @@ l = logging.getLogger(name=__name__)
 
 
 class GetSimpleName(JavaSimProcedure):
+    __provides__ = (("java.lang.Class", "getSimpleName()"),)
 
-    __provides__ = (
-        ("java.lang.Class", "getSimpleName()"),
-    )
-
-    def run(self, this): # pylint: disable=arguments-differ
-        class_simple_name = this.type.split('.')[-1]
+    def run(self, this):  # pylint: disable=arguments-differ
+        class_simple_name = this.type.split(".")[-1]
         str_ref = SimSootValue_StringRef.new_string(self.state, self.state.solver.StringV(class_simple_name))
         return str_ref
