@@ -1,4 +1,3 @@
-
 """
 Manage OS-level configuration.
 """
@@ -12,6 +11,7 @@ from .linux import SimLinux
 from .cgc import SimCGC
 from .windows import SimWindows
 from .javavm import SimJavaVM
+from .snimmuc_nxp import SimSnimmucNxp
 
 os_mapping = defaultdict(lambda: SimOS)
 
@@ -24,7 +24,8 @@ def register_simos(name, cls):
 for k, v in _DESCR_EI_OSABI.items():
     register_simos(v, SimLinux)
 
-register_simos('linux', SimLinux)
-register_simos('windows', SimWindows)
-register_simos('cgc', SimCGC)
-register_simos('javavm', SimJavaVM)
+register_simos("linux", SimLinux)
+register_simos("windows", SimWindows)
+register_simos("cgc", SimCGC)
+register_simos("javavm", SimJavaVM)
+register_simos("snimmuc_nxp", SimSnimmucNxp)
