@@ -1,10 +1,9 @@
-
 from claripy import Annotation
 
 
 class StackLocationAnnotation(Annotation):
     def __init__(self, offset):
-        super(StackLocationAnnotation, self).__init__()
+        super().__init__()
 
         self.offset = offset
 
@@ -17,7 +16,7 @@ class StackLocationAnnotation(Annotation):
         return False
 
     def __hash__(self):
-        return hash(('stack_location', self.offset))
+        return hash(("stack_location", self.offset))
 
     def __eq__(self, other):
         if not isinstance(other, StackLocationAnnotation):
@@ -28,7 +27,7 @@ class StackLocationAnnotation(Annotation):
 
 class VariableSourceAnnotation(Annotation):
     def __init__(self, block_addr, stmt_idx, ins_addr):
-        super(VariableSourceAnnotation, self).__init__()
+        super().__init__()
 
         self.block_addr = block_addr
         self.stmt_idx = stmt_idx
@@ -49,9 +48,9 @@ class VariableSourceAnnotation(Annotation):
         if not isinstance(other, VariableSourceAnnotation):
             return False
 
-        return self.block_addr == other.block_addr and \
-               self.stmt_idx == other.stmt_idx and \
-               self.ins_addr == other.ins_addr
+        return (
+            self.block_addr == other.block_addr and self.stmt_idx == other.stmt_idx and self.ins_addr == other.ins_addr
+        )
 
     @staticmethod
     def from_state(state):
