@@ -1,9 +1,8 @@
-
 import logging
 
 from . import JNISimProcedure
 
-l = logging.getLogger('angr.procedures.java_jni.global_and_local_references')
+l = logging.getLogger("angr.procedures.java_jni.global_and_local_references")
 
 # pylint: disable=arguments-differ,unused-argument
 
@@ -11,44 +10,47 @@ l = logging.getLogger('angr.procedures.java_jni.global_and_local_references')
 # NewGlobalRef / NewWeakGlobalRef
 #
 
-class NewGlobalRef(JNISimProcedure):
 
-    return_ty = 'reference'
+class NewGlobalRef(JNISimProcedure):
+    return_ty = "reference"
 
     def run(self, ptr_env, obj_):
         obj = self.state.jni_references.lookup(obj_)
         return self.state.jni_references.create_new_reference(obj, global_ref=True)
 
+
 #
 # DeleteGlobalRef / DeleteWeakGlobalRef
 #
 
-class DeleteGlobalRef(JNISimProcedure):
 
-    return_ty = 'void'
+class DeleteGlobalRef(JNISimProcedure):
+    return_ty = "void"
 
     def run(self, ptr_env, obj_):
         self.state.jni_references.delete_reference(obj_, global_ref=True)
 
+
 #
 # NewLocalRef
 #
 
-class NewLocalRef(JNISimProcedure):
 
-    return_ty = 'reference'
+class NewLocalRef(JNISimProcedure):
+    return_ty = "reference"
 
     def run(self, ptr_env, obj_):
         obj = self.state.jni_references.lookup(obj_)
         return self.state.jni_references.create_new_reference(obj)
 
+
 #
 # DeleteLocalRef
 #
 
-class DeleteLocalRef(JNISimProcedure):
 
-    return_ty = 'void'
+class DeleteLocalRef(JNISimProcedure):
+    return_ty = "void"
 
     def run(self, ptr_env, obj_):
         self.state.jni_references.delete_reference(obj_)
