@@ -14,6 +14,7 @@ import claripy
 
 l = logging.getLogger("angr.vault")
 
+
 class VaultPickler(pickle.Pickler):
     def __init__(self, vault, file, *args, assigned_objects=(), **kwargs):
         """
@@ -36,6 +37,7 @@ class VaultPickler(pickle.Pickler):
         # l.debug("Persistent store: %s %s", obj, pid)
         return self.vault._store(obj, pid)
 
+
 class VaultUnpickler(pickle.Unpickler):
     def __init__(self, vault, file, *args, **kwargs):
         super().__init__(file, *args, **kwargs)
@@ -44,6 +46,7 @@ class VaultUnpickler(pickle.Unpickler):
     def persistent_load(self, pid):
         return self.vault._load(pid)
 
+
 class Vault(collections.abc.MutableMapping):
     """
     The vault is a serializer for angr.
@@ -81,13 +84,16 @@ class Vault(collections.abc.MutableMapping):
         self.stored = set()
         self.storing = set()
         self.hash_dedup = {
-            claripy.ast.Base, claripy.ast.BV, claripy.ast.FP, claripy.ast.Bool, claripy.ast.Int, claripy.ast.Bits,
-        }
-        self.module_dedup = set() # {'claripy', 'angr', 'archinfo', 'pyvex' } # cle causes recursion
-        self.uuid_dedup = { SimState, Project }
-        self.unsafe_key_baseclasses = {
-            claripy.ast.Base, SimType
+            claripy.ast.Base,
+            claripy.ast.BV,
+            claripy.ast.FP,
+            claripy.ast.Bool,
+            claripy.ast.Int,
+            claripy.ast.Bits,
         }
+        self.module_dedup = set()  # {'claripy', 'angr', 'archinfo', 'pyvex' } # cle causes recursion
+        self.uuid_dedup = {SimState, Project}
+        self.unsafe_key_baseclasses = {claripy.ast.Base, SimType}
 
     def _get_persistent_id(self, o):
         """
@@ -99,7 +105,7 @@ class Vault(collections.abc.MutableMapping):
             self._object_cache[oid] = o
             return oid
 
-        if any(isinstance(o,c) for c in self.unsafe_key_baseclasses):
+        if any(isinstance(o, c) for c in self.unsafe_key_baseclasses):
             return None
 
         try:
@@ -109,10 +115,10 @@ class Vault(collections.abc.MutableMapping):
         except TypeError:
             return None
 
-        #if type(o) in self.uuid_dedup:
+        # if type(o) in self.uuid_dedup:
         #    return self._get_id(o)
-        if o.__class__.__module__.split('.')[0] in self.module_dedup or o.__class__ in self.uuid_dedup:
-            oid = o.__class__.__name__.split(".")[-1] + '-' + str(uuid.uuid4())
+        if o.__class__.__module__.split(".")[0] in self.module_dedup or o.__class__ in self.uuid_dedup:
+            oid = o.__class__.__name__.split(".")[-1] + "-" + str(uuid.uuid4())
             self._object_cache[oid] = o
             self._uuid_cache[o] = oid
             return oid
@@ -152,15 +158,17 @@ class Vault(collections.abc.MutableMapping):
         except KeyError:
             # l.debug("... cached failed")
             with self._read_context(oid) as u:
-                return VaultUnpickler(self, u).load()
+                # add newly loaded object into the object cache
+                o = VaultUnpickler(self, u).load()
+                self._object_cache[oid] = o
+                return o
 
     def store(self, o):
-
-        actual_id = self._get_persistent_id(o) or "TMP-"+str(uuid.uuid4())
+        actual_id = self._get_persistent_id(o) or "TMP-" + str(uuid.uuid4())
 
         return self._store(o, actual_id)
 
-    def _store(self, o, oid): #pylint:disable=redefined-builtin
+    def _store(self, o, oid):  # pylint:disable=redefined-builtin
         """
         Stores an object and returns its ID.
 
@@ -208,7 +216,6 @@ class Vault(collections.abc.MutableMapping):
         return VaultUnpickler(self, f).load()
 
     def _clear_cache(self):
-
         self._object_cache.clear()
         self._uuid_cache.clear()
         self.stored.clear()
@@ -237,13 +244,15 @@ class Vault(collections.abc.MutableMapping):
     def __len__(self):
         return len(self.keys())
 
+
 class VaultDict(Vault):
     """
     A Vault that uses a dictionary for storage.
     """
+
     def __init__(self, d=None):
         super().__init__()
-        self._dict = { } if d is None else d
+        self._dict = {} if d is None else d
 
     @contextlib.contextmanager
     def _write_context(self, i):
@@ -266,10 +275,12 @@ class VaultDict(Vault):
     def keys(self):
         return self._dict.keys()
 
+
 class VaultDir(Vault):
     """
     A Vault that uses a directory for storage.
     """
+
     def __init__(self, d=None):
         super().__init__()
         self._dir = tempfile.mkdtemp() if d is None else d
@@ -297,6 +308,7 @@ class VaultShelf(VaultDict):
     """
     A Vault that uses a shelve.Shelf for storage.
     """
+
     def __init__(self, path=None):
         self._path = tempfile.mktemp() if path is None else path
         s = shelve.open(self._path, protocol=-1)
@@ -329,14 +341,13 @@ class VaultDirShelf(VaultDict):
             self._clear_cache()
 
     def store(self, o):
-        oid = self._get_persistent_id(o) or "TMP-"+str(uuid.uuid4())
+        oid = self._get_persistent_id(o) or "TMP-" + str(uuid.uuid4())
         shelve_path = os.path.join(self._d, oid)
         with self._locked_shelve(shelve_path):
             super()._store(o, oid)
         return oid
 
     def load(self, oid):
-
         shelve_path = os.path.join(self._d, oid)
         with self._locked_shelve(shelve_path):
             o = super().load(oid)
@@ -349,7 +360,7 @@ class VaultDirShelf(VaultDict):
             if "." not in n:
                 s.add(n)
             else:
-                s.add(n[:n.rfind(".")])  # remove the suffix
+                s.add(n[: n.rfind(".")])  # remove the suffix
         return s
 
 
