@@ -1,9 +1,11 @@
 # pylint: disable=wildcard-import
+# pylint: disable=wrong-import-position
 
-__version__ = (9, 0, 6642)
+__version__ = "9.2.85"
 
 if bytes is str:
-    raise Exception("""
+    raise Exception(
+        """
 
 =-=-=-=-=-=-=-=-=-=-=-=-=  WELCOME TO THE FUTURE!  =-=-=-=-=-=-=-=-=-=-=-=-=-=
 
@@ -16,21 +18,36 @@ vaccinations. It hurts a little bit initially but in the end it's worth it.
 For more information, see here: https://docs.angr.io/appendix/migration
 
 Good luck!
-""")
+"""
+    )
 
-# first: let's set up some bootstrap logging
+from .utils.formatting import setup_terminal
+
+setup_terminal()
+del setup_terminal
+
+# let's set up some bootstrap logging
 import logging
+
 logging.getLogger("angr").addHandler(logging.NullHandler())
 from .misc.loggers import Loggers
+
 loggers = Loggers()
 del Loggers
 del logging
 
+# import hook: pkg_resources is too slow to import, but it is used by some other packages that angr depends on. while
+# we upstream our fixes, we replace it with a light-weight solution.
+from .misc.import_hooks import import_fake_pkg_resources
+
+import_fake_pkg_resources(force=False)
+
 # this must happen first, prior to initializing analyses
 from .sim_procedure import SimProcedure
 from .procedures import SIM_PROCEDURES, SimProcedures, SIM_LIBRARIES
 
 from . import sim_options
+
 options = sim_options  # alias
 
 # enums
@@ -40,7 +57,7 @@ from .state_plugins.inspect import BP_BEFORE, BP_AFTER, BP_BOTH, BP_IPDB, BP_IPY
 from .state_plugins.inspect import BP
 from .state_plugins import SimStatePlugin
 
-from .project import *
+from .project import Project, load_shellcode
 from .errors import *
 from .blade import Blade
 from .simos import SimOS
@@ -51,21 +68,82 @@ from . import analyses
 from . import knowledge_plugins
 from . import exploration_techniques
 from .exploration_techniques import ExplorationTechnique
-from . import type_backend
 from . import sim_type as types
 from .state_hierarchy import StateHierarchy
 
 from .sim_state import SimState
 from . import engines
-from .calling_conventions import DEFAULT_CC, SYSCALL_CC, PointerWrapper, SimCC
-from .storage.file import SimFileBase, SimFile, SimPackets, SimFileStream, SimPacketsStream, SimFileDescriptor, SimFileDescriptorDuplex
+from .calling_conventions import default_cc, DEFAULT_CC, SYSCALL_CC, PointerWrapper, SimCC
+from .storage.file import (
+    SimFileBase,
+    SimFile,
+    SimPackets,
+    SimFileStream,
+    SimPacketsStream,
+    SimFileDescriptor,
+    SimFileDescriptorDuplex,
+)
 from .state_plugins.filesystem import SimMount, SimHostFilesystem
 from .state_plugins.heap import SimHeapBrk, SimHeapPTMalloc, PTChunk
 from . import concretization_strategies
 from .distributed import Server
+from .knowledge_base import KnowledgeBase
 
 # for compatibility reasons
 from . import sim_manager as manager
 
 # now that we have everything loaded, re-grab the list of loggers
 loggers.load_all_loggers()
+
+__all__ = (
+    "SimProcedure",
+    "SIM_PROCEDURES",
+    "SIM_LIBRARIES",
+    "sim_options",
+    "options",
+    "BP_BEFORE",
+    "BP_AFTER",
+    "BP_BOTH",
+    "BP_IPDB",
+    "BP_IPYTHON",
+    "BP",
+    "SimStatePlugin",
+    "Project",
+    "load_shellcode",
+    "Blade",
+    "SimOS",
+    "Block",
+    "SimulationManager",
+    "Analysis",
+    "register_analysis",
+    "analyses",
+    "knowledge_plugins",
+    "exploration_techniques",
+    "ExplorationTechnique",
+    "types",
+    "StateHierarchy",
+    "SimState",
+    "engines",
+    "DEFAULT_CC",
+    "default_cc",
+    "SYSCALL_CC",
+    "PointerWrapper",
+    "SimCC",
+    "SimFileBase",
+    "SimFile",
+    "SimPackets",
+    "SimFileStream",
+    "SimPacketsStream",
+    "SimFileDescriptor",
+    "SimFileDescriptorDuplex",
+    "SimMount",
+    "SimHostFilesystem",
+    "SimHeapBrk",
+    "SimHeapPTMalloc",
+    "PTChunk",
+    "concretization_strategies",
+    "Server",
+    "manager",
+    "SimProcedures",
+    "KnowledgeBase",
+)
