@@ -1,11 +1,16 @@
-
 import logging
 import math
 import types
 from collections import deque, defaultdict
 
 import networkx
-from . import Analysis
+
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from angr.knowledge_plugins import Function
+
+from . import Analysis, CFGEmulated
 
 from ..errors import SimEngineError, SimMemoryError
 
@@ -26,14 +31,14 @@ class UnmatchedStatementsException(Exception):
 
 
 # statement difference classes
-class Difference(object):
+class Difference:
     def __init__(self, diff_type, value_a, value_b):
         self.type = diff_type
         self.value_a = value_a
         self.value_b = value_b
 
 
-class ConstantChange(object):
+class ConstantChange:
     def __init__(self, offset, value_a, value_b):
         self.offset = offset
         self.value_a = value_a
@@ -48,8 +53,8 @@ def _euclidean_dist(vector_a, vector_b):
     :returns:           The euclidean distance between the two vectors.
     """
     dist = 0
-    for (x, y) in zip(vector_a, vector_b):
-        dist += (x-y)*(x-y)
+    for x, y in zip(vector_a, vector_b):
+        dist += (x - y) * (x - y)
     return math.sqrt(dist)
 
 
@@ -64,7 +69,7 @@ def _get_closest_matches(input_attributes, target_attributes):
 
     # for each object in the first set find the objects with the closest target attributes
     for a in input_attributes:
-        best_dist = float('inf')
+        best_dist = float("inf")
         best_matches = []
         for b in target_attributes:
             dist = _euclidean_dist(input_attributes[a], target_attributes[b])
@@ -78,7 +83,7 @@ def _get_closest_matches(input_attributes, target_attributes):
     return closest_matches
 
 
-# from http://rosettacode.org/wiki/Levenshtein_distance
+# from https://rosettacode.org/wiki/Levenshtein_distance
 def _levenshtein_distance(s1, s2):
     """
     :param s1:  A list or string
@@ -94,9 +99,7 @@ def _levenshtein_distance(s1, s2):
             if num1 == num2:
                 new_distances.append(distances[index1])
             else:
-                new_distances.append(1 + min((distances[index1],
-                                             distances[index1+1],
-                                             new_distances[-1])))
+                new_distances.append(1 + min((distances[index1], distances[index1 + 1], new_distances[-1])))
         distances = new_distances
     return distances[-1]
 
@@ -113,7 +116,7 @@ def _normalized_levenshtein_distance(s1, s2, acceptable_differences):
     """
     if len(s1) > len(s2):
         s1, s2 = s2, s1
-        acceptable_differences = set(-i for i in acceptable_differences)
+        acceptable_differences = {-i for i in acceptable_differences}
     distances = range(len(s1) + 1)
     for index2, num2 in enumerate(s2):
         new_distances = [index2 + 1]
@@ -121,9 +124,7 @@ def _normalized_levenshtein_distance(s1, s2, acceptable_differences):
             if num2 - num1 in acceptable_differences:
                 new_distances.append(distances[index1])
             else:
-                new_distances.append(1 + min((distances[index1],
-                                             distances[index1+1],
-                                             new_distances[-1])))
+                new_distances.append(1 + min((distances[index1], distances[index1 + 1], new_distances[-1])))
         distances = new_distances
     return distances[-1]
 
@@ -244,7 +245,7 @@ def compare_statement_dict(statement_1, statement_2):
     return differences
 
 
-class NormalizedBlock(object):
+class NormalizedBlock:
     # block may span multiple calls
     def __init__(self, block, function):
         addresses = [block.addr]
@@ -280,18 +281,18 @@ class NormalizedBlock(object):
 
     def __repr__(self):
         size = sum([b.size for b in self.blocks])
-        return '<Normalized Block for %#x, %d bytes>' % (self.addr, size)
+        return "<Normalized Block for %#x, %d bytes>" % (self.addr, size)
 
 
-class NormalizedFunction(object):
+class NormalizedFunction:
     # a more normalized function
-    def __init__(self, function):
+    def __init__(self, function: "Function"):
         # start by copying the graph
-        self.graph = function.graph.copy()
+        self.graph: networkx.DiGraph = function.graph.copy()
         self.project = function._function_manager._kb._project
-        self.call_sites = dict()
+        self.call_sites = {}
         self.startpoint = function.startpoint
-        self.merged_blocks = dict()
+        self.merged_blocks = {}
         self.orig_function = function
 
         # find nodes which end in call and combine them
@@ -306,8 +307,12 @@ class NormalizedFunction(object):
 
                 # merge if it ends with a single call, and the successor has only one predecessor and succ is after
                 successors = list(self.graph.successors(node))
-                if bl.vex.jumpkind == "Ijk_Call" and len(successors) == 1 and \
-                        len(list(self.graph.predecessors(successors[0]))) == 1 and successors[0].addr > node.addr:
+                if (
+                    bl.vex.jumpkind == "Ijk_Call"
+                    and len(successors) == 1
+                    and len(list(self.graph.predecessors(successors[0]))) == 1
+                    and successors[0].addr > node.addr
+                ):
                     # add edges to the successors of its successor, and delete the original successors
                     succ = list(self.graph.successors(node))[0]
                     for s in self.graph.successors(succ):
@@ -339,11 +344,12 @@ class NormalizedFunction(object):
                 self.call_sites[n] = call_targets
 
 
-class FunctionDiff(object):
+class FunctionDiff:
     """
     This class computes the a diff between two functions.
     """
-    def __init__(self, function_a, function_b, bindiff=None):
+
+    def __init__(self, function_a: "Function", function_b: "Function", bindiff=None):
         """
         :param function_a: The first angr Function object to diff.
         :param function_b: The second angr Function object.
@@ -355,8 +361,8 @@ class FunctionDiff(object):
         self._project_b = self._function_b.project
         self._bindiff = bindiff
 
-        self._attributes_a = dict()
-        self._attributes_a = dict()
+        self._attributes_a = {}
+        self._attributes_b = {}
 
         self._block_matches = set()
         self._unmatched_blocks_from_a = set()
@@ -371,7 +377,7 @@ class FunctionDiff(object):
         """
         if len(self._unmatched_blocks_from_a | self._unmatched_blocks_from_b) > 0:
             return False
-        for (a, b) in self._block_matches:
+        for a, b in self._block_matches:
             if not self.blocks_probably_identical(a, b):
                 return False
         return True
@@ -382,7 +388,7 @@ class FunctionDiff(object):
         :returns: A list of block matches which appear to be identical
         """
         identical_blocks = []
-        for (block_a, block_b) in self._block_matches:
+        for block_a, block_b in self._block_matches:
             if self.blocks_probably_identical(block_a, block_b):
                 identical_blocks.append((block_a, block_b))
         return identical_blocks
@@ -393,7 +399,7 @@ class FunctionDiff(object):
         :returns: A list of block matches which appear to differ
         """
         differing_blocks = []
-        for (block_a, block_b) in self._block_matches:
+        for block_a, block_b in self._block_matches:
             if not self.blocks_probably_identical(block_a, block_b):
                 differing_blocks.append((block_a, block_b))
         return differing_blocks
@@ -404,10 +410,11 @@ class FunctionDiff(object):
         :return: A list of block matches which appear to differ
         """
         differing_blocks = []
-        diffs = dict()
-        for (block_a, block_b) in self._block_matches:
-            if self.blocks_probably_identical(block_a, block_b) and \
-                    not self.blocks_probably_identical(block_a, block_b, check_constants=True):
+        diffs = {}
+        for block_a, block_b in self._block_matches:
+            if self.blocks_probably_identical(block_a, block_b) and not self.blocks_probably_identical(
+                block_a, block_b, check_constants=True
+            ):
                 differing_blocks.append((block_a, block_b))
         for block_a, block_b in differing_blocks:
             ba = NormalizedBlock(block_a, self._function_a)
@@ -544,8 +551,9 @@ class FunctionDiff(object):
                 continue
             # if both are in the binary we'll assume it's okay, although we should really match globals
             # TODO use global matches
-            if self._project_a.loader.main_object.contains_addr(c.value_a) and \
-                    self._project_b.loader.main_object.contains_addr(c.value_b):
+            if self._project_a.loader.main_object.contains_addr(
+                c.value_a
+            ) and self._project_b.loader.main_object.contains_addr(c.value_b):
                 continue
             # if the difference is equal to the difference in block addr's or successor addr's we'll say it's also okay
             if c.value_b - c.value_a in acceptable_differences:
@@ -564,7 +572,7 @@ class FunctionDiff(object):
         return diff_constants
 
     @staticmethod
-    def _compute_block_attributes(function):
+    def _compute_block_attributes(function: NormalizedFunction):
         """
         :param function:    A normalized function object.
         :returns:           A dictionary of basic block addresses to tuples of attributes.
@@ -590,21 +598,20 @@ class FunctionDiff(object):
         return attributes
 
     @staticmethod
-    def _distances_from_function_start(function):
+    def _distances_from_function_start(function: NormalizedFunction):
         """
         :param function:    A normalized Function object.
         :returns:           A dictionary of basic block addresses and their distance to the start of the function.
         """
-        return networkx.single_source_shortest_path_length(function.graph,
-                                                           function.startpoint)
+        return networkx.single_source_shortest_path_length(function.graph, function.startpoint)
 
     @staticmethod
-    def _distances_from_function_exit(function):
+    def _distances_from_function_exit(function: NormalizedFunction):
         """
         :param function:    A normalized Function object.
         :returns:           A dictionary of basic block addresses and their distance to the exit of the function.
         """
-        reverse_graph = function.graph.reverse()
+        reverse_graph: networkx.DiGraph = function.graph.reverse()
         # we aren't guaranteed to have an exit from the function so explicitly add the node
         reverse_graph.add_node("start")
         found_exits = False
@@ -616,7 +623,7 @@ class FunctionDiff(object):
         # if there were no exits (a function with a while 1) let's consider the block with the highest address to
         # be the exit. This isn't the most scientific way, but since this case is pretty rare it should be okay
         if not found_exits:
-            last = max(function.graph.nodes(), key=lambda x:x.addr)
+            last = max(function.graph.nodes(), key=lambda x: x.addr)
             reverse_graph.add_edge("start", last)
 
         dists = networkx.single_source_shortest_path_length(reverse_graph, "start")
@@ -635,27 +642,29 @@ class FunctionDiff(object):
         Computes the diff of the functions and saves the result.
         """
         # get the attributes for all blocks
-        l.debug("Computing diff of functions: %s, %s",
-                ("%#x" % self._function_a.startpoint.addr) if self._function_a.startpoint is not None else "None",
-                ("%#x" % self._function_b.startpoint.addr) if self._function_b.startpoint is not None else "None"
-                )
+        l.debug(
+            "Computing diff of functions: %s, %s",
+            ("%#x" % self._function_a.startpoint.addr) if self._function_a.startpoint is not None else "None",
+            ("%#x" % self._function_b.startpoint.addr) if self._function_b.startpoint is not None else "None",
+        )
         self.attributes_a = self._compute_block_attributes(self._function_a)
         self.attributes_b = self._compute_block_attributes(self._function_b)
 
         # get the initial matches
-        initial_matches = self._get_block_matches(self.attributes_a, self.attributes_b,
-                                                  tiebreak_with_block_similarity=False)
+        initial_matches = self._get_block_matches(
+            self.attributes_a, self.attributes_b, tiebreak_with_block_similarity=False
+        )
 
         # Use a queue so we process matches in the order that they are found
         to_process = deque(initial_matches)
 
         # Keep track of which matches we've already added to the queue
-        processed_matches = set((x, y) for (x, y) in initial_matches)
+        processed_matches = {(x, y) for (x, y) in initial_matches}
 
         # Keep a dict of current matches, which will be updated if better matches are found
-        matched_a = dict()
-        matched_b = dict()
-        for (x, y) in processed_matches:
+        matched_a = {}
+        matched_b = {}
+        for x, y in processed_matches:
             matched_a[x] = y
             matched_b[y] = x
 
@@ -671,7 +680,7 @@ class FunctionDiff(object):
             block_b_pred = list(self._function_b.graph.predecessors(block_b))
 
             # propagate the difference in blocks as delta
-            delta = tuple((i-j) for i, j in zip(self.attributes_b[block_b], self.attributes_a[block_a]))
+            delta = tuple((i - j) for i, j in zip(self.attributes_b[block_b], self.attributes_a[block_a]))
 
             # get possible new matches
             new_matches = []
@@ -682,13 +691,25 @@ class FunctionDiff(object):
                 ordered_succ_b = self._get_ordered_successors(self._project_b, block_b, block_b_succ)
                 new_matches.extend(zip(ordered_succ_a, ordered_succ_b))
 
-            new_matches += self._get_block_matches(self.attributes_a, self.attributes_b, block_a_succ, block_b_succ,
-                                                   delta, tiebreak_with_block_similarity=True)
-            new_matches += self._get_block_matches(self.attributes_a, self.attributes_b, block_a_pred, block_b_pred,
-                                                   delta, tiebreak_with_block_similarity=True)
+            new_matches += self._get_block_matches(
+                self.attributes_a,
+                self.attributes_b,
+                block_a_succ,
+                block_b_succ,
+                delta,
+                tiebreak_with_block_similarity=True,
+            )
+            new_matches += self._get_block_matches(
+                self.attributes_a,
+                self.attributes_b,
+                block_a_pred,
+                block_b_pred,
+                delta,
+                tiebreak_with_block_similarity=True,
+            )
 
             # for each of the possible new matches add it if it improves the matching
-            for (x, y) in new_matches:
+            for x, y in new_matches:
                 if (x, y) not in processed_matches:
                     processed_matches.add((x, y))
                     l.debug("FunctionDiff: checking if (%#x, %#x) is better", x.addr, y.addr)
@@ -706,11 +727,11 @@ class FunctionDiff(object):
                         to_process.appendleft((x, y))
 
         # reformat matches into a set of pairs
-        self._block_matches = set((x, y) for (x, y) in matched_a.items())
+        self._block_matches = {(x, y) for (x, y) in matched_a.items()}
 
         # get the unmatched blocks
-        self._unmatched_blocks_from_a = set(x for x in self._function_a.graph.nodes() if x not in matched_a)
-        self._unmatched_blocks_from_b = set(x for x in self._function_b.graph.nodes() if x not in matched_b)
+        self._unmatched_blocks_from_a = {x for x in self._function_a.graph.nodes() if x not in matched_a}
+        self._unmatched_blocks_from_b = {x for x in self._function_b.graph.nodes() if x not in matched_b}
 
     @staticmethod
     def _get_ordered_successors(project, block, succ):
@@ -725,14 +746,21 @@ class FunctionDiff(object):
                     ordered_succ.append(x)
 
             # add the rest (sorting might be better than no order)
-            for s in sorted(succ - set(ordered_succ), key=lambda x:x.addr):
+            for s in sorted(succ - set(ordered_succ), key=lambda x: x.addr):
                 ordered_succ.append(s)
             return ordered_succ
         except (SimMemoryError, SimEngineError):
-            return sorted(succ, key=lambda x:x.addr)
+            return sorted(succ, key=lambda x: x.addr)
 
-    def _get_block_matches(self, attributes_a, attributes_b, filter_set_a=None, filter_set_b=None, delta=(0, 0, 0),
-                           tiebreak_with_block_similarity=False):
+    def _get_block_matches(
+        self,
+        attributes_a,
+        attributes_b,
+        filter_set_a=None,
+        filter_set_b=None,
+        delta=(0, 0, 0),
+        tiebreak_with_block_similarity=False,
+    ):
         """
         :param attributes_a:    A dict of blocks to their attributes
         :param attributes_b:    A dict of blocks to their attributes
@@ -757,9 +785,9 @@ class FunctionDiff(object):
 
         # add delta
         for k in filtered_attributes_a:
-            filtered_attributes_a[k] = tuple((i+j) for i, j in zip(filtered_attributes_a[k], delta))
+            filtered_attributes_a[k] = tuple((i + j) for i, j in zip(filtered_attributes_a[k], delta))
         for k in filtered_attributes_b:
-            filtered_attributes_b[k] = tuple((i+j) for i, j in zip(filtered_attributes_b[k], delta))
+            filtered_attributes_b[k] = tuple((i + j) for i, j in zip(filtered_attributes_b[k], delta))
 
         # get closest
         closest_a = _get_closest_matches(filtered_attributes_a, filtered_attributes_b)
@@ -823,8 +851,10 @@ class FunctionDiff(object):
 
         # get the difference between the data segments
         # this is hackish
-        if ".bss" in self._project_a.loader.main_object.sections_map and \
-                ".bss" in self._project_b.loader.main_object.sections_map:
+        if (
+            ".bss" in self._project_a.loader.main_object.sections_map
+            and ".bss" in self._project_b.loader.main_object.sections_map
+        ):
             bss_a = self._project_a.loader.main_object.sections_map[".bss"].min_addr
             bss_b = self._project_b.loader.main_object.sections_map[".bss"].min_addr
             acceptable_differences.add(bss_b - bss_a)
@@ -837,6 +867,7 @@ class BinDiff(Analysis):
     """
     This class computes the a diff between two binaries represented by angr Projects
     """
+
     def __init__(self, other_project, enable_advanced_backward_slicing=False, cfg_a=None, cfg_b=None):
         """
         :param other_project: The second project to diff
@@ -846,17 +877,21 @@ class BinDiff(Analysis):
         back_traversal = not enable_advanced_backward_slicing
 
         if cfg_a is None:
-            #self.cfg_a = self.project.analyses.CFG(resolve_indirect_jumps=True)
-            #self.cfg_b = other_project.analyses.CFG(resolve_indirect_jumps=True)
-            self.cfg_a = self.project.analyses.CFGEmulated(context_sensitivity_level=1,
-                                                            keep_state = True,
-                                                            enable_symbolic_back_traversal = back_traversal,
-                                                            enable_advanced_backward_slicing = enable_advanced_backward_slicing)
+            # self.cfg_a = self.project.analyses.CFG(resolve_indirect_jumps=True)
+            # self.cfg_b = other_project.analyses.CFG(resolve_indirect_jumps=True)
+            self.cfg_a = self.project.analyses[CFGEmulated].prep()(
+                context_sensitivity_level=1,
+                keep_state=True,
+                enable_symbolic_back_traversal=back_traversal,
+                enable_advanced_backward_slicing=enable_advanced_backward_slicing,
+            )
 
-            self.cfg_b = other_project.analyses.CFGEmulated(context_sensitivity_level=1,
-                                                            keep_state = True,
-                                                            enable_symbolic_back_traversal = back_traversal,
-                                                            enable_advanced_backward_slicing = enable_advanced_backward_slicing)
+            self.cfg_b = other_project.analyses[CFGEmulated].prep()(
+                context_sensitivity_level=1,
+                keep_state=True,
+                enable_symbolic_back_traversal=back_traversal,
+                enable_advanced_backward_slicing=enable_advanced_backward_slicing,
+            )
 
         else:
             self.cfg_a = cfg_a
@@ -865,10 +900,10 @@ class BinDiff(Analysis):
         l.debug("Done computing cfg's")
 
         self._p2 = other_project
-        self._attributes_a = dict()
-        self._attributes_a = dict()
+        self._attributes_a = {}
+        self._attributes_b = {}
 
-        self._function_diffs = dict()
+        self._function_diffs = {}
         self.function_matches = set()
         self._unmatched_functions_from_a = set()
         self._unmatched_functions_from_b = set()
@@ -898,7 +933,7 @@ class BinDiff(Analysis):
         :returns: A list of function matches that appear to be identical
         """
         identical_funcs = []
-        for (func_a, func_b) in self.function_matches:
+        for func_a, func_b in self.function_matches:
             if self.functions_probably_identical(func_a, func_b):
                 identical_funcs.append((func_a, func_b))
         return identical_funcs
@@ -909,7 +944,7 @@ class BinDiff(Analysis):
         :returns: A list of function matches that appear to differ
         """
         different_funcs = []
-        for (func_a, func_b) in self.function_matches:
+        for func_a, func_b in self.function_matches:
             if not self.functions_probably_identical(func_a, func_b):
                 different_funcs.append((func_a, func_b))
         return different_funcs
@@ -919,7 +954,7 @@ class BinDiff(Analysis):
         :return: A list of function matches that appear to differ including just by constants
         """
         different_funcs = []
-        for (func_a, func_b) in self.function_matches:
+        for func_a, func_b in self.function_matches:
             if not self.functions_probably_identical(func_a, func_b, check_consts=True):
                 different_funcs.append((func_a, func_b))
         return different_funcs
@@ -930,7 +965,7 @@ class BinDiff(Analysis):
         :returns: A list of block matches that appear to differ
         """
         differing_blocks = []
-        for (func_a, func_b) in self.function_matches:
+        for func_a, func_b in self.function_matches:
             differing_blocks.extend(self.get_function_diff(func_a, func_b).differing_blocks)
         return differing_blocks
 
@@ -940,7 +975,7 @@ class BinDiff(Analysis):
         :return A list of all block matches that appear to be identical
         """
         identical_blocks = []
-        for (func_a, func_b) in self.function_matches:
+        for func_a, func_b in self.function_matches:
             identical_blocks.extend(self.get_function_diff(func_a, func_b).identical_blocks)
         return identical_blocks
 
@@ -949,8 +984,8 @@ class BinDiff(Analysis):
         """
         :return: A dict of block matches with differing constants to the tuple of constants
         """
-        diffs = dict()
-        for (func_a, func_b) in self.function_matches:
+        diffs = {}
+        for func_a, func_b in self.function_matches:
             diffs.update(self.get_function_diff(func_a, func_b).blocks_with_differing_constants)
         return diffs
 
@@ -979,7 +1014,7 @@ class BinDiff(Analysis):
         :returns:    a dictionary of function addresses to tuples of attributes
         """
         # the attributes we use are the number of basic blocks, number of edges, and number of subfunction calls
-        attributes = dict()
+        attributes = {}
         all_funcs = set(cfg.kb.callgraph.nodes())
         for function_addr in cfg.kb.functions:
             # skip syscalls and functions which are None in the cfg
@@ -1013,15 +1048,14 @@ class BinDiff(Analysis):
         basic_block_matches = fd.block_matches
         function_a = fd._function_a
         function_b = fd._function_b
-        for (a, b) in basic_block_matches:
+        for a, b in basic_block_matches:
             if a in function_a.call_sites and b in function_b.call_sites:
                 # add them in order
                 for target_a, target_b in zip(function_a.call_sites[a], function_b.call_sites[b]):
                     possible_matches.add((target_a, target_b))
                 # add them in reverse, since if a new call was added the ordering from each side
                 # will remain constant until the change
-                for target_a, target_b in zip(reversed(function_a.call_sites[a]),
-                                              reversed(function_b.call_sites[b])):
+                for target_a, target_b in zip(reversed(function_a.call_sites[a]), reversed(function_b.call_sites[b])):
                     possible_matches.add((target_a, target_b))
 
         return possible_matches
@@ -1033,15 +1067,15 @@ class BinDiff(Analysis):
                 plt_matches.append((addr, self._p2.loader.main_object.plt[name]))
 
         # in the case of sim procedures the actual sim procedure might be in the interfunction graph, not the plt entry
-        func_to_addr_a = dict()
-        func_to_addr_b = dict()
-        for (k, hook) in self.project._sim_procedures.items():
+        func_to_addr_a = {}
+        func_to_addr_b = {}
+        for k, hook in self.project._sim_procedures.items():
             if "resolves" in hook.kwargs:
-                func_to_addr_a[hook.kwargs['resolves']] = k
+                func_to_addr_a[hook.kwargs["resolves"]] = k
 
-        for (k, hook) in self._p2._sim_procedures.items():
+        for k, hook in self._p2._sim_procedures.items():
             if "resolves" in hook.kwargs:
-                func_to_addr_b[hook.kwargs['resolves']] = k
+                func_to_addr_b[hook.kwargs["resolves"]] = k
 
         for name, addr in func_to_addr_a.items():
             if name in func_to_addr_b:
@@ -1084,19 +1118,19 @@ class BinDiff(Analysis):
         initial_matches = self._get_plt_matches()
         initial_matches += self._get_name_matches()
         initial_matches += self._get_function_matches(self.attributes_a, self.attributes_b)
-        for (a, b) in initial_matches:
+        for a, b in initial_matches:
             l.debug("Initially matched (%#x, %#x)", a, b)
 
         # Use a queue so we process matches in the order that they are found
         to_process = deque(initial_matches)
 
         # Keep track of which matches we've already added to the queue
-        processed_matches = set((x, y) for (x, y) in initial_matches)
+        processed_matches = {(x, y) for (x, y) in initial_matches}
 
         # Keep a dict of current matches, which will be updated if better matches are found
-        matched_a = dict()
-        matched_b = dict()
-        for (x, y) in processed_matches:
+        matched_a = {}
+        matched_b = {}
+        for x, y in processed_matches:
             matched_a[x] = y
             matched_b[y] = x
 
@@ -1120,16 +1154,18 @@ class BinDiff(Analysis):
             func_b_pred = self.cfg_b.kb.callgraph.predecessors(func_b) if func_b in callgraph_b_nodes else []
 
             # get possible new matches
-            new_matches = set(self._get_function_matches(self.attributes_a, self.attributes_b,
-                                                         func_a_succ, func_b_succ))
-            new_matches |= set(self._get_function_matches(self.attributes_a, self.attributes_b,
-                                                          func_a_pred, func_b_pred))
+            new_matches = set(
+                self._get_function_matches(self.attributes_a, self.attributes_b, func_a_succ, func_b_succ)
+            )
+            new_matches |= set(
+                self._get_function_matches(self.attributes_a, self.attributes_b, func_a_pred, func_b_pred)
+            )
 
             # could also find matches as function calls of matched basic blocks
             new_matches.update(self._get_call_site_matches(func_a, func_b))
 
             # for each of the possible new matches add it if it improves the matching
-            for (x, y) in new_matches:
+            for x, y in new_matches:
                 # skip none functions and syscalls
                 func_a = self.cfg_a.kb.functions.function(x)
                 if func_a is None or func_a.is_simprocedure or func_a.is_syscall:
@@ -1158,17 +1194,17 @@ class BinDiff(Analysis):
 
         # reformat matches into a set of pairs
         self.function_matches = set()
-        for x,y in matched_a.items():
+        for x, y in matched_a.items():
             # only keep if the pair is in the binary ranges
             if self.project.loader.main_object.contains_addr(x) and self._p2.loader.main_object.contains_addr(y):
                 self.function_matches.add((x, y))
 
         # get the unmatched functions
-        self._unmatched_functions_from_a = set(x for x in self.attributes_a.keys() if x not in matched_a)
-        self._unmatched_functions_from_b = set(x for x in self.attributes_b.keys() if x not in matched_b)
+        self._unmatched_functions_from_a = {x for x in self.attributes_a.keys() if x not in matched_a}
+        self._unmatched_functions_from_b = {x for x in self.attributes_b.keys() if x not in matched_b}
 
         # remove unneeded function diffs
-        for (x, y) in dict(self._function_diffs):
+        for x, y in dict(self._function_diffs):
             if (x, y) not in self.function_matches:
                 del self._function_diffs[(x, y)]
 
@@ -1209,5 +1245,7 @@ class BinDiff(Analysis):
 
         return matches
 
+
 from angr.analyses import AnalysesHub
-AnalysesHub.register_default('BinDiff', BinDiff)
+
+AnalysesHub.register_default("BinDiff", BinDiff)
