@@ -1,4 +1,3 @@
-
 # analyses
 from .cfg_fast import CFGFast
 from .cfg_emulated import CFGEmulated
@@ -8,5 +7,5 @@ from .cfg_fast_soot import CFGFastSoot
 
 # things to make your life easier
 from .cfg_arch_options import CFGArchOptions
-from .cfg_utils import CFGUtils
+from .cfg_base import CFGBase
 from ...knowledge_plugins.cfg import CFGNode
