@@ -1,9 +1,13 @@
 import logging
-import cffi
+from typing import Any, Callable, Optional, Set
+
 import cle
+from cle.backends.externs import KernelObject, ExternObject
+from cle.backends.tls.elf_tls import ELFTLSObject
 
 from sortedcontainers import SortedDict
 
+from ...knowledge_plugins.cfg.memory_data import MemoryDataSort, MemoryData
 from ..analysis import Analysis
 
 _l = logging.getLogger(name=__name__)
@@ -13,11 +17,11 @@ class CFBlanketView:
     """
     A view into the control-flow blanket.
     """
+
     def __init__(self, cfb):
         self._cfb = cfb
 
     def __getitem__(self, item):
-
         if isinstance(item, slice):
             addr = item.start
             start_addr = self._cfb.floor_addr(addr)
@@ -46,6 +50,9 @@ class MemoryRegion:
         self.object = object_
         self.cle_region = cle_region
 
+    def __repr__(self):
+        return f"<MemoryRegion {self.addr:#x}-{self.addr+self.size:#x}, type {self.type}>"
+
 
 #
 # An address can be mapped to one of the following types of object
@@ -70,55 +77,98 @@ class Unknown:
             raise Exception("You cannot create an unknown region of size 0.")
 
     def __repr__(self):
-        s = "<Unknown %#x-%#x>" % (self.addr, self.addr + self.size)
+        s = f"<Unknown {self.addr:#x}-{self.addr + self.size:#x}>"
         return s
 
 
 class CFBlanket(Analysis):
     """
     A Control-Flow Blanket is a representation for storing all instructions, data entries, and bytes of a full program.
+
+    Region types:
+    - section
+    - segment
+    - extern
+    - tls
+    - kernel
     """
-    def __init__(self, cfg=None):
+
+    def __init__(
+        self,
+        exclude_region_types: Optional[Set[str]] = None,
+        on_object_added: Optional[Callable[[int, Any], None]] = None,
+    ):
+        """
+        :param on_object_added: Callable with parameters (addr, obj) called after an object is added to the blanket.
+        """
         self._blanket = SortedDict()
 
-        self._regions = [ ]
+        self._on_object_added_callback = on_object_added
+        self._regions = []
+        self._exclude_region_types = set() if not exclude_region_types else exclude_region_types
 
         self._init_regions()
 
-        if cfg is not None:
-            self._from_cfg(cfg)
-        else:
-            _l.debug("CFG is not specified. Initialize CFBlanket from the knowledge base.")
-            for func in self.kb.functions.values():
-                self.add_function(func)
+        # initialize
+        for func in self.kb.functions.values():
+            self.add_function(func)
+        self._mark_memory_data()
+        self._mark_unknowns()
 
     def _init_regions(self):
-
         for obj in self.project.loader.all_objects:
             if isinstance(obj, cle.MetaELF):
                 if obj.sections:
-                    # Enumerate sections in an ELF file
-                    for section in obj.sections:
-                        if section.occupies_memory:
-                            mr = MemoryRegion(section.vaddr, section.memsize, 'TODO', obj, section)
-                            self._regions.append(mr)
+                    if "section" not in self._exclude_region_types:
+                        # Enumerate sections in an ELF file
+                        for section in obj.sections:
+                            if section.occupies_memory:
+                                mr = MemoryRegion(section.vaddr, section.memsize, "section", obj, section)
+                                self._regions.append(mr)
+                elif obj.segments:
+                    if "segment" not in self._exclude_region_types:
+                        for segment in obj.segments:
+                            if segment.memsize > 0:
+                                mr = MemoryRegion(segment.vaddr, segment.memsize, "segment", obj, segment)
+                                self._regions.append(mr)
                 else:
-                    raise NotImplementedError("Currently ELFs without sections are not supported. Please implement or "
-                                              "complain on GitHub.")
+                    raise NotImplementedError(
+                        "Currently ELFs without sections or segments are not supported. Please "
+                        "implement or complain on GitHub."
+                    )
             elif isinstance(obj, cle.PE):
                 if obj.sections:
-                    for section in obj.sections:
-                        mr = MemoryRegion(section.vaddr, section.memsize, 'TODO', obj, section)
-                        self._regions.append(mr)
+                    if "section" not in self._exclude_region_types:
+                        for section in obj.sections:
+                            mr = MemoryRegion(section.vaddr, section.memsize, "section", obj, section)
+                            self._regions.append(mr)
                 else:
-                    raise NotImplementedError("Currently PEs without sections are not supported. Please report to "
-                                              "GitHub and provide an example binary.")
+                    raise NotImplementedError(
+                        "Currently PEs without sections are not supported. Please report to "
+                        "GitHub and provide an example binary."
+                    )
+            elif isinstance(obj, KernelObject):
+                if "kernel" not in self._exclude_region_types:
+                    size = obj.max_addr - obj.min_addr
+                    mr = MemoryRegion(obj.min_addr, size, "kernel", obj, None)
+                    self._regions.append(mr)
+            elif isinstance(obj, ExternObject):
+                if "extern" not in self._exclude_region_types:
+                    size = obj.max_addr - obj.min_addr
+                    mr = MemoryRegion(obj.min_addr, size, "extern", obj, None)
+                    self._regions.append(mr)
+            elif isinstance(obj, ELFTLSObject):
+                if "tls" not in self._exclude_region_types:
+                    size = obj.max_addr - obj.min_addr
+                    mr = MemoryRegion(obj.min_addr, size, "tls", obj, None)
+                    self._regions.append(mr)
             else:
                 if hasattr(obj, "size"):
                     size = obj.size
                 else:
                     size = obj.max_addr - obj.min_addr
-                mr = MemoryRegion(obj.min_addr, size, 'TODO', obj, None)
+                type_ = "TODO"
+                mr = MemoryRegion(obj.min_addr, size, type_, obj, obj)
                 self._regions.append(mr)
 
         # Sort them just in case
@@ -184,6 +234,8 @@ class CFBlanket(Analysis):
         Adds an object `obj` to the blanket at the specified address `addr`
         """
         self._blanket[addr] = obj
+        if self._on_object_added_callback:
+            self._on_object_added_callback(addr, obj)
 
     def add_function(self, func):
         """
@@ -200,7 +252,7 @@ class CFBlanket(Analysis):
         :rtype:     str
         """
 
-        output = [ ]
+        output = []
 
         for obj in self.project.loader.all_objects:
             for section in obj.sections:
@@ -208,16 +260,18 @@ class CFBlanket(Analysis):
                     continue
                 min_addr, max_addr = section.min_addr, section.max_addr
                 output.append("### Object %s" % repr(section))
-                output.append("### Range %#x-%#x" % (min_addr, max_addr))
+                output.append(f"### Range {min_addr:#x}-{max_addr:#x}")
 
                 pos = min_addr
                 while pos < max_addr:
                     try:
                         addr, thing = self.floor_item(pos)
-                        output.append("%#x: %s" % (addr, repr(thing)))
+                        output.append(f"{addr:#x}: {repr(thing)}")
 
-                        if thing.size == 0: pos += 1
-                        else: pos += thing.size
+                        if thing.size == 0:
+                            pos += 1
+                        else:
+                            pos += thing.size
                     except KeyError:
                         pos += 1
 
@@ -225,19 +279,22 @@ class CFBlanket(Analysis):
 
         return "\n".join(output)
 
-    def _from_cfg(self, cfg):
+    def _mark_memory_data(self):
         """
-        Initialize CFBlanket from a CFG instance.
+        Mark all memory data.
 
-        :param cfg: A CFG instance.
-        :return:    None
+        :return: None
         """
+        if "CFGFast" not in self.kb.cfgs:
+            return
+        cfg_model = self.kb.cfgs["CFGFast"]
 
-        # Let's first add all functions first
-        for func in cfg.kb.functions.values():
-            self.add_function(func)
-
-        self._mark_unknowns()
+        for addr, memory_data in cfg_model.memory_data.items():
+            memory_data: MemoryData
+            if memory_data.sort == MemoryDataSort.CodeReference:
+                # skip Code Reference
+                continue
+            self.add_obj(addr, memory_data)
 
     def _mark_unknowns(self):
         """
@@ -249,13 +306,13 @@ class CFBlanket(Analysis):
         for obj in self.project.loader.all_objects:
             if isinstance(obj, cle.ELF):
                 # sections?
-                if obj.sections:
+                if obj.sections and "section" not in self._exclude_region_types:
                     for section in obj.sections:
                         if not section.memsize or not section.vaddr:
                             continue
                         min_addr, max_addr = section.min_addr, section.max_addr
                         self._mark_unknowns_core(min_addr, max_addr + 1, obj=obj, section=section)
-                elif obj.segments:
+                elif obj.segments and "segment" not in self._exclude_region_types:
                     for segment in obj.segments:
                         if not segment.memsize:
                             continue
@@ -274,12 +331,25 @@ class CFBlanket(Analysis):
                 else:
                     # is it empty?
                     _l.warning("Empty PE object %s.", repr(obj))
+            elif isinstance(obj, ELFTLSObject):
+                if "tls" in self._exclude_region_types:
+                    # Skip them for now
+                    pass
+                else:
+                    min_addr, max_addr = obj.min_addr, obj.max_addr
+                    self._mark_unknowns_core(min_addr, max_addr + 1, obj=obj)
+            elif isinstance(obj, KernelObject):
+                if "kernel" in self._exclude_region_types:
+                    # skip
+                    pass
+                else:
+                    min_addr, max_addr = obj.min_addr, obj.max_addr
+                    self._mark_unknowns_core(min_addr, max_addr + 1, obj=obj)
             else:
                 min_addr, max_addr = obj.min_addr, obj.max_addr
                 self._mark_unknowns_core(min_addr, max_addr + 1, obj=obj)
 
     def _mark_unknowns_core(self, min_addr, max_addr, obj=None, segment=None, section=None):
-
         # The region should be [min_addr, max_addr)
 
         try:
@@ -300,24 +370,29 @@ class CFBlanket(Analysis):
                 bytes_ = None
             else:
                 try:
-                    _l.debug("Loading bytes from object %s, section %s, segmeng %s, addresss %#x.",
-                             obj, section, segment, min_addr)
+                    _l.debug(
+                        "Loading bytes from object %s, section %s, segment %s, address %#x.",
+                        obj,
+                        section,
+                        segment,
+                        min_addr,
+                    )
                     bytes_ = self.project.loader.memory.load(min_addr, size)
                 except KeyError:
                     # The address does not exist
                     bytes_ = None
-            self.add_obj(min_addr,
-                         Unknown(min_addr, size, bytes_=bytes_, object_=obj, segment=segment, section=section)
-                         )
+            self.add_obj(
+                min_addr, Unknown(min_addr, size, bytes_=bytes_, object_=obj, segment=segment, section=section)
+            )
 
         addr = min_addr
         while addr < max_addr:
             last_addr, last_item = self.floor_item(addr)
             if last_addr < min_addr:
                 # impossible
-                raise Exception('Impossible')
+                raise Exception("Impossible")
 
-            if last_item.size == 0:
+            if last_item.size == 0 or last_item.size is None:
                 # Make sure everything has a non-zero size
                 last_item_size = 1
             else:
@@ -335,20 +410,26 @@ class CFBlanket(Analysis):
                         bytes_ = None
                     else:
                         try:
-                            _l.debug("Loading bytes from object %s, section %s, segmeng %s, addresss %#x.",
-                                     obj, section, segment, next_addr)
-                            bytes_ = self.project.loader.memory.load(next_addr, size)
+                            _l.debug(
+                                "Loading bytes from object %s, section %s, segment %s, address %#x.",
+                                obj,
+                                section,
+                                segment,
+                                end_addr,
+                            )
+                            bytes_ = self.project.loader.memory.load(end_addr, size)
                         except KeyError:
                             # The address does not exist
                             bytes_ = None
-                    self.add_obj(end_addr,
-                                 Unknown(end_addr, size, bytes_=bytes_, object_=obj, segment=segment, section=section)
-                                 )
+                    self.add_obj(
+                        end_addr, Unknown(end_addr, size, bytes_=bytes_, object_=obj, segment=segment, section=section)
+                    )
                 addr = next_addr
             else:
                 addr = max_addr
 
 
 from angr.analyses import AnalysesHub
-AnalysesHub.register_default('CFB', CFBlanket)
-AnalysesHub.register_default('CFBlanket', CFBlanket)
+
+AnalysesHub.register_default("CFB", CFBlanket)
+AnalysesHub.register_default("CFBlanket", CFBlanket)
