@@ -1,5 +1,7 @@
 import logging
 
+from capstone.x86_const import X86_OP_MEM
+
 from ....simos import SimWindows
 from .resolver import IndirectJumpResolver
 
@@ -7,8 +9,12 @@ l = logging.getLogger(name=__name__)
 
 
 class X86PeIatResolver(IndirectJumpResolver):
+    """
+    A timeless indirect jump resolver for IAT in x86 PEs.
+    """
+
     def __init__(self, project):
-        super(X86PeIatResolver, self).__init__(project, timeless=True)
+        super().__init__(project, timeless=True)
 
     def filter(self, cfg, addr, func_addr, block, jumpkind):
         if not isinstance(self.project.simos, SimWindows):
@@ -16,13 +22,21 @@ class X86PeIatResolver(IndirectJumpResolver):
         if jumpkind != "Ijk_Call":
             return False
 
-        opnd = self.project.factory.block(addr).capstone.insns[-1].insn.operands[0]
+        insns = self.project.factory.block(addr).capstone.insns
+        if not insns:
+            return False
+        if not insns[-1].insn.operands:
+            return False
+
+        opnd = insns[-1].insn.operands[0]
         # Must be of the form: call ds:0xABCD
-        if opnd.mem and opnd.mem.disp and not opnd.mem.base and not opnd.mem.index:
+        if opnd.type == X86_OP_MEM and opnd.mem.disp and not opnd.mem.base and not opnd.mem.index:
             return True
         return False
 
-    def resolve(self, cfg, addr, func_addr, block, jumpkind):
+    def resolve(
+        self, cfg, addr, func_addr, block, jumpkind, func_graph_complete: bool = True, **kwargs
+    ):  # pylint:disable=unused-argument
         slot = self.project.factory.block(addr).capstone.insns[-1].insn.disp
         target = cfg._fast_memory_load_pointer(slot)
         if target is None:
