@@ -1,11 +1,13 @@
 from .region_identifier import RegionIdentifier
-from .structurer import Structurer
-from .structured_codegen import StructuredCodeGenerator
+from .structured_codegen import CStructuredCodeGenerator, ImportSourceCode
 from .clinic import Clinic
-from .region_simplifier import RegionSimplifier
+from .region_simplifiers import RegionSimplifier
 from .decompiler import Decompiler
 from .decompilation_options import options, options_by_category
 from .block_simplifier import BlockSimplifier
 from .callsite_maker import CallSiteMaker
 from .ail_simplifier import AILSimplifier
+from . import structuring
 from . import optimization_passes
+
+StructuredCodeGenerator = CStructuredCodeGenerator
