@@ -6,11 +6,10 @@ from .base import PeepholeOptimizationExprBase
 class ASubASubN(PeepholeOptimizationExprBase):
     __slots__ = ()
 
-    name = "expr - (expr - N) => N"
-    expr_classes = (BinaryOp, )  # all expressions are allowed
-
-    def optimize(self, expr: BinaryOp):
+    NAME = "expr - (expr - N) => N"
+    expr_classes = (BinaryOp,)  # all expressions are allowed
 
+    def optimize(self, expr: BinaryOp, **kwargs):
         # Sub(A, Sub(A, N)) ==> N
         if expr.op == "Sub" and isinstance(expr.operands[1], BinaryOp) and expr.operands[1].op == "Sub":
             if expr.operands[0] == expr.operands[1].operands[0]:
