@@ -1,4 +1,3 @@
-
 import random
 
 from ..func import Func, TestData
@@ -14,7 +13,7 @@ class memset(Func):
     non_null = [chr(i) for i in range(1, 256)]
 
     def __init__(self):
-        super(memset, self).__init__() #pylint disable=useless-super-delegation
+        super().__init__()  # pylint disable=useless-super-delegation
 
     def get_name(self):
         return "memset"
@@ -22,7 +21,7 @@ class memset(Func):
     def num_args(self):
         return 3
 
-    def args(self): #pylint disable=no-self-use
+    def args(self):  # pylint disable=no-self-use
         return ["buf", "char", "size"]
 
     def can_call_other_funcs(self):
@@ -32,9 +31,9 @@ class memset(Func):
         # TODO we don't check the return val
         set_len = random.randint(1, 40)
         char = random.randint(0, 255)
-        result_buf = rand_str(set_len+5)
+        result_buf = rand_str(set_len + 5)
         test_input = [result_buf, char, set_len]
-        test_output = [chr(char)*set_len + result_buf[-5:], None, None]
+        test_output = [chr(char) * set_len + result_buf[-5:], None, None]
         max_steps = 20
         return_val = None
         test = TestData(test_input, test_output, return_val, max_steps)
