@@ -5,7 +5,8 @@ from . import Analysis
 
 l = logging.getLogger(name=__name__)
 
-class Loop(object):
+
+class Loop:
     def __init__(self, entry, entry_edges, break_edges, continue_edges, body_nodes, graph, subloops):
         self.entry = entry
         self.entry_edges = entry_edges
@@ -19,7 +20,7 @@ class Loop(object):
 
         if not self.has_calls:
             for _, _, data in self.graph.edges(data=True):
-                if 'type' in data and data['type'] == 'fake_return':
+                if "type" in data and data["type"] == "fake_return":
                     # this is a function call.
                     self.has_calls = True
                     break
@@ -28,6 +29,7 @@ class Loop(object):
         s = "<Loop @ %s, %d blocks>" % (self.entry.addr, len(self.body_nodes))
         return s
 
+
 class LoopFinder(Analysis):
     """
     Extracts all the loops from all the functions in a binary.
@@ -41,9 +43,7 @@ class LoopFinder(Analysis):
         self.loops = []
         self.loops_hierarchy = {}
         for function in functions:
-
-            if self.project.is_hooked(function.addr) or \
-                    self.project.simos.is_syscall_addr(function.addr):
+            if self.project.is_hooked(function.addr) or self.project.simos.is_syscall_addr(function.addr):
                 # skip SimProcedures and syscalls
                 continue
 
@@ -58,7 +58,7 @@ class LoopFinder(Analysis):
         if not found_any:
             l.error("No knowledge of functions is present. Did you forget to construct a CFG?")
 
-    def _parse_loop_graph(self, subg, bigg):
+    def _parse_loop_graph(self, subg: networkx.DiGraph, bigg: networkx.DiGraph):
         """
         Create a Loop object for a strongly connected graph, and any strongly
         connected subgraphs, if possible.
@@ -137,20 +137,15 @@ class LoopFinder(Analysis):
                     else:
                         subg.add_edge(subloop, exit_edge[1])
                         removed_exits[exit_edge] = subloop
-                _subgraphs = (networkx.induced_subgraph(subg, nodes).copy() for nodes in
-                             networkx.weakly_connected_components(subg))
-                subg = next(filter( lambda g: entry_node in g.nodes(),
-                                    _subgraphs))
-        me = Loop(entry_node,
-             entry_edges,
-             break_edges,
-             continue_edges,
-             loop_body_nodes,
-             subg,
-             tops[:])
+                _subgraphs = (
+                    networkx.induced_subgraph(subg, nodes).copy()
+                    for nodes in networkx.weakly_connected_components(subg)
+                )
+                subg = next(filter(lambda g: entry_node in g.nodes(), _subgraphs))
+        me = Loop(entry_node, entry_edges, break_edges, continue_edges, loop_body_nodes, subg, tops[:])
         return me, [me] + alls
 
-    def _parse_loops_from_graph(self, graph):
+    def _parse_loops_from_graph(self, graph: networkx.DiGraph):
         """
         Return all Loop instances that can be extracted from a graph.
 
@@ -160,7 +155,10 @@ class LoopFinder(Analysis):
         """
         outtop = []
         outall = []
-        for subg in ( networkx.induced_subgraph(graph, nodes).copy() for nodes in networkx.strongly_connected_components(graph)):
+        subg: networkx.DiGraph
+        for subg in (
+            networkx.induced_subgraph(graph, nodes).copy() for nodes in networkx.strongly_connected_components(graph)
+        ):
             if len(subg.nodes()) == 1:
                 if len(list(subg.successors(list(subg.nodes())[0]))) == 0:
                     continue
@@ -170,5 +168,7 @@ class LoopFinder(Analysis):
                 outtop.append(thisloop)
         return outtop, outall
 
+
 from angr.analyses import AnalysesHub
-AnalysesHub.register_default('LoopFinder', LoopFinder)
+
+AnalysesHub.register_default("LoopFinder", LoopFinder)
