@@ -1,10 +1,8 @@
-
 # pylint: disable=unused-argument
 
 
 class Top:
-
-    __slots__ = ('size', )
+    __slots__ = ("size",)
 
     def __init__(self, size):
         self.size = size
