@@ -1,9 +1,8 @@
-
 # VEX Variables
 
-class VEXVariable:
 
-    __slots__ = tuple()
+class VEXVariable:
+    __slots__ = ()
 
     def __hash__(self):
         raise NotImplementedError()
@@ -12,9 +11,31 @@ class VEXVariable:
         raise NotImplementedError()
 
 
-class VEXReg(VEXVariable):
+class VEXMemVar:
+    __slots__ = (
+        "addr",
+        "size",
+    )
 
-    __slots__ = ('offset', 'size', )
+    def __init__(self, addr, size):
+        self.addr = addr
+        self.size = size
+
+    def __hash__(self):
+        return hash((VEXMemVar, self.addr, self.size))
+
+    def __eq__(self, other):
+        return type(other) is VEXMemVar and other.addr == self.addr and other.size == self.size
+
+    def __repr__(self):
+        return "<mem %#x[%d bytes]>" % (self.addr, self.size)
+
+
+class VEXReg(VEXVariable):
+    __slots__ = (
+        "offset",
+        "size",
+    )
 
     def __init__(self, offset, size):
         self.offset = offset
@@ -31,8 +52,7 @@ class VEXReg(VEXVariable):
 
 
 class VEXTmp(VEXVariable):
-
-    __slots__ = ('tmp', )
+    __slots__ = ("tmp",)
 
     def __init__(self, tmp):
         self.tmp = tmp
