@@ -9,7 +9,7 @@ class SimConcretizationStrategy:
     memory index concretization behavior can be modified.
     """
 
-    def __init__(self, filter=None, exact=True): #pylint:disable=redefined-builtin
+    def __init__(self, filter=None, exact=True):  # pylint:disable=redefined-builtin
         """
         Initializes the base SimConcretizationStrategy.
 
@@ -25,19 +25,19 @@ class SimConcretizationStrategy:
         """
         Gets the minimum solution of an address.
         """
-        return memory.state.solver.min(addr, exact=kwargs.pop('exact', self._exact), **kwargs)
+        return memory.state.solver.min(addr, exact=kwargs.pop("exact", self._exact), **kwargs)
 
     def _max(self, memory, addr, **kwargs):
         """
         Gets the maximum solution of an address.
         """
-        return memory.state.solver.max(addr, exact=kwargs.pop('exact', self._exact), **kwargs)
+        return memory.state.solver.max(addr, exact=kwargs.pop("exact", self._exact), **kwargs)
 
     def _any(self, memory, addr, **kwargs):
         """
         Gets any solution of an address.
         """
-        return memory.state.solver.eval(addr, exact=kwargs.pop('exact', self._exact), **kwargs)
+        return memory.state.solver.eval(addr, exact=kwargs.pop("exact", self._exact), **kwargs)
 
     def _eval(self, memory, addr, n, **kwargs):
         """
@@ -45,7 +45,7 @@ class SimConcretizationStrategy:
         """
         if isinstance(addr, claripy.vsa.StridedInterval):
             return addr.eval(n)
-        return memory.state.solver.eval_upto(addr, n, exact=kwargs.pop('exact', self._exact), **kwargs)
+        return memory.state.solver.eval_upto(addr, n, exact=kwargs.pop("exact", self._exact), **kwargs)
 
     def _range(self, memory, addr, **kwargs):
         """
@@ -53,17 +53,18 @@ class SimConcretizationStrategy:
         """
         return (self._min(memory, addr, **kwargs), self._max(memory, addr, **kwargs))
 
-    def concretize(self, memory, addr):
+    def concretize(self, memory, addr, **kwargs):
         """
         Concretizes the address into a list of values.
         If this strategy cannot handle this address, returns None.
         """
         if self._filter is None or self._filter(memory, addr):
-            return self._concretize(memory, addr)
+            return self._concretize(memory, addr, **kwargs)
 
-    def _concretize(self, memory, addr):
+    def _concretize(self, memory, addr, **kwargs):
         """
         Should be implemented by child classes to handle concretization.
+        :param **kwargs:
         """
         raise NotImplementedError()
 
@@ -81,6 +82,7 @@ class SimConcretizationStrategy:
         """
         pass
 
+
 from .any import SimConcretizationStrategyAny
 from .controlled_data import SimConcretizationStrategyControlledData
 from .eval import SimConcretizationStrategyEval
