@@ -1,5 +1,6 @@
 from . import SimConcretizationStrategy
 
+
 class SimConcretizationStrategySolutions(SimConcretizationStrategy):
     """
     Concretization strategy that resolves an address into some
@@ -7,10 +8,11 @@ class SimConcretizationStrategySolutions(SimConcretizationStrategy):
     """
 
     def __init__(self, limit, **kwargs):
-        super(SimConcretizationStrategySolutions, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self._limit = limit
 
-    def _concretize(self, memory, addr):
-        addrs = self._eval(memory, addr, self._limit + 1)
+    def _concretize(self, memory, addr, **kwargs):
+        addrs = self._eval(memory, addr, self._limit + 1, **kwargs)
         if len(addrs) <= self._limit:
             return addrs
+        return None
