@@ -1,5 +1,5 @@
 from .successors import SimSuccessors
-from .engine import SimEngine, SuccessorsMixin
+from .engine import SimEngine, SuccessorsMixin, TLSMixin
 
 from .vex import HeavyVEXMixin, TrackActionsMixin, SimInspectMixin, HeavyResilienceMixin, SuperFastpathMixin
 from .procedure import ProcedureMixin, ProcedureEngine
@@ -14,16 +14,17 @@ from .soot import SootMixin
 # The default execution engine
 # You may remove unused mixins from this default engine to speed up execution
 class UberEngine(
-        SimEngineFailure,
-        SimEngineSyscall,
-        HooksMixin,
-        SimEngineUnicorn,
-        SuperFastpathMixin,
-        TrackActionsMixin,
-        SimInspectMixin,
-        HeavyResilienceMixin,
-        SootMixin,
-        HeavyVEXMixin
+    SimEngineFailure,
+    SimEngineSyscall,
+    HooksMixin,
+    SimEngineUnicorn,
+    SuperFastpathMixin,
+    TrackActionsMixin,
+    SimInspectMixin,
+    HeavyResilienceMixin,
+    SootMixin,
+    HeavyVEXMixin,
+    TLSMixin,
 ):
     pass
 
@@ -31,7 +32,10 @@ class UberEngine(
 try:
     from .pcode import HeavyPcodeMixin
 
-    class UberEnginePcode(SimEngineFailure, SimEngineSyscall, HooksMixin, HeavyPcodeMixin):  # pylint:disable=abstract-method
+    class UberEnginePcode(
+        SimEngineFailure, SimEngineSyscall, HooksMixin, HeavyPcodeMixin
+    ):  # pylint:disable=abstract-method
         pass
+
 except ImportError:
     pass
