@@ -1,4 +1,3 @@
-
 import operator
 
 from .base import SimSootExpr
@@ -24,5 +23,5 @@ class SimSootExpr_Binop(SimSootExpr):
         "xor": operator.xor,
         "rem": operator.imod,
         "cmpl": operator.lt,
-        "cmp": operator.ne
+        "cmp": operator.ne,
     }
