@@ -1,11 +1,12 @@
-
 import logging
 
-l = logging.getLogger('angr.engines.soot.statements')
+l = logging.getLogger("angr.engines.soot.statements")
+
 
 def translate_stmt(stmt, state):
     stmt_name = stmt.__class__.__name__.split(".")[-1]
-    if stmt_name.endswith("Stmt"): stmt_name = stmt_name[:-4]
+    if stmt_name.endswith("Stmt"):
+        stmt_name = stmt_name[:-4]
 
     stmt_cls_name = "SimSootStmt_%s" % stmt_name
     if stmt_cls_name in globals():
@@ -18,6 +19,7 @@ def translate_stmt(stmt, state):
         l.warning("Unsupported Soot statement %s.", stmt_cls_name)
         return None
 
+
 from .assign import SimSootStmt_Assign
 from .return_ import SimSootStmt_Return, SimSootStmt_ReturnVoid
 from .identity import SimSootStmt_Identity
