@@ -1,9 +1,8 @@
-
 import logging
 
 from .base import SimSootStmt
 
-l = logging.getLogger('angr.engines.soot.statements.identity')
+l = logging.getLogger("angr.engines.soot.statements.identity")
 
 
 class SimSootStmt_Identity(SimSootStmt):
