@@ -1,9 +1,8 @@
-
 import logging
 
 from .base import SimSootStmt
 
-l = logging.getLogger('angr.engines.soot.statements.invoke')
+l = logging.getLogger("angr.engines.soot.statements.invoke")
 
 
 class SimSootStmt_Invoke(SimSootStmt):
