@@ -1,10 +1,9 @@
-
 from .base import SimSootStmt
 
 
 class SimSootStmt_Return(SimSootStmt):
     def __init__(self, stmt, state):
-        super(SimSootStmt_Return, self).__init__(stmt, state)
+        super().__init__(stmt, state)
         self.return_value = None
 
     def _execute(self):
@@ -13,7 +12,7 @@ class SimSootStmt_Return(SimSootStmt):
 
 class SimSootStmt_ReturnVoid(SimSootStmt):
     def __init__(self, stmt, state):
-        super(SimSootStmt_ReturnVoid, self).__init__(stmt, state)
+        super().__init__(stmt, state)
         self.return_value = None
 
     def _execute(self):
