@@ -1,11 +1,9 @@
-
 import logging
 
-from claripy import Or
-
 from .base import SimSootStmt
 
-l = logging.getLogger('angr.engines.soot.statements.switch')
+l = logging.getLogger("angr.engines.soot.statements.switch")
+
 
 class SwitchBase(SimSootStmt):
     def _execute(self):
@@ -21,7 +19,7 @@ class SwitchBase(SimSootStmt):
         # add all targets, conditioned by the key value
         for lookup_value, target in self.stmt.lookup_values_and_targets.items():
             jmp_target = self._get_bb_addr_from_instr(target)
-            jmp_condition = (lookup_value == key_val)
+            jmp_condition = lookup_value == key_val
             self._add_jmp_target(jmp_target, jmp_condition)
             # add condition for the default target
             default_jmp_conditions += [(lookup_value != key_val)]
@@ -31,8 +29,10 @@ class SwitchBase(SimSootStmt):
         default_jmp_cond = self.state.solver.And(*default_jmp_conditions)
         self._add_jmp_target(default_jmp_target, default_jmp_cond)
 
+
 class SimSootStmt_TableSwitch(SwitchBase):
     pass
 
+
 class SimSootStmt_LookupSwitch(SwitchBase):
     pass
