@@ -1,7 +1,7 @@
-
 def translate_value(value, state):
     value_name = value.__class__.__name__
-    if value_name.startswith("Soot"): value_name = value_name[4:]
+    if value_name.startswith("Soot"):
+        value_name = value_name[4:]
     value_cls_name = "SimSootValue_" + value_name
 
     g = globals()
@@ -13,6 +13,7 @@ def translate_value(value, state):
     value_ = value_cls.from_sootvalue(value, state)
     return value_
 
+
 from .local import SimSootValue_Local
 from .paramref import SimSootValue_ParamRef
 from .arrayref import SimSootValue_ArrayRef, SimSootValue_ArrayBaseRef
