@@ -1,7 +1,11 @@
-import claripy
-import pyvex
+from typing import Optional
 import logging
 
+import claripy
+import pyvex
+
+from . import irop
+from . import ccall
 from ..light import VEXMixin
 from .... import errors
 from .... import sim_options as o
@@ -9,23 +13,31 @@ from .... import sim_options as o
 l = logging.getLogger(__name__)
 zero = claripy.BVV(0, 32)
 
-def value(ty, val):
-    if ty == 'Ity_F32':
+
+def value(ty, val, size: Optional[int] = None):
+    if ty == "Ity_F32":
         return claripy.FPV(float(val), claripy.FSORT_FLOAT)
-    elif ty == 'Ity_F64':
+    elif ty == "Ity_F64":
         return claripy.FPV(float(val), claripy.FSORT_DOUBLE)
     else:
+        if size is not None:
+            return claripy.BVV(int(val), size)
         return claripy.BVV(int(val), pyvex.get_type_size(ty))
 
+
 def symbol(ty, name):
-    if ty == 'Ity_F32':
+    if ty == "Ity_F32":
         return claripy.FPS(name, claripy.FSORT_FLOAT)
-    elif ty == 'Ity_F64':
+    elif ty == "Ity_F64":
         return claripy.FPS(name, claripy.FSORT_DOUBLE)
     else:
         return claripy.BVS(name, pyvex.get_type_size(ty))
 
+
 class ClaripyDataMixin(VEXMixin):
+    """
+    This mixin provides methods that makes the vex engine process guest code using claripy ASTs as the data domain.
+    """
 
     # util methods
 
@@ -38,8 +50,12 @@ class ClaripyDataMixin(VEXMixin):
     def _optimize_guarded_addr(self, addr, guard):
         # optimization: is the guard the same as the condition inside the address? if so, unpack the address and remove
         # the guarding condition.
-        if isinstance(guard, claripy.ast.Base) and guard.op == 'If' \
-                and isinstance(addr, claripy.ast.Base) and addr.op == 'If':
+        if (
+            isinstance(guard, claripy.ast.Base)
+            and guard.op == "If"
+            and isinstance(addr, claripy.ast.Base)
+            and addr.op == "If"
+        ):
             if guard.args[0] is addr.args[0]:
                 # the address is guarded by the same guard! unpack the addr
                 return addr.args[1]
@@ -74,14 +90,14 @@ class ClaripyDataMixin(VEXMixin):
 
     def _perform_vex_expr_Get(self, offset, ty, **kwargs):
         res = super()._perform_vex_expr_Get(offset, ty, **kwargs)
-        if ty.startswith('Ity_F'):
+        if ty.startswith("Ity_F"):
             return res.raw_to_fp()
         else:
             return res
 
     def _perform_vex_expr_Load(self, addr, ty, endness, **kwargs):
         res = super()._perform_vex_expr_Load(addr, ty, endness, **kwargs)
-        if ty.startswith('Ity_F'):
+        if ty.startswith("Ity_F"):
             return res.raw_to_fp()
         else:
             return res
@@ -95,12 +111,17 @@ class ClaripyDataMixin(VEXMixin):
     # op support
 
     def _perform_vex_expr_ITE(self, cond, ifTrue, ifFalse):
-        return claripy.If(cond != 0, ifTrue, ifFalse)
+        try:
+            return claripy.If(cond != 0, ifTrue, ifFalse)
+        except claripy.ClaripyError as e:
+            raise errors.SimError("Claripy failed") from e
 
     def _perform_vex_expr_Op(self, op, args):
-        # TODO: get rid of these hacks (i.e. state options and modes) and move these switches into the engine initializer
-        options = getattr(self.state, 'options', {o.SUPPORT_FLOATING_POINT})
-        simop = irop.vexop_to_simop(op, extended=o.EXTENDED_IROP_SUPPORT in options, fp=o.SUPPORT_FLOATING_POINT in options)
+        # TODO: get rid of these hacks (i.e. state options and modes) and move these switches into engine properties
+        options = getattr(self.state, "options", {o.SUPPORT_FLOATING_POINT})
+        simop = irop.vexop_to_simop(
+            op, extended=o.EXTENDED_IROP_SUPPORT in options, fp=o.SUPPORT_FLOATING_POINT in options
+        )
         return simop.calculate(*args)
 
     # ccall support
@@ -110,13 +131,20 @@ class ClaripyDataMixin(VEXMixin):
             try:
                 func = getattr(ccall, func_name)
             except AttributeError as e:
-                raise errors.UnsupportedCCallError("Unsupported ccall %s" % func_name) from e
+                raise errors.UnsupportedCCallError(f"Unsupported ccall {func_name}") from e
 
         try:
             return func(self.state, *args)
         except ccall.CCallMultivaluedException as e:
-            cases = e.args[0]
-            return claripy.ite_cases([(case, func(self.state, value, *args[1:])) for case, value in cases], value(ty, 0))
-
-from . import irop
-from . import ccall
+            cases, to_replace = e.args
+            # pylint: disable=undefined-loop-variable
+            for i, arg in enumerate(args):
+                if arg is to_replace:
+                    break
+            else:
+                raise errors.UnsupportedCCallError("Trying to concretize a value which is not an argument")
+            evaluated_cases = [(case, func(self.state, *args[:i], value_, *args[i + 1 :])) for case, value_ in cases]
+            try:
+                return claripy.ite_cases(evaluated_cases, value(ty, 0))
+            except claripy.ClaripyError as ce:
+                raise errors.SimOperationError("Claripy failed") from ce
