@@ -1,6 +1,7 @@
 from . import ExplorationTechnique
 import psutil
 
+
 class MemoryWatcher(ExplorationTechnique):
     """Memory Watcher
 
@@ -14,11 +15,11 @@ class MemoryWatcher(ExplorationTechnique):
     off states to effectively stop execution if we're below a given threshold.
     """
 
-    def __init__(self, min_memory=512, memory_stash='lowmem'):
-        super(MemoryWatcher, self).__init__()
+    def __init__(self, min_memory=512, memory_stash="lowmem"):
+        super().__init__()
 
         if min_memory is not None:
-            self.min_memory = 1024*1024*min_memory
+            self.min_memory = 1024 * 1024 * min_memory
 
         else:
             self.min_memory = int(psutil.virtual_memory().total * 0.05)
@@ -29,10 +30,9 @@ class MemoryWatcher(ExplorationTechnique):
         if self.memory_stash not in simgr.stashes:
             simgr.stashes[self.memory_stash] = []
 
-    def step(self, simgr, stash='active', **kwargs):
-
+    def step(self, simgr, stash="active", **kwargs):
         if psutil.virtual_memory().available <= self.min_memory:
-            simgr.move(from_stash='active', to_stash=self.memory_stash)
+            simgr.move(from_stash="active", to_stash=self.memory_stash)
 
         else:
             simgr = simgr.step(stash=stash, **kwargs)
