@@ -1,6 +1,6 @@
 import logging
 import weakref
-from typing import Union, TYPE_CHECKING
+from typing import Union, Optional, Tuple, TYPE_CHECKING
 
 from sortedcontainers import SortedDict
 
@@ -12,13 +12,12 @@ l = logging.getLogger(name=__name__)
 
 
 class StoredObject:
-
-    __slots__ = ('__weakref__', 'start', 'obj', 'size')
+    __slots__ = ("__weakref__", "start", "obj", "size")
 
     def __init__(self, start, obj, size):
         self.start = start
         self.obj = obj
-        self.size: Union['UnknownSize',int] = size
+        self.size: Union["UnknownSize", int] = size
 
     def __eq__(self, other):
         assert type(other) is StoredObject
@@ -29,7 +28,7 @@ class StoredObject:
         return hash((self.start, self.size, self.obj))
 
     def __repr__(self):
-        return "<SO %s@%#x, %s bytes>" % (repr(self.obj), self.start, self.size)
+        return f"<SO {repr(self.obj)}@{self.start:#x}, {self.size} bytes>"
 
     @property
     def obj_id(self):
@@ -41,7 +40,7 @@ class RegionObject:
     Represents one or more objects occupying one or more bytes in KeyedRegion.
     """
 
-    __slots__ = ('start', 'size', 'stored_objects', '_internal_objects')
+    __slots__ = ("start", "size", "stored_objects", "_internal_objects")
 
     def __init__(self, start, size, objects=None):
         self.start = start
@@ -54,8 +53,12 @@ class RegionObject:
                 self._internal_objects.add(obj.obj)
 
     def __eq__(self, other):
-        return type(other) is RegionObject and self.start == other.start and self.size == other.size and \
-               self.stored_objects == other.stored_objects
+        return (
+            type(other) is RegionObject
+            and self.start == other.start
+            and self.size == other.size
+            and self.stored_objects == other.stored_objects
+        )
 
     def __ne__(self, other):
         return not self == other
@@ -110,7 +113,12 @@ class KeyedRegion:
     Registers and function frames can all be viewed as a keyed region.
     """
 
-    __slots__ = ('_storage', '_object_mapping', '_phi_node_contains', '_canonical_size', )
+    __slots__ = (
+        "_storage",
+        "_object_mapping",
+        "_phi_node_contains",
+        "_canonical_size",
+    )
 
     def __init__(self, tree=None, phi_node_contains=None, canonical_size=8):
         self._storage = SortedDict() if tree is None else tree
@@ -125,7 +133,7 @@ class KeyedRegion:
         self._storage, om, self._phi_node_contains = s
         self._object_mapping = weakref.WeakValueDictionary(om)
 
-    def _get_container(self, offset):
+    def _get_container(self, offset) -> Tuple[int, Optional[RegionObject]]:
         try:
             base_offset = next(self._storage.irange(maximum=offset, reverse=True))
         except StopIteration:
@@ -187,8 +195,11 @@ class KeyedRegion:
             raise ValueError("The canonical sizes of two KeyedRegion objects must equal.")
 
         # TODO: is the current solution not optimal enough?
-        for _, item in other._storage.items():  # type: RegionObject
-            for so in item.stored_objects:  # type: StoredObject
+        _: RegionObject
+        item: RegionObject
+        for _, item in other._storage.items():
+            so: StoredObject
+            for so in item.stored_objects:
                 if replacements and so.obj in replacements:
                     so = StoredObject(so.start, replacements[so.obj], so.size)
                 self._object_mapping[so.obj_id] = so
@@ -205,8 +216,11 @@ class KeyedRegion:
         :return:        self
         """
 
-        for _, item in other._storage.items():  # type: RegionObject
-            for so in item.stored_objects:  # type: StoredObject
+        _: RegionObject
+        item: RegionObject
+        for _, item in other._storage.items():
+            so: StoredObject
+            for so in item.stored_objects:
                 if replacements and so.obj in replacements:
                     so = StoredObject(so.start, replacements[so.obj], so.size)
                 self._object_mapping[so.obj_id] = so
@@ -226,7 +240,7 @@ class KeyedRegion:
             old_var_id = id(old_var)
             if old_var_id in self._object_mapping:
                 # FIXME: we need to check if old_var still exists in the storage
-                old_so = self._object_mapping[old_var_id]  # type: StoredObject
+                old_so: StoredObject = self._object_mapping[old_var_id]
                 self._store(old_so.start, new_var, old_so.size, overwrite=True)
 
         return self
@@ -237,16 +251,16 @@ class KeyedRegion:
         :return: A string of debugging output.
         """
         keys = self._storage.keys()
-        offset_to_vars = { }
+        offset_to_vars = {}
 
         for key in sorted(keys):
             ro = self._storage[key]
-            variables = [ obj.obj for obj in ro.stored_objects ]
+            variables = [obj.obj for obj in ro.stored_objects]
             offset_to_vars[key] = variables
 
-        s = [ ]
+        s = []
         for offset, variables in offset_to_vars.items():
-            s.append("Offset %#x: %s" % (offset, variables))
+            s.append(f"Offset {offset:#x}: {variables}")
         return "\n".join(s)
 
     def add_variable(self, start, variable):
@@ -361,10 +375,11 @@ class KeyedRegion:
     # Private methods
     #
 
-    def _canonicalize_size(self, size: Union[int,'UnknownSize']) -> int:
-
+    def _canonicalize_size(self, size: Union[int, "UnknownSize"]) -> int:
         # delayed import
-        from .knowledge_plugins.key_definitions.unknown_size import UnknownSize  # pylint:disable=import-outside-toplevel
+        from .knowledge_plugins.key_definitions.unknown_size import (
+            UnknownSize,
+        )  # pylint:disable=import-outside-toplevel
 
         if isinstance(size, UnknownSize):
             return self._canonical_size
@@ -400,11 +415,27 @@ class KeyedRegion:
         end: int = start + object_size
 
         # region items in the middle
-        overlapping_items = list(self._storage.irange(start, end-1))
+        overlapping_items = list(self._storage.irange(start, end - 1))
 
         # is there a region item that begins before the start and overlaps with this variable?
         floor_key, floor_item = self._get_container(start)
-        if floor_item is not None and floor_key not in overlapping_items:
+        if floor_item is None:
+            # fast path: just insert it
+            self._storage[start] = RegionObject(start, object_size, {stored_object})
+            return
+
+        # fast path: if there is a perfect overlap, just update the item
+        if len(overlapping_items) == 1 and floor_item.start == start and floor_item.end == end:
+            if overwrite:
+                floor_item.set_object(stored_object)
+            elif merge_to_top is False and top is None:
+                floor_item.add_object(stored_object)
+            else:
+                self._add_object_with_check(floor_item, stored_object, merge_to_top=merge_to_top, top=top)
+            return
+
+        # slower path: there are multiple overlapping items
+        if floor_key not in overlapping_items:
             # insert it into the beginning
             overlapping_items.insert(0, floor_key)
 
@@ -451,12 +482,11 @@ class KeyedRegion:
         self._storage.update(to_update)
 
     def _is_overlapping(self, start, variable):
-
         if variable.size is not None:
             # make sure this variable does not overlap with any other variable
             end = start + variable.size
             try:
-                prev_offset = next(self._storage.irange(maximum=end-1, reverse=True))
+                prev_offset = next(self._storage.irange(maximum=end - 1, reverse=True))
             except StopIteration:
                 prev_offset = None
 
