@@ -1,6 +1,6 @@
-
 from .functions import FunctionManager, Function
 from .variables import VariableManager
+from .debug_variables import DebugVariableManager
 from .comments import Comments
 from .data import Data
 from .indirect_jumps import IndirectJumps
@@ -8,7 +8,11 @@ from .labels import Labels
 from .cfg import CFGManager
 from .xrefs import XRefManager
 from .plugin import KnowledgeBasePlugin
-from .sync import SynchronizationManager
+from .sync import SyncController
 from .patches import PatchManager
 from .key_definitions import KeyDefinitionManager
-from .propagations import Propagations
+from .propagations import PropagationManager
+from .structured_code import StructuredCodeManager
+from .types import TypesStore
+from .callsite_prototypes import CallsitePrototypes
+from .custom_strings import CustomStrings
