@@ -1,16 +1,22 @@
-from typing import Optional
+# pylint:disable=raise-missing-from
+from typing import Dict, Generator, Optional, Set
 import logging
 import collections.abc
+import re
 from sortedcontainers import SortedDict
+
 import networkx
 
+from archinfo.arch_soot import SootMethodDescriptor
+
 from ...errors import SimEngineError
 from ..plugin import KnowledgeBasePlugin
-
 from .function import Function
 from .soot_function import SootFunction
 
-from archinfo.arch_soot import SootMethodDescriptor
+
+QUERY_PATTERN = re.compile(r"^(::(.+?))?::(.+)$")
+ADDR_PATTERN = re.compile(r"^(0x[\dA-Fa-f]+)|(\d+)$")
 
 l = logging.getLogger(name=__name__)
 
@@ -20,17 +26,18 @@ class FunctionDict(SortedDict):
     FunctionDict is a dict where the keys are function starting addresses and
     map to the associated :class:`Function`.
     """
+
     def __init__(self, backref, *args, **kwargs):
         self._backref = backref
-        self._key_types = kwargs.pop('key_types', int)
-        super(FunctionDict, self).__init__(*args, **kwargs)
+        self._key_types = kwargs.pop("key_types", int)
+        super().__init__(*args, **kwargs)
 
     def __getitem__(self, addr):
         try:
-            return super(FunctionDict, self).__getitem__(addr)
-        except KeyError:
+            return super().__getitem__(addr)
+        except KeyError as ex:
             if not isinstance(addr, self._key_types):
-                raise TypeError("FunctionDict only supports %s as key type" % self._key_types)
+                raise TypeError("FunctionDict only supports %s as key type" % self._key_types) from ex
 
             if isinstance(addr, SootMethodDescriptor):
                 t = SootFunction(self._backref, addr)
@@ -38,13 +45,13 @@ class FunctionDict(SortedDict):
                 t = Function(self._backref, addr)
             try:
                 self[addr] = t
-            except:
+            except Exception:  # pylint:disable=broad-except
                 pass
             self._backref._function_added(t)
             return t
 
     def get(self, addr):
-        return super(FunctionDict, self).__getitem__(addr)
+        return super().__getitem__(addr)
 
     def floor_addr(self, addr):
         try:
@@ -58,24 +65,56 @@ class FunctionDict(SortedDict):
         except StopIteration:
             raise KeyError(addr)
 
+    def __setstate__(self, state):
+        for v, k in state.items():
+            self[k] = v
+
+    def __getstate__(self):
+        return {v: k for (v, k) in self.items()}
+
 
 class FunctionManager(KnowledgeBasePlugin, collections.abc.Mapping):
     """
     This is a function boundaries management tool. It takes in intermediate
     results during CFG generation, and manages a function map of the binary.
     """
+
     def __init__(self, kb):
-        super(FunctionManager, self).__init__()
+        super().__init__()
         self._kb = kb
         self.function_address_types = self._kb._project.arch.function_address_types
         self.address_types = self._kb._project.arch.address_types
-        self._function_map = FunctionDict(self, key_types=self.function_address_types)
+        self._function_map: Dict[int, Function] = FunctionDict(self, key_types=self.function_address_types)
+        self.function_addrs_set: Set = set()
         self.callgraph = networkx.MultiDiGraph()
         self.block_map = {}
 
         # Registers used for passing arguments around
         self._arg_registers = kb._project.arch.argument_registers
 
+    def __setstate__(self, state):
+        self._kb = state["_kb"]
+        self.function_address_types = state["function_address_types"]
+        self.address_types = state["address_types"]
+        self._function_map = state["_function_map"]
+        self.callgraph = state["callgraph"]
+        self.block_map = state["block_map"]
+
+        self._function_map._backref = self
+        for func in self._function_map.values():
+            func._function_manager = self
+
+    def __getstate__(self):
+        s = {
+            "_kb": self._kb,
+            "function_address_types": self.function_address_types,
+            "address_types": self.address_types,
+            "_function_map": self._function_map,
+            "callgraph": self.callgraph,
+            "block_map": self.block_map,
+        }
+        return s
+
     def copy(self):
         fm = FunctionManager(self._kb)
         fm._function_map = self._function_map.copy()
@@ -83,13 +122,15 @@ class FunctionManager(KnowledgeBasePlugin, collections.abc.Mapping):
             fm._function_map[address] = function.copy()
         fm.callgraph = networkx.MultiDiGraph(self.callgraph)
         fm._arg_registers = self._arg_registers.copy()
+        fm.function_addrs_set = self.function_addrs_set.copy()
 
         return fm
 
     def clear(self):
-        self._function_map.clear()
+        self._function_map = FunctionDict(self, key_types=self.function_address_types)
         self.callgraph = networkx.MultiDiGraph()
         self.block_map.clear()
+        self.function_addrs_set = set()
 
     def _genenare_callmap_sif(self, filepath):
         """
@@ -100,7 +141,7 @@ class FunctionManager(KnowledgeBasePlugin, collections.abc.Mapping):
         """
         with open(filepath, "wb") as f:
             for src, dst in self.callgraph.edges():
-                f.write("%#x\tDirectEdge\t%#x\n" % (src, dst))
+                f.write(f"{src:#x}\tDirectEdge\t{dst:#x}\n")
 
     def _add_node(self, function_addr, node, syscall=None, size=None):
         if isinstance(node, self.address_types):
@@ -111,8 +152,17 @@ class FunctionManager(KnowledgeBasePlugin, collections.abc.Mapping):
         dst_func._register_nodes(True, node)
         self.block_map[node.addr] = node
 
-    def _add_call_to(self, function_addr, from_node, to_addr, retn_node=None, syscall=None, stmt_idx=None, ins_addr=None,
-                     return_to_outside=False):
+    def _add_call_to(
+        self,
+        function_addr,
+        from_node,
+        to_addr,
+        retn_node=None,
+        syscall=None,
+        stmt_idx=None,
+        ins_addr=None,
+        return_to_outside=False,
+    ):
         """
         Add a call to a function.
 
@@ -139,22 +189,30 @@ class FunctionManager(KnowledgeBasePlugin, collections.abc.Mapping):
             dest_func = self._function_map[to_addr]
             if syscall in (True, False):
                 dest_func.is_syscall = syscall
-            func._call_to(from_node, dest_func, retn_node, stmt_idx=stmt_idx, ins_addr=ins_addr,
-                          return_to_outside=return_to_outside
-                          )
+            func._call_to(
+                from_node,
+                dest_func,
+                retn_node,
+                stmt_idx=stmt_idx,
+                ins_addr=ins_addr,
+                return_to_outside=return_to_outside,
+            )
 
         if return_to_outside:
             func.add_retout_site(from_node)
 
         # is there any existing edge on the callgraph?
-        edge_data = {'type': 'call'}
-        if function_addr not in self.callgraph or \
-                to_addr not in self.callgraph[function_addr] or \
-                edge_data not in self.callgraph[function_addr][to_addr].values():
+        edge_data = {"type": "call"}
+        if to_addr is not None and (
+            function_addr not in self.callgraph
+            or to_addr not in self.callgraph[function_addr]
+            or edge_data not in self.callgraph[function_addr][to_addr].values()
+        ):
             self.callgraph.add_edge(function_addr, to_addr, **edge_data)
 
-    def _add_fakeret_to(self, function_addr, from_node, to_node, confirmed=None, syscall=None, to_outside=False,
-                        to_function_addr=None):
+    def _add_fakeret_to(
+        self, function_addr, from_node, to_node, confirmed=None, syscall=None, to_outside=False, to_function_addr=None
+    ):
         if isinstance(from_node, self.address_types):
             from_node = self._kb._project.factory.snippet(from_node)
         if isinstance(to_node, self.address_types):
@@ -168,10 +226,12 @@ class FunctionManager(KnowledgeBasePlugin, collections.abc.Mapping):
 
         if to_outside and to_function_addr is not None:
             # mark it on the callgraph
-            edge_data = {'type': 'fakeret'}
-            if function_addr not in self.callgraph or \
-                    to_function_addr not in self.callgraph[function_addr] or \
-                    edge_data not in self.callgraph[function_addr][to_function_addr].values():
+            edge_data = {"type": "fakeret"}
+            if (
+                function_addr not in self.callgraph
+                or to_function_addr not in self.callgraph[function_addr]
+                or edge_data not in self.callgraph[function_addr][to_function_addr].values()
+            ):
                 self.callgraph.add_edge(function_addr, to_function_addr, **edge_data)
 
     def _remove_fakeret(self, function_addr, from_node, to_node):
@@ -181,7 +241,7 @@ class FunctionManager(KnowledgeBasePlugin, collections.abc.Mapping):
             to_node = self._kb._project.factory.snippet(to_node)
         self._function_map[function_addr]._remove_fakeret(from_node, to_node)
 
-    def _add_return_from(self, function_addr, from_node, to_node=None): #pylint:disable=unused-argument
+    def _add_return_from(self, function_addr, from_node, to_node=None):  # pylint:disable=unused-argument
         if isinstance(from_node, self.address_types):  # pylint: disable=unidiomatic-typecheck
             from_node = self._kb._project.factory.snippet(from_node)
         self._function_map[function_addr]._add_return_site(from_node)
@@ -191,11 +251,13 @@ class FunctionManager(KnowledgeBasePlugin, collections.abc.Mapping):
             from_node = self._kb._project.factory.snippet(from_node)
         if isinstance(to_node, self.address_types):  # pylint: disable=unidiomatic-typecheck
             to_node = self._kb._project.factory.snippet(to_node)
-        self._function_map[function_addr]._transit_to(from_node, to_node, ins_addr=ins_addr, stmt_idx=stmt_idx,
-                                                      is_exception=is_exception)
+        self._function_map[function_addr]._transit_to(
+            from_node, to_node, ins_addr=ins_addr, stmt_idx=stmt_idx, is_exception=is_exception
+        )
 
-    def _add_outside_transition_to(self, function_addr, from_node, to_node, to_function_addr=None, ins_addr=None,
-                                   stmt_idx=None, is_exception=False):
+    def _add_outside_transition_to(
+        self, function_addr, from_node, to_node, to_function_addr=None, ins_addr=None, stmt_idx=None, is_exception=False
+    ):
         if type(from_node) is int:  # pylint: disable=unidiomatic-typecheck
             from_node = self._kb._project.factory.snippet(from_node)
         if type(to_node) is int:  # pylint: disable=unidiomatic-typecheck
@@ -205,20 +267,26 @@ class FunctionManager(KnowledgeBasePlugin, collections.abc.Mapping):
                 # we cannot get the snippet, but we should at least tell the function that it's going to jump out here
                 self._function_map[function_addr].add_jumpout_site(from_node)
                 return
-        self._function_map[function_addr]._transit_to(from_node, to_node, outside=True, ins_addr=ins_addr,
-                                                      stmt_idx=stmt_idx, is_exception=is_exception,
-                                                      )
+        self._function_map[function_addr]._transit_to(
+            from_node,
+            to_node,
+            outside=True,
+            ins_addr=ins_addr,
+            stmt_idx=stmt_idx,
+            is_exception=is_exception,
+        )
 
         if to_function_addr is not None:
             # mark it on the callgraph
-            edge_data = {'type': 'transition' if not is_exception else 'exception'}
-            if function_addr not in self.callgraph or \
-                    to_function_addr not in self.callgraph[function_addr] or \
-                    edge_data not in self.callgraph[function_addr][to_function_addr].values():
+            edge_data = {"type": "transition" if not is_exception else "exception"}
+            if (
+                function_addr not in self.callgraph
+                or to_function_addr not in self.callgraph[function_addr]
+                or edge_data not in self.callgraph[function_addr][to_function_addr].values()
+            ):
                 self.callgraph.add_edge(function_addr, to_function_addr, **edge_data)
 
     def _add_return_from_call(self, function_addr, src_function_addr, to_node, to_outside=False):
-
         # Note that you will never return to a syscall
 
         if type(to_node) is int:  # pylint: disable=unidiomatic-typecheck
@@ -232,7 +300,6 @@ class FunctionManager(KnowledgeBasePlugin, collections.abc.Mapping):
     #
 
     def __contains__(self, item):
-
         if type(item) is int:
             # this is an address
             return item in self._function_map
@@ -268,19 +335,27 @@ class FunctionManager(KnowledgeBasePlugin, collections.abc.Mapping):
             del self._function_map[k]
             if k in self.callgraph:
                 self.callgraph.remove_node(k)
+            self.function_addrs_set.discard(k)
         else:
-            raise ValueError("FunctionManager.__delitem__ only accepts int as key")
+            raise ValueError(
+                f"FunctionManager.__delitem__ only accepts the following address types: "
+                f"{self.function_address_types}"
+            )
 
     def __len__(self):
         return len(self._function_map)
 
     def __iter__(self):
-        for i in sorted(self._function_map.keys()):
-            yield i
+        yield from sorted(self._function_map.keys())
 
     def get_by_addr(self, addr) -> Function:
         return self._function_map.get(addr)
 
+    def get_by_name(self, name: str) -> Generator[Function, None, None]:
+        for f in self._function_map.values():
+            if f.name == name:
+                yield f
+
     def _function_added(self, func: Function):
         """
         A callback method for adding a new function instance to the manager.
@@ -289,6 +364,9 @@ class FunctionManager(KnowledgeBasePlugin, collections.abc.Mapping):
         :return:       None
         """
 
+        # Add the function address to the set of function addresses
+        self.function_addrs_set.add(func.addr)
+
         # make sure all functions exist in the call graph
         self.callgraph.add_node(func.addr)
 
@@ -334,6 +412,37 @@ class FunctionManager(KnowledgeBasePlugin, collections.abc.Mapping):
         except KeyError:
             return None
 
+    def query(self, query: str) -> Optional[Function]:
+        """
+        Query for a function using selectors to disambiguate. Supported variations:
+
+            ::<name>           Function <name> in the main object
+            ::<addr>::<name>   Function <name> at <addr>
+            ::<obj>::<name>    Function <name> in <obj>
+
+        """
+        # FIXME: Proper mangle handling
+        matches = QUERY_PATTERN.match(query)
+        if matches:
+            selector = matches.group(2)
+            name = matches.group(3)
+
+            if selector is not None and ADDR_PATTERN.fullmatch(selector):
+                addr = int(matches.group(2), 0)
+                try:
+                    func = self._function_map.get(addr)
+                    if func.name == name:
+                        return func
+                except KeyError:
+                    pass
+
+            obj_name = selector or self._kb._project.loader.main_object.binary_basename
+            for func in self.get_by_name(name):
+                if func.binary_name == obj_name:
+                    return func
+
+        return None
+
     def function(self, addr=None, name=None, create=False, syscall=False, plt=None) -> Optional[Function]:
         """
         Get a function object from the function manager.
@@ -361,19 +470,33 @@ class FunctionManager(KnowledgeBasePlugin, collections.abc.Mapping):
                     if name is not None:
                         f.name = name
                     if syscall:
-                        f.is_syscall=True
+                        f.is_syscall = True
                     return f
         elif name is not None:
-            for func in self._function_map.values():
-                if func.name == name:
-                    if plt is None or func.is_plt == plt:
-                        return func
+            func = self.query(name)
+            if func is not None:
+                return func
+
+            for func in self.get_by_name(name):
+                if plt is None or func.is_plt == plt:
+                    return func
 
         return None
 
-    def dbg_draw(self, prefix='dbg_function_'):
+    def dbg_draw(self, prefix="dbg_function_"):
         for func_addr, func in self._function_map.items():
-            filename = "%s%#08x.png" % (prefix, func_addr)
+            filename = f"{prefix}{func_addr:#08x}.png"
             func.dbg_draw(filename)
 
-KnowledgeBasePlugin.register_default('functions', FunctionManager)
+    def rebuild_callgraph(self):
+        self.callgraph = networkx.MultiDiGraph()
+        for func_addr in self._function_map.keys():
+            self.callgraph.add_node(func_addr)
+        for func in self._function_map.values():
+            if func.block_addrs_set:
+                for node in func.transition_graph.nodes():
+                    if isinstance(node, Function):
+                        self.callgraph.add_edge(func.addr, node.addr)
+
+
+KnowledgeBasePlugin.register_default("functions", FunctionManager)
