@@ -4,9 +4,8 @@ from .plugin import KnowledgeBasePlugin
 
 
 class IndirectJumps(KnowledgeBasePlugin, dict):
-
     def __init__(self, kb):
-        super(IndirectJumps, self).__init__()
+        super().__init__()
         self._kb = kb
         self.unresolved = set()
 
@@ -31,4 +30,4 @@ class IndirectJumps(KnowledgeBasePlugin, dict):
             self.resolved[indirect_address] = list(resolved_addresses)
 
 
-KnowledgeBasePlugin.register_default('indirect_jumps', IndirectJumps)
+KnowledgeBasePlugin.register_default("indirect_jumps", IndirectJumps)
