@@ -7,11 +7,13 @@ class HeapAddress:
     """
     The representation of an address on the heap.
     """
-    def __init__(self, value: Union[int,Undefined]):
+
+    def __init__(self, value: Union[int, Undefined]):
         self._value = value
 
     @property
-    def value(self): return self._value
+    def value(self):
+        return self._value
 
     def __repr__(self):
         address_as_string = ("%#x" % self._value) if isinstance(self._value, int) else ("%s" % self._value)
