@@ -1,5 +1,9 @@
+import warnings
+
+
 once_set = set()
 
+
 def once(key):
     if key in once_set:
         return False
@@ -7,6 +11,7 @@ def once(key):
         once_set.add(key)
         return True
 
+
 already_complained = set()
 
 
@@ -15,10 +20,12 @@ def deprecated(replacement=None):
         def inner(*args, **kwargs):
             if func not in already_complained:
                 if replacement is None:
-                    print("\x1b[31;1mDeprecation warning: Don't use %s\x1b[0m" % (func.__name__))
+                    warnings.warn("Don't use %s" % (func.__name__), DeprecationWarning)
                 else:
-                    print("\x1b[31;1mDeprecation warning: Use %s instead of %s\x1b[0m" % (replacement, func.__name__))
+                    warnings.warn(f"Use {replacement} instead of {func.__name__}", DeprecationWarning)
                 already_complained.add(func)
             return func(*args, **kwargs)
+
         return inner
+
     return outer
