@@ -8,8 +8,9 @@ from typing import Optional, Dict, Type, TYPE_CHECKING
 
 
 from ...sim_type import parse_cpp_file, SimTypeFunction
-from ...calling_conventions import SimCC, DEFAULT_CC
+from ...calling_conventions import DEFAULT_CC
 from ...misc import autoimport
+from ...misc.ux import once
 from ...sim_type import parse_file
 from ..stubs.ReturnUnconstrained import ReturnUnconstrained
 from ..stubs.syscall_stub import syscall as stub_syscall
@@ -19,7 +20,7 @@ if TYPE_CHECKING:
 
 
 l = logging.getLogger(name=__name__)
-SIM_LIBRARIES: Dict[str,'SimLibrary'] = {}
+SIM_LIBRARIES: Dict[str, "SimLibrary"] = {}
 
 
 class SimLibrary:
@@ -34,10 +35,11 @@ class SimLibrary:
     :ivar fallback_proc:    A SimProcedure class that should be used to provide stub procedures. By default,
                             ``ReturnUnconstrained``.
     """
+
     def __init__(self):
         self.procedures = {}
         self.non_returning = set()
-        self.prototypes: Dict[str,SimTypeFunction] = {}
+        self.prototypes: Dict[str, SimTypeFunction] = {}
         self.default_ccs = {}
         self.names = []
         self.fallback_cc = dict(DEFAULT_CC)
@@ -73,7 +75,7 @@ class SimLibrary:
         """
         The first common name of this library, e.g. libc.so.6, or '??????' if none are known.
         """
-        return self.names[0] if self.names else '??????'
+        return self.names[0] if self.names else "??????"
 
     def set_library_names(self, *names):
         """
@@ -133,7 +135,7 @@ class SimLibrary:
         parsed = parse_file(c_decl)
         parsed_decl = parsed[0]
         if not parsed_decl:
-            raise ValueError('Cannot parse the function prototype.')
+            raise ValueError("Cannot parse the function prototype.")
         func_name, func_proto = next(iter(parsed_decl.items()))
 
         self.set_prototype(func_name, func_proto)
@@ -173,22 +175,24 @@ class SimLibrary:
             new_procedure = copy.deepcopy(old_procedure)
             new_procedure.display_name = alt
             self.procedures[alt] = new_procedure
+            if name in self.prototypes:
+                self.prototypes[alt] = self.prototypes[name]
+            if name in self.non_returning:
+                self.non_returning.add(alt)
 
     def _apply_metadata(self, proc, arch):
         if proc.cc is None and arch.name in self.default_ccs:
             proc.cc = self.default_ccs[arch.name](arch)
-            if proc.cc.func_ty is not None:
-                # Use inspect to extract the parameters from the run python function
-                proc.cc.func_ty.arg_names = inspect.getfullargspec(proc.run).args[1:]
         if proc.cc is None and arch.name in self.fallback_cc:
-            proc.cc = self.fallback_cc[arch.name](arch)
+            proc.cc = self.fallback_cc[arch.name]["Linux"](arch)
         if proc.display_name in self.prototypes:
-            proc.cc.func_ty = self.prototypes[proc.display_name].with_arch(arch)
-            # Use inspect to extract the parameters from the run python function
-            proc.cc.func_ty.arg_names = inspect.getfullargspec(proc.run).args[1:]
+            proc.prototype = self.prototypes[proc.display_name].with_arch(arch)
+            proc.guessed_prototype = False
+            if proc.prototype.arg_names is None:
+                # Use inspect to extract the parameters from the run python function
+                proc.prototype.arg_names = inspect.getfullargspec(proc.run).args[1:]
             if not proc.ARGS_MISMATCH:
-                proc.cc.num_args = len(proc.cc.func_ty.args)
-                proc.num_args = len(proc.cc.func_ty.args)
+                proc.num_args = len(proc.prototype.args)
         if proc.display_name in self.non_returning:
             proc.returns = False
         proc.library_name = self.name
@@ -248,9 +252,7 @@ class SimLibrary:
         :param name:    The name of the function as a string
         :return:        A bool indicating if anything is known about the function
         """
-        return self.has_implementation(name) or \
-            name in self.non_returning or \
-            name in self.prototypes
+        return self.has_implementation(name) or name in self.non_returning or name in self.prototypes
 
     def has_implementation(self, name):
         """
@@ -293,13 +295,13 @@ class SimCppLibrary(SimLibrary):
         return name
 
     @staticmethod
-    def _proto_from_demangled_name(name: str) -> Optional[SimCC]:
+    def _proto_from_demangled_name(name: str) -> Optional[SimTypeFunction]:
         """
         Attempt to extract arguments and calling convention information for a C++ function whose name was mangled
         according to the Itanium C++ ABI symbol mangling language.
 
         :param name:    The demangled function name.
-        :return:        A calling convention or None if a calling convention cannot be found.
+        :return:        A prototype or None if a prototype cannot be found.
         """
 
         try:
@@ -340,10 +342,13 @@ class SimCppLibrary(SimLibrary):
         # try to determine a prototype from the function name if possible
         if demangled_name != name:
             # itanium-mangled function name
-            stub.cc.set_func_type_with_arch(self._proto_from_demangled_name(demangled_name))
-            if stub.cc.func_ty is not None and not stub.ARGS_MISMATCH:
-                stub.cc.num_args = len(stub.cc.func_ty.args)
-                stub.num_args = len(stub.cc.func_ty.args)
+            stub.prototype = self._proto_from_demangled_name(demangled_name)
+            if stub.prototype is not None:
+                stub.prototype = stub.prototype.with_arch(arch)
+                stub.guessed_prototype = False
+                if not stub.ARGS_MISMATCH:
+                    stub.cc.num_args = len(stub.prototype.args)
+                    stub.num_args = len(stub.prototype.args)
         return stub
 
     def get_prototype(self, name: str, arch=None) -> Optional[SimTypeFunction]:
@@ -400,12 +405,13 @@ class SimSyscallLibrary(SimLibrary):
     All the SimLibrary methods for adding functions still work, but now there's an additional layer on top that
     associates them with numbers.
     """
+
     def __init__(self):
         super().__init__()
-        self.syscall_number_mapping: Dict[str,Dict[int,str]] = defaultdict(dict)  # keyed by abi
-        self.syscall_name_mapping: Dict[str,Dict[str,int]] = defaultdict(dict)  # keyed by abi
-        self.default_cc_mapping: Dict[str,Type['SimCCSyscall']] = {}  # keyed by abi
-        self.syscall_prototypes: Dict[str,Dict[str,SimTypeFunction]] = defaultdict(dict)  # keyed by abi
+        self.syscall_number_mapping: Dict[str, Dict[int, str]] = defaultdict(dict)  # keyed by abi
+        self.syscall_name_mapping: Dict[str, Dict[str, int]] = defaultdict(dict)  # keyed by abi
+        self.default_cc_mapping: Dict[str, Type["SimCCSyscall"]] = {}  # keyed by abi
+        self.syscall_prototypes: Dict[str, Dict[str, SimTypeFunction]] = defaultdict(dict)  # keyed by abi
         self.fallback_proc = stub_syscall
 
     def copy(self):
@@ -415,10 +421,10 @@ class SimSyscallLibrary(SimLibrary):
         o.prototypes = dict(self.prototypes)
         o.default_ccs = dict(self.default_ccs)
         o.names = list(self.names)
-        o.syscall_number_mapping = defaultdict(dict, self.syscall_number_mapping) # {abi: {number: name}}
-        o.syscall_name_mapping = defaultdict(dict, self.syscall_name_mapping) # {abi: {name: number}}
-        o.syscall_prototypes = defaultdict(dict, self.syscall_prototypes) # as above
-        o.default_cc_mapping = dict(self.default_cc_mapping) # {abi: cc}
+        o.syscall_number_mapping = defaultdict(dict, self.syscall_number_mapping)  # {abi: {number: name}}
+        o.syscall_name_mapping = defaultdict(dict, self.syscall_name_mapping)  # {abi: {name: number}}
+        o.syscall_prototypes = defaultdict(dict, self.syscall_prototypes)  # as above
+        o.default_cc_mapping = dict(self.default_cc_mapping)  # {abi: cc}
         return o
 
     def update(self, other):
@@ -432,8 +438,7 @@ class SimSyscallLibrary(SimLibrary):
         :param abi: The abi to evaluate
         :return:    The smallest syscall number known for the given abi
         """
-        if abi not in self.syscall_number_mapping or \
-                not self.syscall_number_mapping[abi]:
+        if abi not in self.syscall_number_mapping or not self.syscall_number_mapping[abi]:
             return 0
         return min(self.syscall_number_mapping[abi])
 
@@ -442,8 +447,7 @@ class SimSyscallLibrary(SimLibrary):
         :param abi: The abi to evaluate
         :return:    The largest syscall number known for the given abi
         """
-        if abi not in self.syscall_number_mapping or \
-                not self.syscall_number_mapping[abi]:
+        if abi not in self.syscall_number_mapping or not self.syscall_number_mapping[abi]:
             return 0
         return max(self.syscall_number_mapping[abi])
 
@@ -487,7 +491,7 @@ class SimSyscallLibrary(SimLibrary):
         """
         self.syscall_prototypes[abi][name] = proto
 
-    def set_prototypes(self, abi: str, protos: Dict[str,SimTypeFunction]) -> None:  # pylint: disable=arguments-differ
+    def set_prototypes(self, abi: str, protos: Dict[str, SimTypeFunction]) -> None:  # pylint: disable=arguments-differ
         """
         Set the prototypes of many syscalls.
 
@@ -505,20 +509,19 @@ class SimSyscallLibrary(SimLibrary):
             mapping = self.syscall_number_mapping[abi]
             if number in mapping:
                 return mapping[number], arch, abi
-        return 'sys_%d' % number, arch, None
+        return "sys_%d" % number, arch, None
 
     def _apply_numerical_metadata(self, proc, number, arch, abi):
         proc.syscall_number = number
         proc.abi = abi
         if abi in self.default_cc_mapping:
             cc = self.default_cc_mapping[abi](arch)
-            if proc.cc is not None:
-                cc.set_func_type_with_arch(proc.cc.func_ty)
             proc.cc = cc
         # a bit of a hack.
         name = proc.display_name
-        if self.syscall_prototypes[abi].get(name, None) is not None and proc.cc is not None:
-            proc.cc.func_ty = self.syscall_prototypes[abi][name].with_arch(arch)
+        if self.syscall_prototypes[abi].get(name, None) is not None:
+            proc.guessed_prototype = False
+            proc.prototype = self.syscall_prototypes[abi][name].with_arch(arch)
 
     # pylint: disable=arguments-differ
     def get(self, number, arch, abi_list=()):
@@ -612,5 +615,53 @@ class SimSyscallLibrary(SimLibrary):
         return name in self.syscall_prototypes[abi]
 
 
-for _ in autoimport.auto_import_modules('angr.procedures.definitions', os.path.dirname(os.path.realpath(__file__))):
+#
+# Autoloading
+#
+
+# By default we only load common API definitions (as defined in COMMON_LIBRARIES). For loading more definitions, the
+# following logic is followed:
+# - We will load all Windows APIs them if the loaded binary is a Windows binary, or when load_win32api_definitions() is
+#   called.
+# - We will load all APIs when load_all_definitions() is called.
+
+_DEFINITIONS_BASEDIR = os.path.dirname(os.path.realpath(__file__))
+
+
+def load_win32api_definitions():
+    if once("load_win32api_definitions"):
+        for _ in autoimport.auto_import_modules(
+            "angr.procedures.definitions",
+            _DEFINITIONS_BASEDIR,
+            filter_func=lambda module_name: module_name.startswith("win32_")
+            or module_name in {"ntoskrnl", "ntdll", "user32"},
+        ):
+            pass
+
+
+def load_all_definitions():
+    if once("load_all_definitions"):
+        for _ in autoimport.auto_import_modules("angr.procedures.definitions", _DEFINITIONS_BASEDIR):
+            pass
+
+
+COMMON_LIBRARIES = {
+    # CGC
+    "cgc",
+    # (mostly) Linux
+    "glibc",
+    "gnulib",  # really just for .o files in coreutils
+    "libstdcpp",
+    "linux_kernel",
+    "linux_loader",
+    # Windows
+    "msvcr",
+}
+
+
+for _ in autoimport.auto_import_modules(
+    "angr.procedures.definitions",
+    _DEFINITIONS_BASEDIR,
+    filter_func=lambda module_name: module_name in COMMON_LIBRARIES,
+):
     pass
