@@ -1,4 +1,3 @@
-
 import itertools
 
 from ...sim_procedure import SimProcedure
@@ -7,7 +6,6 @@ from ...engines.soot.expressions import translate_expr
 
 
 class JavaSimProcedure(SimProcedure):
-
     @property
     def is_java(self):
         return True
@@ -20,7 +18,7 @@ class JavaSimProcedure(SimProcedure):
         sim_args = []
         # try to get 'this' reference
         try:
-            this_ref = state.javavm_memory.load(addr=SimSootValue_Local('this', None))
+            this_ref = state.javavm_memory.load(addr=SimSootValue_Local("this", None))
             sim_args += [this_ref]
         except KeyError:
             pass
