@@ -8,13 +8,10 @@ log = logging.getLogger(name=__name__)
 
 
 class ParseDouble(JavaSimProcedure):
-
-    __provides__ = (
-        ('java.lang.Double', 'parseDouble(java.lang.String)'),
-    )
+    __provides__ = (("java.lang.Double", "parseDouble(java.lang.String)"),)
 
     def run(self, str_ref):
-        log.debug('Called SimProcedure java.lang.Double.parseDouble with args: {}'.format(str_ref))
+        log.debug(f"Called SimProcedure java.lang.Double.parseDouble with args: {str_ref}")
         str_ = self.state.memory.load(str_ref)
 
         if str_.concrete:
