@@ -3,8 +3,9 @@ import logging
 
 l = logging.getLogger(name=__name__)
 
+
 class memcpy(angr.SimProcedure):
-    #pylint:disable=arguments-differ
+    # pylint:disable=arguments-differ
 
     def run(self, dst_addr, src_addr, limit):
         if not self.state.solver.symbolic(limit):
@@ -17,19 +18,20 @@ class memcpy(angr.SimProcedure):
             min_limit = self.state.solver.min_int(limit)
             conditional_size = min(max_memcpy_size, max(min_limit, max_limit))
             if max_limit > max_memcpy_size and conditional_size < max_limit:
-                l.warning("memcpy upper bound of %#x outside limit, limiting to %#x instead",
-                          max_limit, conditional_size)
+                l.warning(
+                    "memcpy upper bound of %#x outside limit, limiting to %#x instead", max_limit, conditional_size
+                )
 
         l.debug("Memcpy running with conditional_size %#x", conditional_size)
 
         if conditional_size > 0:
-            src_mem = self.state.memory.load(src_addr, conditional_size, endness='Iend_BE')
+            src_mem = self.state.memory.load(src_addr, conditional_size, endness="Iend_BE")
             if ABSTRACT_MEMORY in self.state.options:
-                self.state.memory.store(dst_addr, src_mem, size=conditional_size, endness='Iend_BE')
+                self.state.memory.store(dst_addr, src_mem, size=conditional_size, endness="Iend_BE")
             else:
-                self.state.memory.store(dst_addr, src_mem, size=limit, endness='Iend_BE')
-
+                self.state.memory.store(dst_addr, src_mem, size=limit, endness="Iend_BE")
 
         return dst_addr
 
+
 from ...sim_options import ABSTRACT_MEMORY
