@@ -1,9 +1,9 @@
 import angr
 
+
 class strtoul(angr.SimProcedure):
-    #pylint:disable=arguments-differ
+    # pylint:disable=arguments-differ
     def run(self, nptr, endptr, base):
-        strtol = angr.SIM_PROCEDURES['libc']['strtol']
+        strtol = angr.SIM_PROCEDURES["libc"]["strtol"]
         result = self.inline_call(strtol, nptr, endptr, base).ret_expr
         return result
-        
\ No newline at end of file
