@@ -1,12 +1,9 @@
 import angr
 import tempfile
 
-######################################
-# tmpnam
-######################################
 
 class tmpnam(angr.SimProcedure):
-    #pylint:disable=arguments-differ
+    # pylint:disable=arguments-differ
 
     def run(self, tmp_file_path_addr):
         L_tmpnam = 20
@@ -15,9 +12,8 @@ class tmpnam(angr.SimProcedure):
             return tmp_file_path_addr
 
         tmp_file_path = tempfile.mktemp()
-        malloc = angr.SIM_PROCEDURES['libc']['malloc']
+        malloc = angr.SIM_PROCEDURES["libc"]["malloc"]
         addr = self.inline_call(malloc, L_tmpnam).ret_expr
-        self.state.memory.store(addr,
-                                tmp_file_path.encode() + b'\x00')
+        self.state.memory.store(addr, tmp_file_path.encode() + b"\x00")
 
         return addr
