@@ -3,10 +3,9 @@ import logging
 
 l = logging.getLogger(name=__name__)
 
+
 class mprotect(angr.SimProcedure):
-
-    def run(self, addr, length, prot): #pylint:disable=arguments-differ,unused-argument
-
+    def run(self, addr, length, prot):  # pylint:disable=arguments-differ,unused-argument
         try:
             addr = self.state.solver.eval_one(addr)
         except angr.errors.SimValueError:
@@ -22,13 +21,13 @@ class mprotect(angr.SimProcedure):
         except angr.errors.SimValueError:
             raise angr.errors.SimValueError("mprotect can't handle symbolic prot")
 
-        l.debug('mprotect(%#x, %#x, %#x) = ...', addr, length, prot)
+        l.debug("mprotect(%#x, %#x, %#x) = ...", addr, length, prot)
 
-        if addr & 0xfff != 0:
-            l.debug('... = -1 (not aligned)')
+        if addr & 0xFFF != 0:
+            l.debug("... = -1 (not aligned)")
             return -1
 
-        page_end = ((addr + length - 1) & ~0xfff) + 0x1000
+        page_end = ((addr + length - 1) & ~0xFFF) + 0x1000
         try:
             for page in range(addr, page_end, 0x1000):
                 self.state.memory.permissions(page)
@@ -38,5 +37,5 @@ class mprotect(angr.SimProcedure):
 
         for page in range(addr, page_end, 0x1000):
             self.state.memory.permissions(page, prot & 7)
-        l.debug('... = 0 (good)')
+        l.debug("... = 0 (good)")
         return 0
