@@ -3,12 +3,13 @@ import logging
 from ..stubs.format_parser import FormatParser
 
 l = logging.getLogger(name=__name__)
-l.setLevel('INFO')
+l.setLevel("INFO")
+
 
 class syslog(FormatParser):
-    def run(self, priority):
-        fmt = self._parse(1)
-        formatted = fmt.replace(2, self.arg)
+    def run(self, priority, fmt):
+        fmt = self._parse(fmt)
+        formatted = fmt.replace(self.va_arg)
         if not formatted.symbolic:
             formatted = self.state.solver.eval(formatted, cast_to=bytes)
-        l.info("Syslog priority %s: %s", priority, formatted)
\ No newline at end of file
+        l.info("Syslog priority %s: %s", priority, formatted)
