@@ -1,9 +1,5 @@
 import angr
 
-######################################
-# Caller
-######################################
-
 
 class Caller(angr.SimProcedure):
     """
@@ -11,7 +7,7 @@ class Caller(angr.SimProcedure):
     """
 
     def run(self, target_addr=None, target_cc=None):
-        self.call(target_addr, [ ], 'after_call', cc=target_cc)
+        self.call(target_addr, [], "after_call", cc=target_cc, prototype="void x()")
 
     def after_call(self, target_addr=None, target_cc=None):
         pass
