@@ -1,5 +1,6 @@
 import angr
 
+
 class GetCurrentThreadId(angr.SimProcedure):
     def run(self):
-        return 0xbad76ead
+        return 0xBAD76EAD
