@@ -1,6 +1,7 @@
 import angr
 import claripy
 
+
 class GetTempPathA(angr.SimProcedure):
     RESULT = claripy.BVV(b"C:\\Temp\\")
 
@@ -10,10 +11,13 @@ class GetTempPathA(angr.SimProcedure):
         except angr.errors.SimValueError:
             raise angr.errors.SimProcedureError("Can't handle symbolic nBufferLength in GetTempPath")
 
-        copy_len = min(self.RESULT.length//8, length - 1)
-        self.state.memory.store(lpBuffer, self.RESULT[self.RESULT.length - 1 : self.RESULT.length - copy_len*8].concat(claripy.BVV(0, 8)))
+        copy_len = min(self.RESULT.length // 8, length - 1)
+        self.state.memory.store(
+            lpBuffer, self.RESULT[self.RESULT.length - 1 : self.RESULT.length - copy_len * 8].concat(claripy.BVV(0, 8))
+        )
         return self.RESULT.length // 8
 
+
 class GetWindowsDirectoryA(angr.SimProcedure):
     RESULT = claripy.BVV(b"C:\\Windows")
 
@@ -23,6 +27,8 @@ class GetWindowsDirectoryA(angr.SimProcedure):
         except angr.errors.SimValueError:
             raise angr.errors.SimProcedureError("Can't handle symbolic uSize in GetWindowsDirectory")
 
-        copy_len = min(self.RESULT.length//8, length - 1)
-        self.state.memory.store(lpBuffer, self.RESULT[self.RESULT.length - 1 : self.RESULT.length - copy_len*8].concat(claripy.BVV(0, 8)))
+        copy_len = min(self.RESULT.length // 8, length - 1)
+        self.state.memory.store(
+            lpBuffer, self.RESULT[self.RESULT.length - 1 : self.RESULT.length - copy_len * 8].concat(claripy.BVV(0, 8))
+        )
         return self.RESULT.length // 8
