@@ -5,18 +5,21 @@ from io import BytesIO, IOBase
 import pickle
 import string
 from collections import defaultdict
-from typing import Dict, Any
+from pathlib import Path
+from typing import Dict, Any, Optional, Union, cast
 
 import archinfo
 from archinfo.arch_soot import SootAddressDescriptor, ArchSoot
 import cle
+from .sim_procedure import SimProcedure
 
 from .misc.ux import deprecated
 from .errors import AngrNoPluginError
 
 l = logging.getLogger(name=__name__)
 
-def load_shellcode(shellcode, arch, start_offset=0, load_address=0, thumb=False, **kwargs):
+
+def load_shellcode(shellcode: Union[bytes, str], arch, start_offset=0, load_address=0, thumb=False, **kwargs):
     """
     Load a new project based on a snippet of assembly or bytecode.
 
@@ -28,21 +31,23 @@ def load_shellcode(shellcode, arch, start_offset=0, load_address=0, thumb=False,
     """
     if not isinstance(arch, archinfo.Arch):
         arch = archinfo.arch_from_id(arch)
-    if type(shellcode) is str:
-        shellcode = arch.asm(shellcode, load_address, thumb=thumb)
+    if isinstance(shellcode, str):
+        shellcode_bytes: bytes = arch.asm(shellcode, load_address, thumb=thumb)
+    else:
+        shellcode_bytes = shellcode
     if thumb:
         start_offset |= 1
 
     return Project(
-            BytesIO(shellcode),
-            main_opts={
-                'backend': 'blob',
-                'arch': arch,
-                'entry_point': start_offset,
-                'base_addr': load_address,
-            },
-        **kwargs
-        )
+        BytesIO(shellcode_bytes),
+        main_opts={
+            "backend": "blob",
+            "arch": arch,
+            "entry_point": start_offset,
+            "base_addr": load_address,
+        },
+        **kwargs,
+    )
 
 
 class Project:
@@ -66,10 +71,9 @@ class Project:
     :param simos:                       a SimOS class to use for this project.
     :param engine:                      The SimEngine class to use for this project.
     :param bool translation_cache:      If True, cache translated basic blocks rather than re-translating them.
-    :param support_selfmodifying_code:  Whether we aggressively support self-modifying code. When enabled, emulation
+    :param selfmodifying_code:          Whether we aggressively support self-modifying code. When enabled, emulation
                                         will try to read code from the current state instead of the original memory,
                                         regardless of the current memory protections.
-    :type support_selfmodifying_code:   bool
     :param store_function:              A function that defines how the Project should be stored. Default to pickling.
     :param load_function:               A function that defines how the Project should be loaded. Default to unpickling.
     :param analyses_preset:             The plugin preset for the analyses provider (i.e. Analyses instance).
@@ -80,7 +84,8 @@ class Project:
     :ivar analyses:     The available analyses.
     :type analyses:     angr.analysis.Analyses
     :ivar entry:        The program entrypoint.
-    :ivar factory:      Provides access to important analysis elements such as path groups and symbolic execution results.
+    :ivar factory:      Provides access to important analysis elements such as path groups and symbolic execution
+                        results.
     :type factory:      AngrObjectFactory
     :ivar filename:     The filename of the executable.
     :ivar loader:       The program loader.
@@ -89,65 +94,78 @@ class Project:
     :type storage:      defaultdict(list)
     """
 
-    def __init__(self, thing,
-                 default_analysis_mode=None,
-                 ignore_functions=None,
-                 use_sim_procedures=True,
-                 exclude_sim_procedures_func=None,
-                 exclude_sim_procedures_list=(),
-                 arch=None, simos=None,
-                 engine=None,
-                 load_options: Dict[str, Any]=None,
-                 translation_cache=True,
-                 support_selfmodifying_code=False,
-                 store_function=None,
-                 load_function=None,
-                 analyses_preset=None,
-                 concrete_target=None,
-                 **kwargs):
+    arch: archinfo.Arch
 
+    def __init__(
+        self,
+        thing,
+        default_analysis_mode=None,
+        ignore_functions=None,
+        use_sim_procedures=True,
+        exclude_sim_procedures_func=None,
+        exclude_sim_procedures_list=(),
+        arch=None,
+        simos=None,
+        engine=None,
+        load_options: Optional[Dict[str, Any]] = None,
+        translation_cache=True,
+        selfmodifying_code: bool = False,
+        support_selfmodifying_code: Optional[bool] = None,  # deprecated. use selfmodifying_code instead
+        store_function=None,
+        load_function=None,
+        analyses_preset=None,
+        concrete_target=None,
+        eager_ifunc_resolution=None,
+        **kwargs,
+    ):
         # Step 1: Load the binary
 
-        if load_options is None: load_options = {}
+        if load_options is None:
+            load_options = {}
 
         load_options.update(kwargs)
         if arch is not None:
-            load_options.update({'arch': arch})
+            load_options.update({"arch": arch})
         if isinstance(thing, cle.Loader):
             if load_options:
                 l.warning("You provided CLE options to angr but you also provided a completed cle.Loader object!")
             self.loader = thing
             self.filename = self.loader.main_object.binary
-        elif hasattr(thing, 'read') and hasattr(thing, 'seek'):
+        elif isinstance(thing, cle.Backend):
+            self.filename = thing.binary
+            self.loader = cle.Loader(thing, **load_options)
+        elif hasattr(thing, "read") and hasattr(thing, "seek"):
             l.info("Loading binary from stream")
             self.filename = None
             self.loader = cle.Loader(thing, **load_options)
-        elif not isinstance(thing, str) or not os.path.exists(thing) or not os.path.isfile(thing):
+        elif not isinstance(thing, (str, Path)) or not os.path.exists(thing) or not os.path.isfile(thing):
             raise Exception("Not a valid binary file: %s" % repr(thing))
         else:
             # use angr's loader, provided by cle
             l.info("Loading binary %s", thing)
-            self.filename = thing
+            self.filename = str(thing)
             self.loader = cle.Loader(self.filename, concrete_target=concrete_target, **load_options)
 
         # Step 2: determine its CPU architecture, ideally falling back to CLE's guess
         if isinstance(arch, str):
             self.arch = archinfo.arch_from_id(arch)  # may raise ArchError, let the user see this
         elif isinstance(arch, archinfo.Arch):
-            self.arch = arch # type: archinfo.Arch
+            self.arch: archinfo.Arch = arch
         elif arch is None:
             self.arch = self.loader.main_object.arch
         else:
             raise ValueError("Invalid arch specification.")
         # Step 3: Set some defaults and set the public and private properties
         if not default_analysis_mode:
-            default_analysis_mode = 'symbolic'
+            default_analysis_mode = "symbolic"
         if not ignore_functions:
             ignore_functions = []
 
         if isinstance(exclude_sim_procedures_func, types.LambdaType):
-            l.warning("Passing a lambda type as the exclude_sim_procedures_func argument to "
-                      "Project causes the resulting object to be un-serializable.")
+            l.warning(
+                "Passing a lambda type as the exclude_sim_procedures_func argument to "
+                "Project causes the resulting object to be un-serializable."
+            )
 
         self._sim_procedures = {}
 
@@ -155,13 +173,14 @@ class Project:
 
         # It doesn't make any sense to have auto_load_libs
         # if you have the concrete target, let's warn the user about this.
-        if self.concrete_target and load_options.get('auto_load_libs', None):
-
-            l.critical("Incompatible options selected for this project, please disable auto_load_libs if "
-                       "you want to use a concrete target.")
+        if self.concrete_target and load_options.get("auto_load_libs", None):
+            l.critical(
+                "Incompatible options selected for this project, please disable auto_load_libs if "
+                "you want to use a concrete target."
+            )
             raise Exception("Incompatible options for the project")
 
-        if self.concrete_target and self.arch.name not in ['X86', 'AMD64', 'ARMHF']:
+        if self.concrete_target and self.arch.name not in ["X86", "AMD64", "ARMHF", "ARMEL", "MIPS32"]:
             l.critical("Concrete execution does not support yet the selected architecture. Aborting.")
             raise Exception("Incompatible options for the project")
 
@@ -170,11 +189,20 @@ class Project:
         self._exclude_sim_procedures_list = exclude_sim_procedures_list
         self.use_sim_procedures = use_sim_procedures
         self._ignore_functions = ignore_functions
-        self._support_selfmodifying_code = support_selfmodifying_code
+        self.selfmodifying_code = selfmodifying_code
         self._translation_cache = translation_cache
-        self._executing = False # this is a flag for the convenience API, exec() and terminate_execution() below
+        self._eager_ifunc_resolution = eager_ifunc_resolution
+        self._executing = False  # this is a flag for the convenience API, exec() and terminate_execution() below
 
-        if self._support_selfmodifying_code:
+        # deprecation warning
+        if support_selfmodifying_code is not None:
+            l.warning(
+                'Parameter "support_selfmodifying_code" is deprecated. Please use "selfmodifying_code"'
+                ' instead. "support_selfmodifying_code" will be removed in the near future.'
+            )
+            self.selfmodifying_code = bool(support_selfmodifying_code)
+
+        if self.selfmodifying_code:
             if self._translation_cache is True:
                 self._translation_cache = False
                 l.warning("Disabling IRSB translation cache because support for self-modifying code is enabled.")
@@ -184,21 +212,13 @@ class Project:
         self.store_function = store_function or self._store
         self.load_function = load_function or self._load
 
-        # Step 4: Set up the project's hubs
-        # Step 4.1 Factory
-        self.factory = AngrObjectFactory(self, default_engine=engine)
+        # FIXME: BIG HACK
+        if isinstance(self.filename, str) and self.filename.endswith(".xcal") and not simos:
+            simos = "snimmuc_nxp"
 
-        # Step 4.2: Analyses
-        self._analyses_preset = analyses_preset
-        self.analyses = None
-        self._initialize_analyses_hub()
-
-        # Step 4.3: ...etc
-        self.kb = KnowledgeBase(self, name="global")
-
-        # Step 5: determine the guest OS
+        # Step 4: determine the guest OS
         if isinstance(simos, type) and issubclass(simos, SimOS):
-            self.simos = simos(self) #pylint:disable=invalid-name
+            self.simos = simos(self)  # pylint:disable=invalid-name
         elif isinstance(simos, str):
             self.simos = os_mapping[simos](self)
         elif simos is None:
@@ -206,15 +226,29 @@ class Project:
         else:
             raise ValueError("Invalid OS specification or non-matching architecture.")
 
+        # Step 5: Set up the project's hubs
+        # Step 5.1 Factory
+        self.factory = AngrObjectFactory(self, default_engine=engine)
+
+        # Step 5.2: Analyses
+        self._analyses_preset = analyses_preset
+        self._analyses = None
+        self._initialize_analyses_hub()
+
+        # Step 5.3: ...etc
+        self.kb = KnowledgeBase(self, name="global")
+
         self.is_java_project = isinstance(self.arch, ArchSoot)
-        self.is_java_jni_project = isinstance(self.arch, ArchSoot) and self.simos.is_javavm_with_jni_support
+        self.is_java_jni_project = isinstance(self.arch, ArchSoot) and getattr(
+            self.simos, "is_javavm_with_jni_support", False
+        )
 
         # Step 6: Register simprocedures as appropriate for library functions
-        if isinstance(self.arch, ArchSoot) and self.simos.is_javavm_with_jni_support:
+        if isinstance(self.arch, ArchSoot) and getattr(self.simos, "is_javavm_with_jni_support", False):
             # If we execute a Java archive that includes native JNI libraries,
             # we need to use the arch of the native simos for all (native) sim
             # procedures.
-            sim_proc_arch = self.simos.native_arch
+            sim_proc_arch = getattr(self.simos, "native_arch")
         else:
             sim_proc_arch = self.arch
         for obj in self.loader.initial_load_objects:
@@ -223,12 +257,23 @@ class Project:
         # Step 7: Run OS-specific configuration
         self.simos.configure_project()
 
+    @property
+    def analyses(self) -> "AnalysesHubWithDefault":
+        result = self._analyses
+        if result is None:
+            raise ValueError("Cannot access analyses this early in project lifecycle")
+        return result
+
+    @analyses.setter
+    def analyses(self, v: "AnalysesHubWithDefault"):
+        self._analyses = v
+
     def _initialize_analyses_hub(self):
         """
         Initializes self.analyses using a given preset.
         """
-        self.analyses = AnalysesHub(self)
-        self.analyses.use_plugin_preset(self._analyses_preset if self._analyses_preset is not None else 'default')
+        self._analyses = cast("AnalysesHubWithDefault", AnalysesHub(self))
+        self._analyses.use_plugin_preset(self._analyses_preset if self._analyses_preset is not None else "default")
 
     def _register_object(self, obj, sim_proc_arch):
         """
@@ -245,7 +290,7 @@ class Project:
         # additionally provide libraries we _have_ loaded as a fallback fallback
         # this helps in the case that e.g. CLE picked up a linux arm libc to satisfy an android arm binary
         for lib in self.loader.all_objects:
-            if lib.provides in SIM_LIBRARIES:
+            if lib.provides is not None and lib.provides in SIM_LIBRARIES:
                 simlib = SIM_LIBRARIES[lib.provides]
                 if simlib not in missing_libs:
                     missing_libs.append(simlib)
@@ -267,16 +312,19 @@ class Project:
                 # ref: https://github.com/angr/angr/issues/1782
                 # (I also don't know why the TYPE_NONE check in the previous clause is there but I can't find a ref for
                 # that. they are probably related.)
+                # (I believe the TYPE_NONE check is to support ELF object files)
                 continue
             if not reloc.resolved:
                 # This is a hack, effectively to support Binary Ninja, which doesn't provide access to dependency
                 # library names. The backend creates the Relocation objects, but leaves them unresolved so that
-                # we can try to guess them here. Once the Binary Ninja API starts supplying the dependencies,
-                # The if/else, along with Project._guess_simprocedure() can be removed if it has no other utility,
-                # just leave behind the 'unresolved' debug statement from the else clause.
+                # we can try to guess them here.
                 if reloc.owner.guess_simprocs:
-                    l.debug("Looking for matching SimProcedure for unresolved %s from %s with hint %s",
-                            func.name, reloc.owner, reloc.owner.guess_simprocs_hint)
+                    l.debug(
+                        "Looking for matching SimProcedure for unresolved %s from %s with hint %s",
+                        func.name,
+                        reloc.owner,
+                        reloc.owner.guess_simprocs_hint,
+                    )
                     self._guess_simprocedure(func, reloc.owner.guess_simprocs_hint)
                 else:
                     l.debug("Ignoring unresolved import '%s' from %s ...?", func.name, reloc.owner)
@@ -290,8 +338,7 @@ class Project:
             # check if we actually can and want to replace it with a SimProcedure.
             # We opt out of this step if it is blacklisted by ignore_functions, which
             # will cause it to be replaced by ReturnUnconstrained later.
-            if export.owner is not self.loader._extern_object and \
-                    export.name not in self._ignore_functions:
+            if export.owner is not self.loader._extern_object and export.name not in self._ignore_functions:
                 if self._check_user_blacklists(export.name):
                     continue
                 owner_name = export.owner.provides
@@ -343,28 +390,36 @@ class Project:
                         the_lib = missing_libs[0]
                         if export.name and export.name.startswith("_Z"):
                             # GNU C++ name. Use a C++ library to create the stub
-                            if 'libstdc++.so' in SIM_LIBRARIES:
-                                the_lib = SIM_LIBRARIES['libstdc++.so']
+                            if "libstdc++.so" in SIM_LIBRARIES:
+                                the_lib = SIM_LIBRARIES["libstdc++.so"]
                             else:
-                                l.critical("Does not find any C++ library in SIM_LIBRARIES. We may not correctly "
-                                           "create the stub or resolve the function prototype for name %s.", export.name)
+                                l.critical(
+                                    "Does not find any C++ library in SIM_LIBRARIES. We may not correctly "
+                                    "create the stub or resolve the function prototype for name %s.",
+                                    export.name,
+                                )
 
                         self.hook_symbol(export.rebased_addr, the_lib.get(export.name, sim_proc_arch))
 
-            # Step 2.5: If 2.4 didn't work (we have NO SimLibraries to work with), just
+            # Step 2.5: If the requesting object wants us to guess simprocedures, do the guessing
+            elif reloc.owner.guess_simprocs and self._guess_simprocedure(func, reloc.owner.guess_simprocs_hint):
+                continue
+
+            # Step 2.6: If 2.4/2.5 didn't work (we have NO SimLibraries to work with), just
             # use the vanilla ReturnUnconstrained, assuming that this isn't a weak func
             elif not func.is_weak:
                 l.info("Using stub SimProcedure for unresolved %s", export.name)
-                self.hook_symbol(export.rebased_addr, SIM_PROCEDURES['stubs']['ReturnUnconstrained'](display_name=export.name, is_stub=True))
+                self.hook_symbol(
+                    export.rebased_addr,
+                    SIM_PROCEDURES["stubs"]["ReturnUnconstrained"](display_name=export.name, is_stub=True),
+                )
 
     def _guess_simprocedure(self, f, hint):
         """
         Does symbol name `f` exist as a SIM_PROCEDURE? If so, return it, else return None.
         Narrows down the set of libraries to search based on hint.
-        Part of the hack to enable Binary Ninja support. Remove if _register_objects() stops using it.
         """
         # First, filter the SIM_LIBRARIES to a reasonable subset based on the hint
-        hinted_libs = []
         if hint == "win":
             hinted_libs = filter(lambda lib: lib if lib.endswith(".dll") else None, SIM_LIBRARIES)
         else:
@@ -373,34 +428,40 @@ class Project:
         for lib in hinted_libs:
             if SIM_LIBRARIES[lib].has_implementation(f.name):
                 l.debug("Found implementation for %s in %s", f, lib)
-                self.hook_symbol(f.relative_addr, (SIM_LIBRARIES[lib].get(f.name, self.arch)))
-                break
-        else:
-            l.debug("Could not find matching SimProcedure for %s, ignoring.", f.name)
+                if f.resolvedby:
+                    hook_at = f.resolvedby.rebased_addr
+                else:
+                    # ????
+                    hook_at = f.relative_addr
+                self.hook_symbol(hook_at, (SIM_LIBRARIES[lib].get(f.name, self.arch)))
+                return True
+
+        l.debug("Could not find matching SimProcedure for %s, ignoring.", f.name)
+        return False
 
     def _check_user_blacklists(self, f):
         """
         Has symbol name `f` been marked for exclusion by any of the user
         parameters?
         """
-        return not self.use_sim_procedures or \
-            f in self._exclude_sim_procedures_list or \
-            f in self._ignore_functions or \
-            (self._exclude_sim_procedures_func is not None and self._exclude_sim_procedures_func(f))
-
+        return (
+            not self.use_sim_procedures
+            or f in self._exclude_sim_procedures_list
+            or f in self._ignore_functions
+            or (self._exclude_sim_procedures_func is not None and self._exclude_sim_procedures_func(f))
+        )
 
     @staticmethod
     def _addr_to_str(addr):
         return "%s" % repr(addr) if isinstance(addr, SootAddressDescriptor) else "%#x" % addr
 
-
     #
     # Public methods
     # They're all related to hooking!
     #
 
     # pylint: disable=inconsistent-return-statements
-    def hook(self, addr, hook=None, length=0, kwargs=None, replace=False):
+    def hook(self, addr, hook=None, length=0, kwargs=None, replace: Optional[bool] = False):
         """
         Hook a section of code with a custom function. This is used internally to provide symbolic
         summaries of library functions, and can be used to instrument execution or to modify
@@ -432,15 +493,18 @@ class Project:
             # if we haven't been passed a thing to hook with, assume we're being used as a decorator
             return self._hook_decorator(addr, length=length, kwargs=kwargs)
 
-        if kwargs is None: kwargs = {}
+        if kwargs is None:
+            kwargs = {}
 
-        l.debug('hooking %s with %s', self._addr_to_str(addr), str(hook))
+        l.debug("hooking %s with %s", self._addr_to_str(addr), str(hook))
 
         if self.is_hooked(addr):
             if replace is True:
                 pass
             elif replace is False:
-                l.warning("Address is already hooked, during hook(%s, %s). Not re-hooking.", self._addr_to_str(addr), hook)
+                l.warning(
+                    "Address is already hooked, during hook(%s, %s). Not re-hooking.", self._addr_to_str(addr), hook
+                )
                 return
             else:
                 l.warning("Address is already hooked, during hook(%s, %s). Re-hooking.", self._addr_to_str(addr), hook)
@@ -449,11 +513,19 @@ class Project:
             raise TypeError("Please instanciate your SimProcedure before hooking with it")
 
         if callable(hook):
-            hook = SIM_PROCEDURES['stubs']['UserHook'](user_func=hook, length=length, **kwargs)
+            hook = SIM_PROCEDURES["stubs"]["UserHook"](user_func=hook, length=length, **kwargs)
+
+        # if not hook.is_stub and hook.guessed_prototype:
+        #    l.error(
+        #        "You need to instantiate %s with a prototype. "
+        #        "It will attempt to guess one but the consequences of guessing wrong are dire.",
+        #        hook,
+        #    )
+        #    l.error("Consider also using angr.SIM_LIBRARIES instead of angr.SIM_PROCEDURES or angr.procedures.")
 
         self._sim_procedures[addr] = hook
 
-    def is_hooked(self, addr):
+    def is_hooked(self, addr) -> bool:
         """
         Returns True if `addr` is hooked.
 
@@ -462,7 +534,7 @@ class Project:
         """
         return addr in self._sim_procedures
 
-    def hooked_by(self, addr):
+    def hooked_by(self, addr) -> Optional[SimProcedure]:
         """
         Returns the current hook for `addr`.
 
@@ -489,7 +561,7 @@ class Project:
 
         del self._sim_procedures[addr]
 
-    def hook_symbol(self, symbol_name, simproc, kwargs=None, replace=None):
+    def hook_symbol(self, symbol_name, simproc, kwargs=None, replace: Optional[bool] = None):
         """
         Resolve a dependency in a binary. Looks up the address of the given symbol, and then hooks that
         address. If the symbol was not available in the loaded libraries, this address may be provided
@@ -517,12 +589,13 @@ class Project:
                 # it could be a previously unresolved weak symbol..?
                 new_sym = None
                 for reloc in self.loader.find_relevant_relocations(symbol_name):
+                    assert reloc.symbol is not None
                     if not reloc.symbol.is_weak:
                         raise Exception("Symbol is strong but we couldn't find its resolution? Report to @rhelmot.")
                     if new_sym is None:
                         new_sym = self.loader.extern_object.make_extern(symbol_name)
                     reloc.resolve(new_sym)
-                    reloc.relocate([])
+                    reloc.relocate()
 
                 if new_sym is None:
                     l.error("Could not find symbol %s", symbol_name)
@@ -539,6 +612,21 @@ class Project:
         self.hook(hook_addr, simproc, kwargs=kwargs, replace=replace)
         return hook_addr
 
+    def symbol_hooked_by(self, symbol_name) -> Optional[SimProcedure]:
+        """
+        Return the SimProcedure, if it exists, for the given symbol name.
+
+        :param str symbol_name: Name of the symbol.
+
+        :returns:    None if the address is not hooked.
+        """
+        sym = self.loader.find_symbol(symbol_name)
+        if sym is None:
+            l.warning("Could not find symbol %s", symbol_name)
+            return None
+        hook_addr, _ = self.simos.prepare_function_symbol(symbol_name, basic_addr=sym.rebased_addr)
+        return self.hooked_by(hook_addr)
+
     def is_symbol_hooked(self, symbol_name):
         """
         Check if a symbol is already hooked.
@@ -565,8 +653,10 @@ class Project:
             l.warning("Could not find symbol %s", symbol_name)
             return False
         if sym.owner is self.loader._extern_object:
-            l.warning("Refusing to unhook external symbol %s, replace it with another hook if you want to change it",
-                      symbol_name)
+            l.warning(
+                "Refusing to unhook external symbol %s, replace it with another hook if you want to change it",
+                symbol_name,
+            )
             return False
 
         hook_addr, _ = self.simos.prepare_function_symbol(symbol_name, basic_addr=sym.rebased_addr)
@@ -582,7 +672,6 @@ class Project:
         """
         new_sim_procedures = {}
         for key_address, simproc_obj in self._sim_procedures.items():
-
             # if we don't want stubs during the sync let's skip those, we will execute the real function.
             if not stubs_on_sync and simproc_obj.is_stub:
                 continue
@@ -655,12 +744,19 @@ class Project:
     #
 
     def __getstate__(self):
+        store_func, load_func = self.store_function, self.load_function
         try:
-            store_func, load_func = self.store_function, self.load_function
             self.store_function, self.load_function = None, None
             # ignore analyses. we re-initialize analyses when restoring from pickling so that we do not lose any newly
             # added analyses classes
-            d = dict((k, v) for k, v in self.__dict__.items() if k not in {'analyses', })
+            d = {
+                k: v
+                for k, v in self.__dict__.items()
+                if k
+                not in {
+                    "analyses",
+                }
+            }
             return d
         finally:
             self.store_function, self.load_function = store_func, load_func
@@ -671,30 +767,30 @@ class Project:
             self._initialize_analyses_hub()
         except AngrNoPluginError:
             l.warning("Plugin preset %s does not exist any more. Fall back to the default preset.")
-            self._analyses_preset = 'default'
+            self._analyses_preset = "default"
             self._initialize_analyses_hub()
 
     def _store(self, container):
         # If container is a filename.
         if isinstance(container, str):
-            with open(container, 'wb') as f:
+            with open(container, "wb") as f:
                 try:
                     pickle.dump(self, f, pickle.HIGHEST_PROTOCOL)
-                except RuntimeError as e: # maximum recursion depth can be reached here
+                except RuntimeError as e:  # maximum recursion depth can be reached here
                     l.error("Unable to store Project: '%s' during pickling", e)
 
         # If container is an open file.
         elif isinstance(container, IOBase):
             try:
                 pickle.dump(self, container, pickle.HIGHEST_PROTOCOL)
-            except RuntimeError as e: # maximum recursion depth can be reached here
+            except RuntimeError as e:  # maximum recursion depth can be reached here
                 l.error("Unable to store Project: '%s' during pickling", e)
 
         # If container is just a variable.
         else:
             try:
                 container = pickle.dumps(self, pickle.HIGHEST_PROTOCOL)
-            except RuntimeError as e: # maximum recursion depth can be reached here
+            except RuntimeError as e:  # maximum recursion depth can be reached here
                 l.error("Unable to store Project: '%s' during pickling", e)
 
     @staticmethod
@@ -702,12 +798,10 @@ class Project:
         if isinstance(container, str):
             # If container is a filename.
             if all(c in string.printable for c in container) and os.path.exists(container):
-                with open(container, 'rb') as f:
+                with open(container, "rb") as f:
                     return pickle.load(f)
-
-            # If container is a pickle string.
             else:
-                return pickle.loads(container)
+                raise ValueError(container)
 
         # If container is an open file
         elif isinstance(container, IOBase):
@@ -719,20 +813,20 @@ class Project:
             return None
 
     def __repr__(self):
-        return '<Project %s>' % (self.filename if self.filename is not None else 'loaded from stream')
+        return "<Project %s>" % (self.filename if self.filename is not None else "loaded from stream")
 
     #
     # Compatibility
     #
 
     @property
-    @deprecated(replacement='simos')
+    @deprecated(replacement="simos")
     def _simos(self):
         return self.simos
 
 
 from .factory import AngrObjectFactory
 from angr.simos import SimOS, os_mapping
-from .analyses.analysis import AnalysesHub
+from .analyses.analysis import AnalysesHub, AnalysesHubWithDefault
 from .knowledge_base import KnowledgeBase
 from .procedures import SIM_PROCEDURES, SIM_LIBRARIES
