@@ -2,12 +2,16 @@ import inspect
 import copy
 import itertools
 import logging
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, Union, Tuple
+
+import claripy
 from cle import SymbolType
 from archinfo.arch_soot import SootAddressDescriptor
 
 if TYPE_CHECKING:
     import angr
+    import archinfo
+    from angr.sim_state import SimState
 
 l = logging.getLogger(name=__name__)
 symbolic_count = itertools.count()
@@ -40,6 +44,8 @@ class SimProcedure:
     The following class variables should be set if necessary when implementing a new SimProcedure:
 
     :cvar NO_RET:           Set this to true if control flow will never return from this function
+    :cvar DYNAMIC_RET:      Set this to true if whether the control flow returns from this function or not depends on
+                            the context (e.g., libc's error() call). Must implement dynamic_returns() method.
     :cvar ADDS_EXITS:       Set this to true if you do any control flow other than returning
     :cvar IS_FUNCTION:      Does this procedure simulate a function? True by default
     :cvar ARGS_MISMATCH:    Does this procedure have a different list of arguments than what is provided in the
@@ -60,7 +66,7 @@ class SimProcedure:
     :ivar kwargs:           Any extra keyword arguments used to construct the procedure; will be passed to ``run``
     :ivar display_name:     See the eponymous parameter
     :ivar library_name:     See the eponymous parameter
-    :ivar abi:
+    :ivar abi:              If this is a syscall simprocedure, which ABI are we using to map the syscall numbers?
     :ivar symbolic_return:  See the eponymous parameter
     :ivar syscall_number:   If this procedure is a syscall, the number will be populated here.
     :ivar returns:          See eponymous parameter and NO_RET cvar
@@ -79,6 +85,8 @@ class SimProcedure:
     :ivar state:            The SimState we should be mutating to perform the procedure
     :ivar successors:       The SimSuccessors associated with the current step
     :ivar arguments:        The function arguments, deserialized from the state
+    :ivar arg_session:      The ArgSession that was used to parse arguments out of the state, in case you need it for
+                            varargs
     :ivar use_state_arguments:
                             Whether we're using arguments extracted from the state or manually provided
     :ivar ret_to:           The current return address
@@ -86,19 +94,36 @@ class SimProcedure:
     :ivar call_ret_expr:    The return value from having used ``self.call()``
     :ivar inhibit_autoret:  Whether we should avoid automatically adding an exit for returning once the run function
                             ends
+    :ivar arg_session:      The ArgSession object that was used to extract the runtime argument values. Useful for if
+                            you want to extract variadic args.
 
     """
+
+    state: "SimState"
+
     def __init__(
-        self, project=None, cc=None, symbolic_return=None,
-        returns=None, is_syscall=False, is_stub=False,
-        num_args=None, display_name=None, library_name=None,
-        is_function=None, **kwargs
+        self,
+        project=None,
+        cc=None,
+        prototype=None,
+        symbolic_return=None,
+        returns=None,
+        is_syscall=False,
+        is_stub=False,
+        num_args=None,
+        display_name=None,
+        library_name=None,
+        is_function=None,
+        **kwargs,
     ):
         # WE'LL FIGURE IT OUT
-        self.project = project # type: angr.Project
-        self.arch = project.arch if project is not None else None
+        self.project: angr.Project = project
+        self.arch: archinfo.arch.Arch = project.arch if project is not None else None
         self.addr = None
-        self.cc = cc # type: angr.SimCC
+        self.cc: angr.SimCC = cc
+        if type(prototype) is str:
+            prototype = parse_signature(prototype)
+        self.prototype: angr.sim_type.SimTypeFunction = prototype
         self.canonical = self
 
         self.kwargs = kwargs
@@ -115,7 +140,7 @@ class SimProcedure:
         self.is_stub = is_stub
         self.is_continuation = False
         self.continuations = {}
-        self.run_func = 'run'
+        self.run_func = "run"
 
         # Get the concrete number of arguments that should be passed to this procedure
         if num_args is None:
@@ -124,6 +149,13 @@ class SimProcedure:
         else:
             self.num_args = num_args
 
+        if self.prototype is None:
+            charp = SimTypePointer(SimTypeChar())
+            self.prototype = SimTypeFunction([charp] * self.num_args, charp)
+            self.guessed_prototype = True
+        else:
+            self.guessed_prototype = False
+
         # runtime values
         self.state = None
         self.successors = None
@@ -133,6 +165,7 @@ class SimProcedure:
         self.ret_expr = None
         self.call_ret_expr = None
         self.inhibit_autoret = None
+        self.arg_session: Union[None, ArgSession, int] = None
 
     def __repr__(self):
         return "<SimProcedure %s%s%s%s%s>" % self._describe_me()
@@ -143,10 +176,10 @@ class SimProcedure:
         """
         return (
             self.display_name,
-            ' (cont: %s)' % self.run_func if self.is_continuation else '',
-            ' (syscall)' if self.is_syscall else '',
-            ' (inline)' if not self.use_state_arguments else '',
-            ' (stub)' if self.is_stub else '',
+            " (cont: %s)" % self.run_func if self.is_continuation else "",
+            " (syscall)" if self.is_syscall else "",
+            " (inline)" if not self.use_state_arguments else "",
+            " (stub)" if self.is_stub else "",
         )
 
     def execute(self, state, successors=None, arguments=None, ret_to=None):
@@ -157,7 +190,7 @@ class SimProcedure:
         provide arguments to the function.
         """
         # fill out all the fun stuff we don't want to frontload
-        if self.addr is None and not state.regs.ip.symbolic:
+        if self.addr is None and not state.regs._ip.symbolic:
             self.addr = state.addr
         if self.arch is None:
             self.arch = state.arch
@@ -165,10 +198,19 @@ class SimProcedure:
             self.project = state.project
         if self.cc is None:
             if self.arch.name in DEFAULT_CC:
-                self.cc = DEFAULT_CC[self.arch.name](self.arch)
+                self.cc = default_cc(
+                    self.arch.name,
+                    platform=self.project.simos.name
+                    if self.project is not None and self.project.simos is not None
+                    else None,
+                )(self.arch)
             else:
-                raise SimProcedureError('There is no default calling convention for architecture %s.'
-                                        ' You must specify a calling convention.' % self.arch.name)
+                raise SimProcedureError(
+                    "There is no default calling convention for architecture %s."
+                    " You must specify a calling convention." % self.arch.name
+                )
+        if self.prototype._arch is None:
+            self.prototype = self.prototype.with_arch(self.arch)
 
         inst = copy.copy(self)
         inst.state = state
@@ -181,22 +223,22 @@ class SimProcedure:
             state.history.recent_syscall_count = 1
 
         state._inspect(
-            'simprocedure',
+            "simprocedure",
             BP_BEFORE,
             simprocedure_name=inst.display_name,
             simprocedure_addr=self.addr,
             simprocedure=inst,
-            simprocedure_result=NO_OVERRIDE
+            simprocedure_result=NO_OVERRIDE,
         )
 
-        r = state._inspect_getattr('simprocedure_result', NO_OVERRIDE)
+        r = state._inspect_getattr("simprocedure_result", NO_OVERRIDE)
         if r is NO_OVERRIDE:
             # get the arguments
 
             # If the simprocedure is related to a Java function call the appropriate setup_args methos
             # TODO: should we move this?
             if self.is_java:
-                sim_args = self._setup_args(inst, state, arguments) #pylint:disable=assignment-from-no-return
+                sim_args = self._setup_args(inst, state, arguments)  # pylint:disable=assignment-from-no-return
                 self.use_state_arguments = False
 
             # handle if this is a continuation from a return
@@ -212,29 +254,38 @@ class SimProcedure:
                     state.regs.lr = saved_lr
                 inst.arguments = sim_args
                 inst.use_state_arguments = True
-                inst.call_ret_expr = state.registers.load(state.arch.ret_offset, state.arch.bytes, endness=state.arch.register_endness)
+                inst.call_ret_expr = state.registers.load(
+                    state.arch.ret_offset, state.arch.bytes, endness=state.arch.register_endness
+                )
                 for name, val in saved_local_vars:
                     setattr(inst, name, val)
             else:
                 if arguments is None:
                     inst.use_state_arguments = True
-                    sim_args = [ inst.arg(_) for _ in range(inst.num_args) ]
+                    inst.arg_session = inst.cc.arg_session(inst.prototype.returnty)
+                    sim_args = [
+                        inst.cc.next_arg(inst.arg_session, ty).get_value(inst.state) for ty in inst.prototype.args
+                    ]
                     inst.arguments = sim_args
                 else:
                     inst.use_state_arguments = False
-                    sim_args = arguments[:inst.num_args]
+                    sim_args = arguments[: inst.num_args]
                     inst.arguments = arguments
+                    inst.arg_session = 0
 
             # run it
             l.debug("Executing %s%s%s%s%s with %s, %s", *(inst._describe_me() + (sim_args, inst.kwargs)))
             r = getattr(inst, inst.run_func)(*sim_args, **inst.kwargs)
 
         state._inspect(
-            'simprocedure',
+            "simprocedure",
             BP_AFTER,
-            simprocedure_result=r
+            simprocedure_name=inst.display_name,
+            simprocedure_addr=self.addr,
+            simprocedure=inst,
+            simprocedure_result=r,
         )
-        r = state._inspect_getattr('simprocedure_result', r)
+        r = state._inspect_getattr("simprocedure_result", r)
 
         if inst.returns and inst.is_function and not inst.inhibit_autoret:
             inst.ret(r)
@@ -245,10 +296,12 @@ class SimProcedure:
         # make a copy of the canon copy, customize it for the specific continuation, then hook it
         if name not in self.canonical.continuations:
             cont = copy.copy(self.canonical)
-            target_name = '%s.%s' % (self.display_name, name)
+            target_name = f"{self.display_name}.{name}"
             should_be_none = self.project.loader.extern_object.get_symbol(target_name)
             if should_be_none is None:
-                cont.addr = self.project.loader.extern_object.make_extern(target_name, sym_type=SymbolType.TYPE_OTHER).rebased_addr
+                cont.addr = self.project.loader.extern_object.make_extern(
+                    target_name, sym_type=SymbolType.TYPE_OTHER
+                ).rebased_addr
             else:
                 l.error("Trying to make continuation %s but it already exists. This is bad.", target_name)
                 cont.addr = self.project.loader.extern_object.allocate()
@@ -263,19 +316,20 @@ class SimProcedure:
     #
 
     NO_RET = False
+    DYNAMIC_RET = False
     ADDS_EXITS = False
     IS_FUNCTION = True
     ARGS_MISMATCH = False
     ALT_NAMES = None  # alternative names
-    local_vars = ()
+    local_vars: Tuple[str, ...] = ()
 
-    def run(self, *args, **kwargs): # pylint: disable=unused-argument
+    def run(self, *args, **kwargs):  # pylint: disable=unused-argument
         """
         Implement the actual procedure here!
         """
         raise SimProcedureError("%s does not implement a run() method" % self.__class__.__name__)
 
-    def static_exits(self, blocks):  # pylint: disable=unused-argument
+    def static_exits(self, blocks, **kwargs):  # pylint: disable=unused-argument
         """
         Get new exits by performing static analysis and heuristics. This is a fast and best-effort approach to get new
         exits for scenarios where states are not available (e.g. when building a fast CFG).
@@ -291,6 +345,19 @@ class SimProcedure:
         # This SimProcedure does not add any new exit
         return []
 
+    def dynamic_returns(self, blocks, **kwargs) -> bool:  # pylint:disable=unused-argument
+        """
+        Determines if a call to this function returns or not by performing static analysis and heuristics.
+
+        :param blocks:  Blocks that are executed before reaching this SimProcedure.
+        :return:        True if the call returns, False otherwise.
+        """
+
+        if self.DYNAMIC_RET:
+            raise SimProcedureError(f"dynamic_returns() is not implemented for {self}")
+
+        return True
+
     #
     # misc properties
     #
@@ -303,36 +370,32 @@ class SimProcedure:
     # Working with calling conventions
     #
 
-    def _setup_args(self, inst, state, args): #pylint:disable=unused-argument,no-self-use
+    def _setup_args(self, inst, state, args):  # pylint:disable=unused-argument,no-self-use
         raise SimProcedureError("the java-specific _setup_args() method was invoked on a non-Java SimProcedure.")
-    def _compute_ret_addr(self, expr): #pylint:disable=unused-argument,no-self-use
+
+    def _compute_ret_addr(self, expr):  # pylint:disable=unused-argument,no-self-use
         raise SimProcedureError("the java-specific _compute_ret_addr() method was invoked on a non-Java SimProcedure.")
 
     def set_args(self, args):
-        arg_session = self.cc.arg_session
-        for arg in args:
-            if self.cc.is_fp_value(args):
-                arg_session.next_arg(True).set_value(self.state, arg)
-            else:
-                arg_session.next_arg(False).set_value(self.state, arg)
+        arg_session = self.cc.arg_session(self.prototype.returnty)
+        for arg, ty in zip(args, self.prototype.args):
+            self.cc.next_arg(arg_session, ty).set_value(self.state, arg)
 
-    def arg(self, i):
-        """
-        Returns the ith argument. Raise a SimProcedureArgumentError if we don't have such an argument available.
+    def va_arg(self, ty, index=None):
+        if not self.use_state_arguments:
+            if index is not None:
+                return self.arguments[self.num_args + index]
 
-        :param int i: The index of the argument to get
-        :return: The argument
-        :rtype: object
-        """
-        if self.use_state_arguments:
-            r = self.cc.arg(self.state, i)
-        else:
-            if i >= len(self.arguments):
-                raise SimProcedureArgumentError("Argument %d does not exist." % i)
-            r = self.arguments[i]           # pylint: disable=unsubscriptable-object
+            result = self.arguments[self.num_args + self.arg_session]
+            self.arg_session += 1
+            return result
 
-        l.debug("returning argument")
-        return r
+        if index is not None:
+            raise Exception("you think you're so fucking smart? you implement this logic then")
+
+        if type(ty) is str:
+            ty = parse_type(ty, arch=self.arch)
+        return self.cc.next_arg(self.arg_session, ty).get_value(self.state)
 
     #
     # Control Flow
@@ -349,10 +412,16 @@ class SimProcedure:
         :param sim_kwargs:      Any additional keyword args will be passed as sim_kwargs to the
                                 procedure construtor
         """
-        e_args = [ self.state.solver.BVV(a, self.state.arch.bits) if isinstance(a, int) else a for a in arguments ]
+        e_args = [self.state.solver.BVV(a, self.state.arch.bits) if isinstance(a, int) else a for a in arguments]
         p = procedure(project=self.project, **kwargs)
         return p.execute(self.state, None, arguments=e_args)
 
+    def fix_prototype_returnty(self, ret_size):
+        if ret_size not in [8, 16, 32, 64]:
+            raise NotImplementedError(f"return type of size {ret_size} is not handled!")
+        returnty = SimTypeNum(ret_size, signed=False, label=f"u{ret_size}")
+        self.prototype.returnty = returnty
+
     def ret(self, expr=None):
         """
         Add an exit representing a return from this function.
@@ -371,9 +440,8 @@ class SimProcedure:
             if self.symbolic_return:
                 size = len(expr)
                 new_expr = self.state.solver.Unconstrained(
-                        "symbolic_return_" + self.display_name,
-                        size,
-                        key=('symbolic_return', self.display_name)) #pylint:disable=maybe-no-member
+                    "symbolic_return_" + self.display_name, size, key=("symbolic_return", self.display_name)
+                )  # pylint:disable=maybe-no-member
                 self.state.add_constraints(new_expr == expr)
                 expr = new_expr
 
@@ -383,12 +451,17 @@ class SimProcedure:
         # TODO: I had to put this check here because I don't understand why self.use_state_arguments gets reset to true
         # when calling the function ret. at the calling point the attribute is set to False
         if isinstance(self.addr, SootAddressDescriptor):
-            ret_addr = self._compute_ret_addr(expr) #pylint:disable=assignment-from-no-return
+            ret_addr = self._compute_ret_addr(expr)  # pylint:disable=assignment-from-no-return
         elif self.use_state_arguments:
-            ret_addr = self.cc.teardown_callsite(
-                    self.state,
-                    expr,
-                    arg_types=[False]*self.num_args if self.cc.args is None else None)
+            # in case we guess the prototype wrong, use the return value's size as a hint to fix it
+            if (
+                self.guessed_prototype
+                and isinstance(expr, claripy.ast.bv.BV)
+                and self.prototype.returnty.size != len(expr)
+            ):
+                self.fix_prototype_returnty(len(expr))
+
+            ret_addr = self.cc.teardown_callsite(self.state, return_val=expr, prototype=self.prototype)
 
         if not self.should_add_successors:
             l.debug("Returning without setting exits due to 'internal' call.")
@@ -403,10 +476,9 @@ class SimProcedure:
         self._prepare_ret_state()
 
         self._exit_action(self.state, ret_addr)
-        self.successors.add_successor(self.state, ret_addr, self.state.solver.true, 'Ijk_Ret')
+        self.successors.add_successor(self.state, ret_addr, self.state.solver.true, "Ijk_Ret")
 
-
-    def call(self, addr, args, continue_at, cc=None):
+    def call(self, addr, args, continue_at, cc=None, prototype=None, jumpkind="Ijk_Call"):
         """
         Add an exit representing calling another function via pointer.
 
@@ -416,50 +488,54 @@ class SimProcedure:
                             procedure will continue in the named method.
         :param cc:          Optional: use this calling convention for calling the new function.
                             Default is to use the current convention.
+        :param prototype:     Optional: The prototype to use for the call. Will default to all-ints.
         """
         self.inhibit_autoret = True
 
         if cc is None:
             cc = self.cc
+        prototype = cc.guess_prototype(args, prototype)
 
         call_state = self.state.copy()
         ret_addr = self.make_continuation(continue_at)
         saved_local_vars = list(zip(self.local_vars, map(lambda name: getattr(self, name), self.local_vars)))
-        simcallstack_entry = (self.state.regs.sp if hasattr(self.state.regs, "sp") else None,
-                              self.arguments,
-                              saved_local_vars,
-                              self.state.regs.lr if self.state.arch.lr_offset is not None else None,
-                              ret_addr)
-        cc.setup_callsite(call_state, ret_addr, args)
+        simcallstack_entry = (
+            self.state.regs.sp if hasattr(self.state.regs, "sp") else None,
+            self.arguments,
+            saved_local_vars,
+            self.state.regs.lr if self.state.arch.lr_offset is not None else None,
+            ret_addr,
+        )
+        cc.setup_callsite(call_state, ret_addr, args, prototype)
         call_state.callstack.top.procedure_data = simcallstack_entry
 
         # TODO: Move this to setup_callsite?
         if isinstance(call_state.addr, SootAddressDescriptor):
             pass
-        elif call_state.libc.ppc64_abiv == 'ppc64_1':
-            call_state.regs.r2 = self.state.mem[addr + 8:].long.resolved
+        elif call_state.libc.ppc64_abiv == "ppc64_1":
+            call_state.regs.r2 = self.state.mem[addr + 8 :].long.resolved
             addr = call_state.mem[addr:].long.resolved
-        elif call_state.arch.name in ('MIPS32', 'MIPS64'):
+        elif call_state.arch.name in ("MIPS32", "MIPS64"):
             call_state.regs.t9 = addr
 
         self._exit_action(call_state, addr)
-        self.successors.add_successor(call_state, addr, call_state.solver.true, 'Ijk_Call')
+        self.successors.add_successor(call_state, addr, call_state.solver.true, jumpkind)
 
         if o.DO_RET_EMULATION in self.state.options:
             # we need to set up the call because the continuation will try to tear it down
             ret_state = self.state.copy()
-            cc.setup_callsite(ret_state, ret_addr, args)
+            cc.setup_callsite(ret_state, ret_addr, args, prototype)
             ret_state.callstack.top.procedure_data = simcallstack_entry
             guard = ret_state.solver.true if o.TRUE_RET_EMULATION_GUARD in ret_state.options else ret_state.solver.false
-            self.successors.add_successor(ret_state, ret_addr, guard, 'Ijk_FakeRet')
+            self.successors.add_successor(ret_state, ret_addr, guard, "Ijk_FakeRet")
 
-    def jump(self, addr):
+    def jump(self, addr, jumpkind="Ijk_Boring"):
         """
         Add an exit representing jumping to an address.
         """
         self.inhibit_autoret = True
         self._exit_action(self.state, addr)
-        self.successors.add_successor(self.state, addr, self.state.solver.true, 'Ijk_Boring')
+        self.successors.add_successor(self.state, addr, self.state.solver.true, jumpkind)
 
     def exit(self, exit_code):
         """
@@ -471,8 +547,8 @@ class SimProcedure:
 
         if isinstance(exit_code, int):
             exit_code = self.state.solver.BVV(exit_code, self.state.arch.bits)
-        self.state.history.add_event('terminate', exit_code=exit_code)
-        self.successors.add_successor(self.state, self.state.regs.ip, self.state.solver.true, 'Ijk_Exit')
+        self.state.history.add_event("terminate", exit_code=exit_code)
+        self.successors.add_successor(self.state, self.state.regs.ip, self.state.solver.true, "Ijk_Exit")
 
     @staticmethod
     def _exit_action(state, addr):
@@ -499,7 +575,7 @@ class SimProcedure:
 
     @argument_types.setter
     def argument_types(self, v):  # pylint: disable=unused-argument,no-self-use
-        l.critical("SimProcedure.argument_types is deprecated. specify the function signature in the cc")
+        l.critical("SimProcedure.argument_types is deprecated. specify the function signature in the prototype param")
 
     @property
     def return_type(self):  # pylint: disable=no-self-use
@@ -507,12 +583,20 @@ class SimProcedure:
 
     @return_type.setter
     def return_type(self, v):  # pylint: disable=unused-argument,no-self-use
-        l.critical("SimProcedure.return_type is deprecated. specify the function signature in the cc")
+        l.critical("SimProcedure.return_type is deprecated. specify the function signature in the prototype param")
 
 
 from . import sim_options as o
-from angr.errors import SimProcedureError, SimProcedureArgumentError, SimShadowStackError
-from angr.sim_type import SimTypePointer
+from angr.errors import SimProcedureError, SimShadowStackError
 from angr.state_plugins.sim_action import SimActionExit
-from angr.calling_conventions import DEFAULT_CC
+from angr.calling_conventions import (
+    DEFAULT_CC,
+    default_cc,
+    SimTypeFunction,
+    SimTypePointer,
+    SimTypeChar,
+    ArgSession,
+    SimTypeNum,
+)
 from .state_plugins import BP_AFTER, BP_BEFORE, NO_OVERRIDE
+from .sim_type import parse_signature, parse_type
