@@ -16,22 +16,26 @@ class SimUserland(SimOS):
     It uses the CLE kernel object to provide addresses for syscalls. Syscalls will be emulated as a jump to one of these
     addresses, where a SimProcedure from the syscall library provided at construction time will be executed.
     """
+
     def __init__(self, project, syscall_library=None, syscall_addr_alignment=4, **kwargs):
-        super(SimUserland, self).__init__(project, **kwargs)
+        super().__init__(project, **kwargs)
         self.syscall_library = syscall_library.copy()
         self.syscall_addr_alignment = syscall_addr_alignment
         self.kernel_base = None
         self.unknown_syscall_number = None
-        self.syscall_abis: Dict[str,Tuple[int,int,int]] = {}
+        self.syscall_abis: Dict[str, Tuple[int, int, int]] = {}
         # syscall_abis is a dict of tuples {name: (base_number, min_number, max_number)}
         # min_number and max_number are just cached from SimSyscallLibrary.{min,max}imum_sysall_number
         # base_number is used to map the syscalls into the syscall address space - it's a "base address"
-        # but a number. to convert from syscall number to address it's (number - min_num + base_num) * alignment + kernel_base
+        # but a number. to convert from syscall number to address it's
+        # (number - min_num + base_num) * alignment + kernel_base
 
-    def configure_project(self, abi_list=None): # pylint: disable=arguments-differ
+    def configure_project(self, abi_list=None):  # pylint: disable=arguments-differ
         if abi_list is None:
             abi_list = list(self.syscall_library.syscall_number_mapping)
-            assert len(abi_list) == 1, "More than one ABI is available for this target - you need to specify which ones are valid"
+            assert (
+                len(abi_list) == 1
+            ), "More than one ABI is available for this target - you need to specify which ones are valid"
         self.kernel_base = self.project.loader.kernel_object.mapped_base
 
         base_no = 0
@@ -40,7 +44,7 @@ class SimUserland(SimOS):
             min_no = self.syscall_library.minimum_syscall_number(abi)
             max_no = self.syscall_library.maximum_syscall_number(abi)
             self.syscall_abis[abi] = (base_no, min_no, max_no)
-            base_no += max_no - min_no + 1 # since max is the actual max and not the array length
+            base_no += max_no - min_no + 1  # since max is the actual max and not the array length
 
         self.unknown_syscall_number = base_no
 
@@ -54,7 +58,7 @@ class SimUserland(SimOS):
         else:
             # Use the default syscall calling convention - it may bring problems
             _l.warning("No syscall calling convention available for %s/%s", state.arch.name, state.os_name)
-            cc = SYSCALL_CC[state.arch.name]['default'](state.arch)
+            cc = SYSCALL_CC[state.arch.name]["default"](state.arch)
         return cc
 
     def syscall(self, state, allow_unsupported=True):
@@ -81,12 +85,10 @@ class SimUserland(SimOS):
                     raise AngrUnsupportedSyscallError("Got a symbolic syscall number")
 
         proc = self.syscall_from_number(num, allow_unsupported=allow_unsupported, abi=abi)
-        if proc.cc is not None:
-            cc.func_ty = proc.cc.func_ty
         proc.cc = cc
         return proc
 
-    def syscall_abi(self, state): # pylint: disable=unused-argument,no-self-use
+    def syscall_abi(self, state):  # pylint: disable=unused-argument,no-self-use
         """
         Optionally, override this function to determine which abi is being used for the state's current syscall.
         """
@@ -135,7 +137,8 @@ class SimUserland(SimOS):
         Get a syscall SimProcedure from its number.
 
         :param number:              The syscall number
-        :param allow_unsupported:   Whether to return a "stub" syscall for unsupported numbers instead of throwing an error
+        :param allow_unsupported:   Whether to return a "stub" syscall for unsupported numbers instead of throwing an
+                                    error
         :param abi:                 The name of the abi to use. If None, will assume that the abis have disjoint
                                     numbering schemes and pick the right one.
         :return: The SimProcedure for the syscall
@@ -145,7 +148,7 @@ class SimUserland(SimOS):
         if self.syscall_library is None:
             if not allow_unsupported:
                 raise AngrUnsupportedSyscallError("%s does not have a library of syscalls implemented" % self.name)
-            proc = P['stubs']['syscall']()
+            proc = P["stubs"]["syscall"]()
         elif not allow_unsupported and not self.syscall_library.has_implementation(number, self.arch, abilist):
             raise AngrUnsupportedSyscallError("No implementation for syscall %d" % number)
         else:
