@@ -18,17 +18,15 @@ from ..errors import (
 from .. import sim_options as o
 from ..tablespecs import StringTableSpec
 from ..procedures import SIM_LIBRARIES as L
+from ..procedures.definitions import load_win32api_definitions
 from .simos import SimOS
 
 _l = logging.getLogger(name=__name__)
 
 
-_VS_Security_Cookie = collections.namedtuple('_VS_Security_Cookie', ('default', 'width'))
+_VS_Security_Cookie = collections.namedtuple("_VS_Security_Cookie", ("default", "width"))
 # security cookie details from visual studio, keyed by architecture name
-VS_SECURITY_COOKIES = {
-    'AMD64': _VS_Security_Cookie(0x2b992ddfa232, 48),
-    'X86': _VS_Security_Cookie(0xbb40e64e, 32)
-}
+VS_SECURITY_COOKIES = {"AMD64": _VS_Security_Cookie(0x2B992DDFA232, 48), "X86": _VS_Security_Cookie(0xBB40E64E, 32)}
 
 
 class SecurityCookieInit(enum.Enum):
@@ -40,10 +38,13 @@ class SecurityCookieInit(enum.Enum):
 
 class SimWindows(SimOS):
     """
-    Environemnt for the Windows Win32 subsystem. Does not support syscalls currently.
+    Environment for the Windows Win32 subsystem. Does not support syscalls currently.
     """
+
     def __init__(self, project):
-        super(SimWindows, self).__init__(project, name='Win32')
+        super().__init__(project, name="Win32")
+
+        load_win32api_definitions()
 
         self._exception_handler = None
         self.fmode_ptr = None
@@ -52,22 +53,22 @@ class SimWindows(SimOS):
         self.wcmdln_ptr = None
 
     def configure_project(self):
-        super(SimWindows, self).configure_project()
+        super().configure_project()
 
         # here are some symbols which we MUST hook, regardless of what the user wants
-        self._weak_hook_symbol('GetProcAddress', L['kernel32.dll'].get('GetProcAddress', self.arch))
-        self._weak_hook_symbol('LoadLibraryA', L['kernel32.dll'].get('LoadLibraryA', self.arch))
-        self._weak_hook_symbol('LoadLibraryExW', L['kernel32.dll'].get('LoadLibraryExW', self.arch))
+        self._weak_hook_symbol("GetProcAddress", L["kernel32.dll"].get("GetProcAddress", self.arch))
+        self._weak_hook_symbol("LoadLibraryA", L["kernel32.dll"].get("LoadLibraryA", self.arch))
+        self._weak_hook_symbol("LoadLibraryExW", L["kernel32.dll"].get("LoadLibraryExW", self.arch))
 
-        self._exception_handler = self._find_or_make('KiUserExceptionDispatcher')
-        self.project.hook(self._exception_handler,
-                          L['ntdll.dll'].get('KiUserExceptionDispatcher', self.arch),
-                          replace=True)
+        self._exception_handler = self._find_or_make("KiUserExceptionDispatcher")
+        self.project.hook(
+            self._exception_handler, L["ntdll.dll"].get("KiUserExceptionDispatcher", self.arch), replace=True
+        )
 
-        self.fmode_ptr = self._find_or_make('_fmode')
-        self.commode_ptr = self._find_or_make('_commode')
-        self.acmdln_ptr = self._find_or_make('_acmdln')
-        self.wcmdln_ptr = self._find_or_make('_wcmdln')
+        self.fmode_ptr = self._find_or_make("_fmode")
+        self.commode_ptr = self._find_or_make("_commode")
+        self.acmdln_ptr = self._find_or_make("_acmdln")
+        self.wcmdln_ptr = self._find_or_make("_wcmdln")
 
         self.is_dump = isinstance(self.project.loader.main_object, cle.backends.Minidump)
 
@@ -83,10 +84,10 @@ class SimWindows(SimOS):
 
     # pylint: disable=arguments-differ
     def state_entry(self, args=None, env=None, argc=None, **kwargs):
-        state = super(SimWindows, self).state_entry(**kwargs)
+        state = super().state_entry(**kwargs)
 
         # Handle default values
-        filename = self.project.filename or 'dummy_filename'
+        filename = self.project.filename or "dummy_filename"
         if args is None:
             args = [filename]
 
@@ -108,18 +109,18 @@ class SimWindows(SimOS):
         cmdline = claripy.BVV(0, 0)
         for arg in args:
             if cmdline.length != 0:
-                cmdline = cmdline.concat(claripy.BVV(b' '))
+                cmdline = cmdline.concat(claripy.BVV(b" "))
 
             if type(arg) is str:
                 arg = arg.encode()
             if type(arg) is bytes:
-                if b'"' in arg or b'\0' in arg:
+                if b'"' in arg or b"\0" in arg:
                     raise AngrSimOSError("Can't handle windows args with quotes or nulls in them")
                 arg = claripy.BVV(arg)
             elif isinstance(arg, claripy.ast.BV):
                 for byte in arg.chop(8):
-                    state.solver.add(byte != claripy.BVV(b'"'))
-                    state.solver.add(byte != claripy.BVV(0, 8))
+                    state.add_constraints(byte != claripy.BVV(b'"'))
+                    state.add_constraints(byte != claripy.BVV(0, 8))
             else:
                 raise TypeError("Argument must be str or bytes or bitvector")
 
@@ -133,7 +134,7 @@ class SimWindows(SimOS):
         # Dump the table onto the stack, calculate pointers to args, env
         stack_ptr = state.regs.sp
         stack_ptr -= 16
-        state.memory.store(stack_ptr, claripy.BVV(0, 8*16))
+        state.memory.store(stack_ptr, claripy.BVV(0, 8 * 16))
 
         stack_ptr -= cmdline.length // 8
         state.memory.store(stack_ptr, cmdline)
@@ -156,16 +157,16 @@ class SimWindows(SimOS):
         state.posix.argc = argc
         state.posix.environ = envp
 
-        state.regs.sp = state.regs.sp - 0x80    # give us some stack space to work with
+        state.regs.sp = state.regs.sp - 0x80  # give us some stack space to work with
 
         # fake return address from entry point
         return_addr = self.return_deadend
-        kernel32 = self.project.loader.shared_objects.get('kernel32.dll', None)
+        kernel32 = self.project.loader.shared_objects.get("kernel32.dll", None)
         if kernel32:
             # some programs will use the return address from start to find the kernel32 base
-            return_addr = kernel32.get_symbol('ExitProcess').rebased_addr
+            return_addr = kernel32.get_symbol("ExitProcess").rebased_addr
 
-        if state.arch.name == 'X86':
+        if state.arch.name == "X86":
             state.mem[state.regs.sp].dword = return_addr
 
             # first argument appears to be PEB
@@ -177,22 +178,22 @@ class SimWindows(SimOS):
 
     def state_blank(self, thread_idx=None, **kwargs):
         if self.project.loader.main_object.supports_nx:
-            add_options = kwargs.get('add_options', set())
+            add_options = kwargs.get("add_options", set())
             add_options.add(o.ENABLE_NX)
-            kwargs['add_options'] = add_options
-        state = super(SimWindows, self).state_blank(thread_idx=thread_idx, **kwargs)
+            kwargs["add_options"] = add_options
+        state = super().state_blank(thread_idx=thread_idx, **kwargs)
 
         if not self.is_dump:
             # yikes!!!
             fun_stuff_addr = state.heap.mmap_base
-            if fun_stuff_addr & 0xffff != 0:
-                fun_stuff_addr += 0x10000 - (fun_stuff_addr & 0xffff)
+            if fun_stuff_addr & 0xFFFF != 0:
+                fun_stuff_addr += 0x10000 - (fun_stuff_addr & 0xFFFF)
             state.memory.map_region(fun_stuff_addr, 0x2000, claripy.BVV(3, 3))
 
             TIB_addr = fun_stuff_addr
             PEB_addr = fun_stuff_addr + 0x1000
 
-            if state.arch.name == 'X86':
+            if state.arch.name == "X86":
                 LDR_addr = fun_stuff_addr + 0x2000
                 if thread_idx is None:
                     thread_idx = 0
@@ -201,13 +202,15 @@ class SimWindows(SimOS):
                 state.mem[TIB_addr + 4].dword = state.regs.sp  # stack base (high addr)
                 state.mem[TIB_addr + 8].dword = state.regs.sp - 0x100000  # stack limit (low addr)
                 state.mem[TIB_addr + 0x18].dword = TIB_addr  # myself!
-                state.mem[TIB_addr + 0x24].dword = 0xbad76ead  # thread id
-                state.mem[TIB_addr + 0x2c].dword = self.project.loader.tls.threads[thread_idx].user_thread_pointer  # tls array pointer
+                state.mem[TIB_addr + 0x24].dword = 0xBAD76EAD  # thread id
+                state.mem[TIB_addr + 0x2C].dword = self.project.loader.tls.threads[
+                    thread_idx
+                ].user_thread_pointer  # tls array pointer
                 state.mem[TIB_addr + 0x30].dword = PEB_addr  # PEB addr, of course
 
                 state.regs.fs = TIB_addr >> 16
 
-                state.mem[PEB_addr + 0xc].dword = LDR_addr
+                state.mem[PEB_addr + 0xC].dword = LDR_addr
 
                 # OKAY IT'S TIME TO SUFFER
                 # http://sandsprite.com/CodeStuff/Understanding_the_Peb_Loader_Data_List.html
@@ -217,35 +220,35 @@ class SimWindows(SimOS):
                 string_alloc_size = 0
                 for obj in self.project.loader.all_pe_objects:
                     bin_name = obj.binary_basename if obj.binary is None else obj.binary
-                    string_alloc_size += len(bin_name)*2 + 2
+                    string_alloc_size += len(bin_name) * 2 + 2
                 total_alloc_size = thunk_alloc_size + string_alloc_size
-                if total_alloc_size & 0xfff != 0:
-                    total_alloc_size += 0x1000 - (total_alloc_size & 0xfff)
+                if total_alloc_size & 0xFFF != 0:
+                    total_alloc_size += 0x1000 - (total_alloc_size & 0xFFF)
                 state.memory.map_region(LDR_addr, total_alloc_size, claripy.BVV(3, 3))
                 state.heap.mmap_base = LDR_addr + total_alloc_size
 
                 string_area = LDR_addr + thunk_alloc_size
                 for i, obj in enumerate(self.project.loader.all_pe_objects):
                     # Create a LDR_MODULE, we'll handle the links later...
-                    obj.module_id = i+1  # HACK HACK HACK HACK
-                    addr = LDR_addr + (i+1) * THUNK_SIZE
-                    state.mem[addr+0x18].dword = obj.mapped_base
-                    state.mem[addr+0x1C].dword = obj.entry
+                    obj.module_id = i + 1  # HACK HACK HACK HACK
+                    addr = LDR_addr + (i + 1) * THUNK_SIZE
+                    state.mem[addr + 0x18].dword = obj.mapped_base
+                    state.mem[addr + 0x1C].dword = obj.entry
 
                     # Allocate some space from the same region to store the paths
                     path = obj.binary_basename if obj.binary is None else obj.binary
                     string_size = len(path) * 2
                     tail_size = len(os.path.basename(path)) * 2
-                    state.mem[addr+0x24].short = string_size
-                    state.mem[addr+0x26].short = string_size
-                    state.mem[addr+0x28].dword = string_area
-                    state.mem[addr+0x2C].short = tail_size
-                    state.mem[addr+0x2E].short = tail_size
-                    state.mem[addr+0x30].dword = string_area + string_size - tail_size
+                    state.mem[addr + 0x24].short = string_size
+                    state.mem[addr + 0x26].short = string_size
+                    state.mem[addr + 0x28].dword = string_area
+                    state.mem[addr + 0x2C].short = tail_size
+                    state.mem[addr + 0x2E].short = tail_size
+                    state.mem[addr + 0x30].dword = string_area + string_size - tail_size
 
                     for j, c in enumerate(path):
                         # if this segfaults, increase the allocation size
-                        state.mem[string_area + j*2].short = ord(c)
+                        state.mem[string_area + j * 2].short = ord(c)
                     state.mem[string_area + string_size].short = 0
                     string_area += string_size + 2
 
@@ -270,7 +273,7 @@ class SimWindows(SimOS):
 
                 def link(a, b):
                     state.mem[a].dword = b
-                    state.mem[b+4].dword = a
+                    state.mem[b + 4].dword = a
 
                 # I have genuinely never felt so dead in my life as I feel writing this code
                 def link_list(mods, offset):
@@ -329,9 +332,13 @@ class SimWindows(SimOS):
                 r = self.project.factory.default_engine.process(successors.initial_state, num_inst=num_inst)
                 if len(r.flat_successors) != 1:
                     if exception.guard.is_true():
-                        _l.error("Got %d successors while re-executing %d instructions at %#x "
-                                 "for unconditional exception windup",
-                                 len(r.flat_successors), num_inst, successors.initial_state.addr)
+                        _l.error(
+                            "Got %d successors while re-executing %d instructions at %#x "
+                            "for unconditional exception windup",
+                            len(r.flat_successors),
+                            num_inst,
+                            successors.initial_state.addr,
+                        )
                         raise exception
                     # Try to figure out which successor is ours...
                     _, _, canon_guard = exception.guard.canonicalize()
@@ -341,22 +348,29 @@ class SimWindows(SimOS):
                             exc_state = possible_succ
                             break
                     else:
-                        _l.error("None of the %d successors while re-executing %d instructions at %#x "
-                                 "for conditional exception windup matched guard",
-                                 len(r.flat_successors), num_inst, successors.initial_state.addr)
+                        _l.error(
+                            "None of the %d successors while re-executing %d instructions at %#x "
+                            "for conditional exception windup matched guard",
+                            len(r.flat_successors),
+                            num_inst,
+                            successors.initial_state.addr,
+                        )
                         raise exception
 
                 else:
                     exc_state = r.flat_successors[0]
-            except:
+            except Exception as e:
                 # lol no
-                _l.error("Got some weirdo error while re-executing %d instructions at %#x "
-                         "for exception windup", num_inst, successors.initial_state.addr)
-                raise exception
+                _l.error(
+                    "Got some weirdo error while re-executing %d instructions at %#x for exception windup",
+                    num_inst,
+                    successors.initial_state.addr,
+                )
+                raise exception from e
         else:
             # duplicate the history-cycle code here...
             exc_state = successors.initial_state.copy()
-            exc_state.register_plugin('history', successors.initial_state.history.make_child())
+            exc_state.register_plugin("history", successors.initial_state.history.make_child())
             exc_state.history.recent_bbl_addrs.append(successors.initial_state.addr)
 
         _l.debug("... wound up state to %#x", exc_state.addr)
@@ -366,12 +380,12 @@ class SimWindows(SimOS):
         tib_addr = exc_state.regs._fs.concat(exc_state.solver.BVV(0, 16))
         if exc_state.solver.is_true(exc_state.mem[tib_addr].long.resolved == -1):
             _l.debug("... no handlers registered")
-            exception.args = ('Unhandled exception: %r' % exception,)
+            exception.args = ("Unhandled exception: %r" % exception,)
             raise exception
         # catch nested exceptions here with magic value
         if exc_state.solver.is_true(exc_state.mem[tib_addr].long.resolved == 0xBADFACE):
             _l.debug("... nested exception")
-            exception.args = ('Unhandled exception: %r' % exception,)
+            exception.args = ("Unhandled exception: %r" % exception,)
             raise exception
 
         # serialize the thread context and set up the exception record...
@@ -382,17 +396,17 @@ class SimWindows(SimOS):
         # https://msdn.microsoft.com/en-us/library/windows/desktop/aa363082(v=vs.85).aspx
         exc_state.mem[record + 0x4].uint32_t = 0  # flags = continuable
         exc_state.mem[record + 0x8].uint32_t = 0  # FUCK chained exceptions
-        exc_state.mem[record + 0xc].uint32_t = exc_state.regs._eip  # exceptionaddress
+        exc_state.mem[record + 0xC].uint32_t = exc_state.regs._eip  # exceptionaddress
         for i in range(16):  # zero out the arg count and args array
-            exc_state.mem[record + 0x10 + 4*i].uint32_t = 0
+            exc_state.mem[record + 0x10 + 4 * i].uint32_t = 0
         # TOTAL SIZE: 0x50
 
         # the rest of the parameters have to be set per-exception type
         # https://msdn.microsoft.com/en-us/library/cc704588.aspx
         if type(exception) is SimSegfaultException:
-            exc_state.mem[record].uint32_t = 0xc0000005  # STATUS_ACCESS_VIOLATION
+            exc_state.mem[record].uint32_t = 0xC0000005  # STATUS_ACCESS_VIOLATION
             exc_state.mem[record + 0x10].uint32_t = 2
-            exc_state.mem[record + 0x14].uint32_t = 1 if exception.reason.startswith('write-') else 0
+            exc_state.mem[record + 0x14].uint32_t = 1 if exception.reason.startswith("write-") else 0
             exc_state.mem[record + 0x18].uint32_t = exception.addr
         elif type(exception) is SimZeroDivisionException:
             exc_state.mem[record].uint32_t = 0xC0000094  # STATUS_INTEGER_DIVIDE_BY_ZERO
@@ -405,58 +419,58 @@ class SimWindows(SimOS):
 
         # let's go let's go!
         # we want to use a true guard here. if it's not true, then it's already been added in windup.
-        successors.add_successor(exc_state, self._exception_handler, exc_state.solver.true, 'Ijk_Exception')
+        successors.add_successor(exc_state, self._exception_handler, exc_state.solver.true, "Ijk_Exception")
         successors.processed = True
 
     # these two methods load and store register state from a struct CONTEXT
     # https://www.nirsoft.net/kernel_struct/vista/CONTEXT.html
     @staticmethod
     def _dump_regs(state, addr):
-        if state.arch.name != 'X86':
+        if state.arch.name != "X86":
             raise SimUnsupportedError("I don't know how to work with struct CONTEXT outside of i386")
 
         # I decline to load and store the floating point/extended registers
-        state.mem[addr + 0].uint32_t = 0x07        # contextflags = control | integer | segments
+        state.mem[addr + 0].uint32_t = 0x07  # contextflags = control | integer | segments
         # dr0 - dr7 are at 0x4-0x18
         # fp state is at 0x1c: 8 ulongs plus a char[80] gives it size 0x70
-        state.mem[addr + 0x8c].uint32_t = state.regs.gs.concat(state.solver.BVV(0, 16))
+        state.mem[addr + 0x8C].uint32_t = state.regs.gs.concat(state.solver.BVV(0, 16))
         state.mem[addr + 0x90].uint32_t = state.regs.fs.concat(state.solver.BVV(0, 16))
         state.mem[addr + 0x94].uint32_t = 0  # es
         state.mem[addr + 0x98].uint32_t = 0  # ds
-        state.mem[addr + 0x9c].uint32_t = state.regs.edi
-        state.mem[addr + 0xa0].uint32_t = state.regs.esi
-        state.mem[addr + 0xa4].uint32_t = state.regs.ebx
-        state.mem[addr + 0xa8].uint32_t = state.regs.edx
-        state.mem[addr + 0xac].uint32_t = state.regs.ecx
-        state.mem[addr + 0xb0].uint32_t = state.regs.eax
-        state.mem[addr + 0xb4].uint32_t = state.regs.ebp
-        state.mem[addr + 0xb8].uint32_t = state.regs.eip
-        state.mem[addr + 0xbc].uint32_t = 0  # cs
-        state.mem[addr + 0xc0].uint32_t = state.regs.eflags
-        state.mem[addr + 0xc4].uint32_t = state.regs.esp
-        state.mem[addr + 0xc8].uint32_t = 0  # ss
+        state.mem[addr + 0x9C].uint32_t = state.regs.edi
+        state.mem[addr + 0xA0].uint32_t = state.regs.esi
+        state.mem[addr + 0xA4].uint32_t = state.regs.ebx
+        state.mem[addr + 0xA8].uint32_t = state.regs.edx
+        state.mem[addr + 0xAC].uint32_t = state.regs.ecx
+        state.mem[addr + 0xB0].uint32_t = state.regs.eax
+        state.mem[addr + 0xB4].uint32_t = state.regs.ebp
+        state.mem[addr + 0xB8].uint32_t = state.regs.eip
+        state.mem[addr + 0xBC].uint32_t = 0  # cs
+        state.mem[addr + 0xC0].uint32_t = state.regs.eflags
+        state.mem[addr + 0xC4].uint32_t = state.regs.esp
+        state.mem[addr + 0xC8].uint32_t = 0  # ss
         # and then 512 bytes of extended registers
         # TOTAL SIZE: 0x2cc
 
     @staticmethod
     def _load_regs(state, addr):
-        if state.arch.name != 'X86':
+        if state.arch.name != "X86":
             raise SimUnsupportedError("I don't know how to work with struct CONTEXT outside of i386")
 
         # TODO: check contextflags to see what parts to deserialize
-        state.regs.gs = state.mem[addr + 0x8c].uint32_t.resolved[31:16]
+        state.regs.gs = state.mem[addr + 0x8C].uint32_t.resolved[31:16]
         state.regs.fs = state.mem[addr + 0x90].uint32_t.resolved[31:16]
 
-        state.regs.edi = state.mem[addr + 0x9c].uint32_t.resolved
-        state.regs.esi = state.mem[addr + 0xa0].uint32_t.resolved
-        state.regs.ebx = state.mem[addr + 0xa4].uint32_t.resolved
-        state.regs.edx = state.mem[addr + 0xa8].uint32_t.resolved
-        state.regs.ecx = state.mem[addr + 0xac].uint32_t.resolved
-        state.regs.eax = state.mem[addr + 0xb0].uint32_t.resolved
-        state.regs.ebp = state.mem[addr + 0xb4].uint32_t.resolved
-        state.regs.eip = state.mem[addr + 0xb8].uint32_t.resolved
-        state.regs.eflags = state.mem[addr + 0xc0].uint32_t.resolved
-        state.regs.esp = state.mem[addr + 0xc4].uint32_t.resolved
+        state.regs.edi = state.mem[addr + 0x9C].uint32_t.resolved
+        state.regs.esi = state.mem[addr + 0xA0].uint32_t.resolved
+        state.regs.ebx = state.mem[addr + 0xA4].uint32_t.resolved
+        state.regs.edx = state.mem[addr + 0xA8].uint32_t.resolved
+        state.regs.ecx = state.mem[addr + 0xAC].uint32_t.resolved
+        state.regs.eax = state.mem[addr + 0xB0].uint32_t.resolved
+        state.regs.ebp = state.mem[addr + 0xB4].uint32_t.resolved
+        state.regs.eip = state.mem[addr + 0xB8].uint32_t.resolved
+        state.regs.eflags = state.mem[addr + 0xC0].uint32_t.resolved
+        state.regs.esp = state.mem[addr + 0xC4].uint32_t.resolved
 
     def initialize_segment_register_x64(self, state, concrete_target):
         """
@@ -465,7 +479,7 @@ class SimWindows(SimOS):
         :param state:               state which will be modified
         :param concrete_target:     concrete target that will be used to read the fs register
         :return: None
-       """
+        """
         _l.debug("Synchronizing gs segment register")
         state.regs.gs = self._read_gs_register_x64(concrete_target)
 
@@ -479,18 +493,19 @@ class SimWindows(SimOS):
         """
         _l.debug("Creating Global Descriptor Table and synchronizing fs segment register")
         fs = self._read_fs_register_x86(concrete_target)
-        gdt = self.generate_gdt(fs,0x0)
-        self.setup_gdt(state,gdt)
+        gdt = self.generate_gdt(fs, 0x0)
+        self.setup_gdt(state, gdt)
         return gdt
 
     @staticmethod
     def _read_fs_register_x86(concrete_target):
-        '''
-        Injects small shellcode to leak the fs segment register address. In Windows x86 this address is pointed by gs:[0x18]
+        """
+        Injects small shellcode to leak the fs segment register address. In Windows x86 this address is pointed by
+        gs:[0x18]
         :param concrete_target: ConcreteTarget which will be used to get the fs register address
         :return: fs register address
         :rtype string
-        '''
+        """
         exfiltration_reg = "eax"
         # instruction to inject for reading the value at segment value = offset
         read_fs0_x86 = b"\x64\xA1\x18\x00\x00\x00\x90\x90\x90\x90"  # mov eax, fs:[0x18]
@@ -498,12 +513,13 @@ class SimWindows(SimOS):
 
     @staticmethod
     def _read_gs_register_x64(concrete_target):
-        '''
-        Injects small shellcode to leak the gs segment register address. In Windows x64 this address is pointed by gs:[0x30]
+        """
+        Injects small shellcode to leak the gs segment register address. In Windows x64 this address is pointed by
+        gs:[0x30]
         :param concrete_target: ConcreteTarget which will be used to get the fs register address
         :return: gs register address
         :rtype string
-        '''
+        """
         exfiltration_reg = "rax"
         # instruction to inject for reading the value at segment value = offset
         read_gs0_x64 = b"\x65\x48\x8B\x04\x25\x30\x00\x00\x00\x90\x90\x90\x90"  # mov rax, gs:[0x30]
@@ -512,33 +528,35 @@ class SimWindows(SimOS):
     def get_segment_register_name(self):
         if isinstance(self.arch, ArchAMD64):
             for register in self.arch.register_list:
-                if register.name == 'gs':
+                if register.name == "gs":
                     return register.vex_offset
         elif isinstance(self.arch, ArchX86):
             for register in self.arch.register_list:
-                if register.name == 'fs':
+                if register.name == "fs":
                     return register.vex_offset
         return None
 
     def _init_object_pe_security_cookie(self, pe_object, state, state_kwargs):
-        sc_init = state_kwargs.pop('security_cookie_init', SecurityCookieInit.STATIC)
+        sc_init = state_kwargs.pop("security_cookie_init", SecurityCookieInit.STATIC)
         if sc_init is SecurityCookieInit.NONE or sc_init is None:
             return
-        cookie = pe_object.load_config.get('SecurityCookie', None)
+        cookie = pe_object.load_config.get("SecurityCookie", None)
         if not cookie:
             return
         vs_cookie = VS_SECURITY_COOKIES.get(self.project.arch.name)
         if vs_cookie is None:
-            _l.warning('Unsupported architecture: %s for /GS, leaving _security_cookie uninitialized', self.project.arch.name)
+            _l.warning(
+                "Unsupported architecture: %s for /GS, leaving _security_cookie uninitialized", self.project.arch.name
+            )
             return
         if sc_init is SecurityCookieInit.RANDOM:
-            sc_value = random.randint(1, (2 ** vs_cookie.width - 1))
+            sc_value = random.randint(1, (2**vs_cookie.width - 1))
             if sc_value == vs_cookie.default:
                 sc_value += 1
         elif sc_init is SecurityCookieInit.STATIC:
-            sc_value = struct.unpack('>I', b'cook')[0]
+            sc_value = struct.unpack(">I", b"cook")[0]
         elif sc_init is SecurityCookieInit.SYMBOLIC:
-            sc_value = claripy.BVS('_security_cookie', state.arch.bits)
+            sc_value = claripy.BVS("_security_cookie", state.arch.bits)
         else:
-            raise TypeError("security_cookie_init must SecurityCookieInit, not {0}".format(type(sc_init).__name__))
-        setattr(state.mem[cookie], "uint{0}_t".format(state.arch.bits), sc_value)
+            raise TypeError(f"security_cookie_init must SecurityCookieInit, not {type(sc_init).__name__}")
+        setattr(state.mem[cookie], f"uint{state.arch.bits}_t", sc_value)
