@@ -1,6 +1,7 @@
 # This module contains data structures for handling memory, code, and register references.
 
 import logging
+
 l = logging.getLogger(name=__name__)
 
 _noneset = frozenset()
@@ -12,11 +13,12 @@ class SimAction(SimEvent):
     """
     A SimAction represents a semantic action that an analyzed program performs.
     """
-    #__slots__ = [ 'bbl_addr', 'inst_addr', 'stmt_idx' ]
 
-    TMP = 'tmp'
-    REG = 'reg'
-    MEM = 'mem'
+    # __slots__ = [ 'bbl_addr', 'inst_addr', 'stmt_idx' ]
+
+    TMP = "tmp"
+    REG = "reg"
+    MEM = "mem"
     _MAX_ACTION_ID = -1
 
     def __init__(self, state, region_type):
@@ -25,7 +27,7 @@ class SimAction(SimEvent):
 
         :param state: the state that's the SimAction is taking place in.
         """
-        SimEvent.__init__(self, state, 'action')
+        SimEvent.__init__(self, state, "action")
         self.type = region_type
         SimAction._MAX_ACTION_ID += 1
         self._action_id = SimAction._MAX_ACTION_ID
@@ -35,20 +37,14 @@ class SimAction(SimEvent):
             location = "%s()" % self.sim_procedure.display_name
         else:
             if self.stmt_idx is not None:
-                location = "0x%x:%d" % (self.bbl_addr, self.stmt_idx)
+                location = "0x%x:%d" % (self.ins_addr, self.stmt_idx)  # TODO: Revert this!
             else:
                 location = "0x%x" % self.bbl_addr
-        return "<%s %s %s>" % (self.__class__.__name__, location, self._desc())
+        return f"<{self.__class__.__name__} {location} {self._desc()}>"
 
     def _desc(self):
         raise NotImplementedError()
 
-    #def __getstate__(self):
-    #   return { k: getattr(self, k) for k in sum([ c.__slots__ for c in self.__class__.mro() if hasattr(c, '__slots__')], []) } #pylint:disable=no-member
-    #def __setstate__(self, s):
-    #   for k,v in s.items():
-    #       setattr(self, k, v)
-
     @staticmethod
     def _make_object(v):
         if v is None:
@@ -102,11 +98,11 @@ class SimActionExit(SimAction):
     An Exit action represents a (possibly conditional) jump.
     """
 
-    CONDITIONAL = 'conditional'
-    DEFAULT = 'default'
+    CONDITIONAL = "conditional"
+    DEFAULT = "default"
 
     def __init__(self, state, target, condition=None, exit_type=None):
-        super(SimActionExit, self).__init__(state, "exit")
+        super().__init__(state, "exit")
         if exit_type is not None:
             self.exit_type = exit_type
         elif condition is None:
@@ -122,7 +118,7 @@ class SimActionExit(SimAction):
 
     @property
     def all_objects(self):
-        return [ a for a in ( self.target, self.condition ) if a is not None ]
+        return [a for a in (self.target, self.condition) if a is not None]
 
     @property
     def is_symbolic(self):
@@ -140,14 +136,14 @@ class SimActionConstraint(SimAction):
     """
 
     def __init__(self, state, constraint, condition=None):
-        super(SimActionConstraint, self).__init__(state, "constraint")
+        super().__init__(state, "constraint")
 
         self.constraint = self._make_object(constraint)
         self.condition = self._make_object(condition)
 
     @property
     def all_objects(self):
-        return [ a for a in ( self.constraint, self.condition ) if a is not None ]
+        return [a for a in (self.constraint, self.condition) if a is not None]
 
     @property
     def is_symbolic(self):
@@ -158,9 +154,9 @@ class SimActionConstraint(SimAction):
         c.condition = self._copy_object(self.condition)
 
     def _desc(self):
-        s = '%s' % str(self.constraint)
+        s = "%s" % str(self.constraint)
         if self.condition is not None:
-            s += ' (cond)'
+            s += " (cond)"
         return s
 
 
@@ -170,7 +166,7 @@ class SimActionOperation(SimAction):
     """
 
     def __init__(self, state, op, exprs, result):
-        super(SimActionOperation, self).__init__(state, 'operation')
+        super().__init__(state, "operation")
 
         self.op = op
         self.exprs = exprs
@@ -179,11 +175,11 @@ class SimActionOperation(SimAction):
 
     @property
     def all_objects(self):
-        return [ ex for ex in self.exprs if isinstance(ex, SimActionObject) ]
+        return [ex for ex in self.exprs if isinstance(ex, SimActionObject)]
 
     @property
     def is_symbolic(self):
-        return any([ getattr(ex, "symbolic", False) for ex in self.exprs ])
+        return any(getattr(ex, "symbolic", False) for ex in self.exprs)
 
     def _copy_objects(self, c):
         c.op = self.op
@@ -198,29 +194,45 @@ class SimActionData(SimAction):
     """
     A Data action represents a read or a write from memory, registers or a file.
     """
-    #__slots__ = [ 'objects' ]
 
-    READ = 'read'
-    WRITE = 'write'
-    OPERATE = 'operate'
+    # __slots__ = [ 'objects' ]
 
-    def __init__(self, state, region_type, action, tmp=None, addr=None, size=None, data=None, condition=None,
-                 fallback=None, fd=None):
-        super(SimActionData, self).__init__(state, region_type)
+    READ = "read"
+    WRITE = "write"
+    OPERATE = "operate"
+
+    def __init__(
+        self,
+        state,
+        region_type,
+        action,
+        tmp=None,
+        addr=None,
+        size=None,
+        data=None,
+        condition=None,
+        fallback=None,
+        fd=None,
+    ):
+        super().__init__(state, region_type)
         self.action = action
 
-        self._reg_dep = _noneset if addr is None or action != SimActionData.READ or not isinstance(addr, int) else frozenset((addr,))
+        self._reg_dep = (
+            _noneset
+            if addr is None or action != SimActionData.READ or not isinstance(addr, int)
+            else frozenset((addr,))
+        )
         self._tmp_dep = _noneset if tmp is None or action != SimActionData.READ else frozenset((tmp,))
 
         self.tmp = tmp
         self.offset = None
-        if region_type == 'reg':
+        if region_type == "reg":
             if isinstance(addr, int):
                 self.offset = addr
             else:
                 if addr.symbolic:
                     # FIXME: we should fix it by allowing .offset taking ASTs instead of concretizing it right away
-                    l.warning('Concretizing a symbolic register offset in SimActionData.')
+                    l.warning("Concretizing a symbolic register offset in SimActionData.")
                     self.offset = state.solver.eval(addr)
                 else:
                     # it's not symbolic
@@ -247,19 +259,43 @@ class SimActionData(SimAction):
 
     @property
     def all_objects(self):
-        return [ a for a in [ self.addr, self.size, self.data, self.condition, self.fallback, self.fd ] if a is not None ]
+        return [a for a in [self.addr, self.size, self.data, self.condition, self.fallback, self.fd] if a is not None]
 
     @property
     def is_symbolic(self):
-        return any([ getattr(a, "symbolic", False) for a in [ self.addr, self.size, self.data ] if a is not None ])
+        return any(getattr(a, "symbolic", False) for a in [self.addr, self.size, self.data] if a is not None)
 
     @property
     def tmp_deps(self):
-        return super(SimActionData, self).tmp_deps | self._tmp_dep
+        return super().tmp_deps | self._tmp_dep
 
     @property
     def reg_deps(self):
-        return super(SimActionData, self).reg_deps | self._reg_dep
+        return super().reg_deps | self._reg_dep
+
+    @property
+    def storage(self):
+        def _repr(o):
+            if type(o) in {bytes, str, int}:
+                return o
+            try:
+                o = o.ast
+            except AttributeError:
+                pass
+            if type(o) in {bytes, str, int}:
+                return o
+            return o.shallow_repr()
+
+        if self.type == "reg":
+            _size = self.size.ast if isinstance(self.size, SimActionObject) else self.size
+            assert isinstance(_size, int)
+            storage = self.arch.register_size_names[(self.offset, _size // self.arch.byte_width)]
+        elif self.type == "tmp":
+            storage = f"tmp_{self.tmp}"
+        else:
+            storage = self.addr
+
+        return _repr(storage)
 
     def _desc(self):
         def _repr(o):
@@ -269,15 +305,20 @@ class SimActionData(SimAction):
                 o = o.ast
             except AttributeError:
                 pass
+            if type(o) in {bytes, str, int}:
+                return o
             return o.shallow_repr()
-        if self.type == 'reg':
-            _size = self.size.ast if isinstance(self.size, SimActionObject) else self.size
-            assert isinstance(_size, int)
-            storage = self.arch.register_size_names[(self.offset, _size // self.arch.byte_width)]
-        else:
-            storage = self.addr
-        direction = '<<----' if self.action == 'write' else '---->>'
-        return f"{self.type}/{self.action}: {_repr(storage)}  {direction}  {_repr(self.data)}"
+
+        # if self.type == 'reg':
+        #     _size = self.size.ast if isinstance(self.size, SimActionObject) else self.size
+        #     assert isinstance(_size, int)
+        #     storage = self.arch.register_size_names[(self.offset, _size // self.arch.byte_width)]
+        # elif self.type == 'tmp':
+        #     storage = f'tmp_{self.tmp}'
+        # else:
+        #     storage = self.addr
+        direction = "<<----" if self.action == "write" else "---->>"
+        return f"{self.type}/{self.action}: {self.storage}  {direction}  {_repr(self.data)}"
 
     def _copy_objects(self, c):
         c.action = self.action
@@ -289,4 +330,5 @@ class SimActionData(SimAction):
         c.fallback = self._copy_object(self.fallback)
         c.fd = self._copy_object(self.fd)
 
+
 from .sim_action_object import SimActionObject
