@@ -1,3 +1,4 @@
+from typing import TYPE_CHECKING
 import logging
 
 import claripy
@@ -5,12 +6,15 @@ from archinfo.arch_soot import ArchSoot, SootAddressDescriptor
 from archinfo.arch_arm import is_arm_arch
 from .plugin import SimStatePlugin
 
+if TYPE_CHECKING:
+    from angr.sim_type import SimType
+
 
 l = logging.getLogger(name=__name__)
 
 
 class SimRegNameView(SimStatePlugin):
-    def __getattr__(self, k):
+    def __getattr__(self, k) -> claripy.ast.base.Base:
         """
         Get the value of a register.
 
@@ -21,9 +25,9 @@ class SimRegNameView(SimStatePlugin):
         :rtype:       claripy.ast.Base
         """
 
-        state = super(SimRegNameView, self).__getattribute__('state') # type: SimState
+        state: SimState = super().__getattribute__("state")
 
-        if isinstance(k, str) and k.startswith('_'):
+        if isinstance(k, str) and k.startswith("_"):
             k = k[1:]
             inspect = False
             disable_actions = True
@@ -36,7 +40,7 @@ class SimRegNameView(SimStatePlugin):
         try:
             return state.registers.load(k, inspect=inspect, disable_actions=disable_actions, events=events)
         except KeyError:
-            return super(SimRegNameView, self).__getattribute__(k)
+            return super().__getattribute__(k)
 
     def __setattr__(self, k, v):
         """
@@ -48,10 +52,10 @@ class SimRegNameView(SimStatePlugin):
         :return:      None
         """
 
-        if k == 'state' or k in dir(SimStatePlugin):
+        if k == "state" or k in dir(SimStatePlugin):
             return object.__setattr__(self, k, v)
 
-        if isinstance(k, str) and k.startswith('_'):
+        if isinstance(k, str) and k.startswith("_"):
             k = k[1:]
             inspect = False
             disable_actions = True
@@ -63,10 +67,12 @@ class SimRegNameView(SimStatePlugin):
         # instruction pointer flag every time the ip gets written.
         # This flag will then toggle between the native and the java view of the
         # state.
-        if self.state.project and \
-           isinstance(self.state.project.arch, ArchSoot) and \
-           k == 'ip' and \
-           self.state.project.simos.is_javavm_with_jni_support:
+        if (
+            self.state.project
+            and isinstance(self.state.project.arch, ArchSoot)
+            and k == "ip"
+            and self.state.project.simos.is_javavm_with_jni_support
+        ):
             self.state.ip_is_soot_addr = isinstance(v, SootAddressDescriptor)
 
         try:
@@ -79,25 +85,31 @@ class SimRegNameView(SimStatePlugin):
                 raise AttributeError(k)
 
     def __dir__(self):
-        if self.state.arch.name in ('X86', 'AMD64'):
-            return list(self.state.arch.registers.keys()) + ['st%d' % n for n in range(8)] + ['tag%d' % n for n in range(8)] + ['flags', 'eflags', 'rflags']
+        if self.state.arch.name in ("X86", "AMD64"):
+            return (
+                list(self.state.arch.registers.keys())
+                + ["st%d" % n for n in range(8)]
+                + ["tag%d" % n for n in range(8)]
+                + ["flags", "eflags", "rflags"]
+            )
         elif is_arm_arch(self.state.arch):
-            return list(self.state.arch.registers.keys()) + ['flags']
+            return list(self.state.arch.registers.keys()) + ["flags"]
         return self.state.arch.registers.keys()
 
     @SimStatePlugin.memo
-    def copy(self, memo): # pylint: disable=unused-argument
+    def copy(self, memo):  # pylint: disable=unused-argument
         return SimRegNameView()
 
-    def merge(self, others, merge_conditions, common_ancestor=None): # pylint: disable=unused-argument
+    def merge(self, others, merge_conditions, common_ancestor=None):  # pylint: disable=unused-argument
         return False
 
-    def widen(self, others): # pylint: disable=unused-argument
+    def widen(self, others):  # pylint: disable=unused-argument
         return False
 
     def get(self, reg_name):
         return self.__getattr__(reg_name)
 
+
 class SimMemView(SimStatePlugin):
     """
     This is a convenient interface with which you can access a program's memory.
@@ -134,29 +146,29 @@ class SimMemView(SimStatePlugin):
     """
 
     def __init__(self, ty=None, addr=None, state=None):
-        super(SimMemView, self).__init__()
+        super().__init__()
         self._type = ty
         self._addr = addr
         if state is not None:
             self.set_state(state)
 
     def set_state(self, state):
-        super(SimMemView, self).set_state(state)
+        super().set_state(state)
 
         # Make sure self._addr is always an AST
         if isinstance(self._addr, int):
             self._addr = self.state.solver.BVV(self._addr, self.state.arch.bits)
 
-    def _deeper(self, **kwargs) -> 'SimMemView':
-        if 'ty' not in kwargs:
-            kwargs['ty'] = self._type
-        if 'addr' not in kwargs:
-            kwargs['addr'] = self._addr
-        if 'state' not in kwargs:
-            kwargs['state'] = self.state
+    def _deeper(self, **kwargs) -> "SimMemView":
+        if "ty" not in kwargs:
+            kwargs["ty"] = self._type
+        if "addr" not in kwargs:
+            kwargs["addr"] = self._addr
+        if "state" not in kwargs:
+            kwargs["state"] = self.state
         return SimMemView(**kwargs)
 
-    def __getitem__(self, k) -> 'SimMemView':
+    def __getitem__(self, k) -> "SimMemView":
         if isinstance(k, slice):
             if k.step is not None:
                 raise ValueError("Slices with strides are not supported")
@@ -180,51 +192,71 @@ class SimMemView(SimStatePlugin):
 
     def __repr__(self):
         if self._addr is None:
-            return '<SimMemView>'
-        value = '<unresolvable>' if not self.resolvable else self.resolved
+            return "<SimMemView>"
+        value = "<unresolvable>" if not self.resolvable else self.resolved
 
         if isinstance(self._addr, claripy.ast.Base):
             addr = self._addr.__repr__(inner=True)
-        elif hasattr(self._addr, 'ast') and isinstance(self._addr.ast, claripy.ast.Base):
+        elif hasattr(self._addr, "ast") and isinstance(self._addr.ast, claripy.ast.Base):
             addr = self._addr.ast.__repr__(inner=True)
         else:
             addr = repr(self._addr)
 
-        type_name = repr(self._type) if self._type is not None else '<untyped>'
-        return '<{} {} at {}>'.format(type_name,
-                                      value,
-                                      addr)
+        type_name = repr(self._type) if self._type is not None else "<untyped>"
+        return f"<{type_name} {value} at {addr}>"
 
     def __dir__(self):
-        return self._type._refine_dir() if self._type else [x for x in SimMemView.types if ' ' not in x] + ['struct']
+        return self._type._refine_dir() if self._type else [x for x in SimMemView.types if " " not in x] + ["struct"]
+
+    struct: "StructMode"
 
     def __getattr__(self, k):
-        if k in ('concrete', 'deref', 'resolvable', 'resolved', 'state', 'array', 'store', '_addr', '_type') or k in dir(SimStatePlugin):
+        if k in (
+            "concrete",
+            "deref",
+            "resolvable",
+            "resolved",
+            "state",
+            "array",
+            "store",
+            "_addr",
+            "_type",
+        ) or k in dir(SimStatePlugin):
             return object.__getattribute__(self, k)
         if self._type and k in self._type._refine_dir():
             return self._type._refine(self, k)
-        if k == 'struct':
+        if k == "struct":
             return StructMode(self)
         if k in SimMemView.types:
             return self._deeper(ty=SimMemView.types[k].with_arch(self.state.arch))
         raise AttributeError(k)
 
     def __setattr__(self, k, v):
-        if k in ('state', '_addr', '_type') or k in dir(SimStatePlugin):
+        if k in ("state", "_addr", "_type") or k in dir(SimStatePlugin):
             return object.__setattr__(self, k, v)
         return self.__getattr__(k).store(v)
 
     def __cmp__(self, other):
         raise ValueError("Trying to compare SimMemView is not what you want to do")
 
+    def with_type(self, sim_type: "SimType") -> "SimMemView":
+        """
+        Returns a copy of the SimMemView with a type.
+
+        :param sim_type:    The new type.
+        :returns:           The typed SimMemView copy.
+        """
+        ty = sim_type.with_arch(self.state.arch)
+        return self._deeper(ty=ty)
+
     @SimStatePlugin.memo
-    def copy(self, memo): # pylint: disable=unused-argument
+    def copy(self, memo):  # pylint: disable=unused-argument
         return SimMemView()
 
-    def merge(self, others, merge_conditions, common_ancestor=None): # pylint: disable=unused-argument
+    def merge(self, others, merge_conditions, common_ancestor=None):  # pylint: disable=unused-argument
         return False
 
-    def widen(self, others): # pylint: disable=unused-argument
+    def widen(self, others):  # pylint: disable=unused-argument
         return False
 
     @property
@@ -244,7 +276,7 @@ class SimMemView(SimStatePlugin):
         return self._type.extract(self.state, self._addr, True)
 
     @property
-    def deref(self):
+    def deref(self) -> "SimMemView":
         if self._addr is None:
             raise ValueError("Trying to dereference pointer without addr defined")
         ptr = self.state.memory.load(self._addr, self.state.arch.bytes, endness=self.state.arch.memory_endness)
@@ -254,13 +286,22 @@ class SimMemView(SimStatePlugin):
 
         return self._deeper(ty=self._type.pts_to if isinstance(self._type, SimTypePointer) else None, addr=ptr)
 
-    def array(self, n):
+    def array(self, n) -> "SimMemView":
         if self._addr is None:
             raise ValueError("Trying to produce array without specifying adddress")
         if self._type is None:
             raise ValueError("Trying to produce array without specifying type")
         return self._deeper(ty=SimTypeFixedSizeArray(self._type, n))
 
+    def member(self, member_name: str) -> "SimMemView":
+        """
+        If self is a struct and member_name is a member of the struct, return
+        that member element. Otherwise raise an exception.
+        """
+        if self._type and member_name in self._type._refine_dir():
+            return self._type._refine(self, member_name)
+        raise ValueError("Trying to find a struct member that cannot be found")
+
     def store(self, value):
         if self._addr is None:
             raise ValueError("Trying to store to location without specifying address")
@@ -271,27 +312,30 @@ class SimMemView(SimStatePlugin):
         return self._type.store(self.state, self._addr, value)
 
 
-class StructMode(object):
+class StructMode:
     def __init__(self, view):
         self._view = view
 
     def __dir__(self):
-        return [x[7:] for x in SimMemView.types if x.startswith('struct ')]
+        return [x[7:] for x in SimMemView.types if x.startswith("struct ")]
 
     def __getattr__(self, k):
-        assert k != '_view'
-        return self._view._deeper(ty=SimMemView.types['struct ' + k].with_arch(self._view.state.arch))
+        assert k != "_view"
+        return self._view._deeper(ty=SimMemView.types["struct " + k].with_arch(self._view.state.arch))
 
     def __setattr__(self, k, v):
-        if k == '_view':
+        if k == "_view":
             object.__setattr__(self, k, v)
         else:
             self.__getattr__(k).store(v)
 
+
 from ..sim_type import ALL_TYPES, SimTypeFixedSizeArray, SimTypePointer
-SimMemView.types = ALL_TYPES # identity purposefully here
+
+SimMemView.types = ALL_TYPES  # identity purposefully here
 
 
 from angr.sim_state import SimState
-SimState.register_default('mem', SimMemView)
-SimState.register_default('regs', SimRegNameView)
+
+SimState.register_default("mem", SimMemView)
+SimState.register_default("regs", SimRegNameView)
