@@ -1,10 +1,9 @@
 # misc
 
-DUMMY_SYMBOLIC_READ_VALUE = 0xc0deb4be
+DUMMY_SYMBOLIC_READ_VALUE = 0xC0DEB4BE
 
-#imports
+# imports
 
 from .file import SimFile
 from .memory_object import SimMemoryObject
 from .memory_mixins import MemoryMixin, DefaultMemory
-
