@@ -9,16 +9,16 @@ class AbstractAddressDescriptor:
     cardinality (the total number of addresses) without enumerating or creating all addresses in static mode.
     """
 
-    __slots__ = ('_regioned_addrs', )
+    __slots__ = ("_regioned_addrs",)
 
     def __init__(self):
-        self._regioned_addrs: List[Tuple[str,StridedInterval]] = [ ]
+        self._regioned_addrs: List[Tuple[str, StridedInterval]] = []
 
     def __len__(self) -> int:
         # this may raise an OverflowError if self.cardinality is greater than sys.maxint
         return self.cardinality
 
-    def __iter__(self) -> Generator[Tuple[str,StridedInterval],None,None]:
+    def __iter__(self) -> Generator[Tuple[str, StridedInterval], None, None]:
         yield from self._regioned_addrs
 
     @property
