@@ -7,9 +7,7 @@ l = logging.getLogger(name=__name__)
 
 
 class AbstractMergerMixin(MemoryMixin):
-
-    def _merge_values(self, values: Iterable[Tuple[Any,Any]], merged_size: int):
-
+    def _merge_values(self, values: Iterable[Tuple[Any, Any]], merged_size: int, **kwargs):
         # if self.category == 'reg' and self.state.arch.register_endness == 'Iend_LE':
         #     should_reverse = True
         # elif self.state.arch.memory_endness == 'Iend_LE':
@@ -41,4 +39,4 @@ class AbstractMergerMixin(MemoryMixin):
 
     @staticmethod
     def _is_uninitialized(a):
-        return getattr(a._model_vsa, 'uninitialized', False)
+        return getattr(a._model_vsa, "uninitialized", False)
