@@ -1,8 +1,7 @@
-
 from .. import MemoryMixin
 
 
 class RegionCategoryMixin(MemoryMixin):
     @property
     def category(self):
-        return 'mem'
+        return "mem"
