@@ -1,8 +1,9 @@
-
 from .timing import timethis
 from . import graph
 from . import constants
 from . import enums_conv
+from . import lazy_import
+from .env import is_pyinstaller
 
 
 def looks_like_sql(s: str) -> bool:
@@ -13,7 +14,7 @@ def looks_like_sql(s: str) -> bool:
     :return:        True if the string looks like an SQL, False otherwise.
     """
 
-    sql_keywords = {'select', 'update', 'union', 'delete', 'from', 'table', 'insert', 'into'}
+    sql_keywords = {"select", "update", "union", "delete", "from", "table", "insert", "into"}
 
     s = s.lower()
     for k in sql_keywords:
@@ -25,7 +26,7 @@ def looks_like_sql(s: str) -> bool:
                 if before not in " /;":
                     continue
             # what's after k? is it a whitespace?
-            following = s[k_index + len(k) : ]
+            following = s[k_index + len(k) :]
             if not following or following.startswith(" "):
                 return True
 
