@@ -1,14 +1,22 @@
+from collections import defaultdict
+from typing import Any, Dict, List, Callable, Optional, Generic, Type, TypeVar, Tuple, Set, TYPE_CHECKING, Union
+
 import networkx
 
-from functools import reduce
-
+from .visitors.graph import NodeType
+from ...sim_state import SimState
 from ...errors import AngrForwardAnalysisError
 from ...errors import AngrSkipJobNotice, AngrDelayJobNotice, AngrJobMergingFailureNotice, AngrJobWideningFailureNotice
+from ...utils.algo import binary_insert
+from .job_info import JobInfo, JobType, JobKey
+
+if TYPE_CHECKING:
+    from .visitors.graph import GraphVisitor
+
+AnalysisState = TypeVar("AnalysisState")
 
 
-from .job_info import JobInfo
-
-class ForwardAnalysis:
+class ForwardAnalysis(Generic[AnalysisState, NodeType, JobType, JobKey]):
     """
     This is my very first attempt to build a static forward analysis framework that can serve as the base of multiple
     static analyses in angr, including CFG analysis, VFG analysis, DDG, etc.
@@ -28,9 +36,14 @@ class ForwardAnalysis:
     Feel free to discuss with me (Fish) if you have any suggestions or complaints.
     """
 
-    def __init__(self, order_jobs=False, allow_merging=False, allow_widening=False, status_callback=None,
-                 graph_visitor=None
-                 ):
+    def __init__(
+        self,
+        order_jobs=False,
+        allow_merging=False,
+        allow_widening=False,
+        status_callback: Optional[Callable[[Type["ForwardAnalysis"]], Any]] = None,
+        graph_visitor: "Optional[GraphVisitor[NodeType]]" = None,
+    ):
         """
         Constructor
 
@@ -50,19 +63,21 @@ class ForwardAnalysis:
 
         # sanity checks
         if self._allow_widening and not self._allow_merging:
-            raise AngrForwardAnalysisError('Merging must be allowed if widening is allowed.')
+            raise AngrForwardAnalysisError("Merging must be allowed if widening is allowed.")
 
         # Analysis progress control
         self._should_abort = False
 
         # All remaining jobs
-        self._job_info_queue = [ ]
+        self._job_info_queue: List[JobInfo[JobType, JobKey]] = []
 
         # A map between job key to job. Jobs with the same key will be merged by calling _merge_jobs()
-        self._job_map = { }
+        self._job_map: Dict[JobKey, JobInfo[JobType, JobKey]] = {}
 
-        # A mapping between node and abstract state
-        self._state_map = { }
+        # A mapping between node and its input states
+        self._input_states: Dict[NodeType, List[AnalysisState]] = defaultdict(list)
+        # A mapping between node and its output state
+        self._output_state: Dict[NodeType, AnalysisState] = {}
 
         # The graph!
         # Analysis results (nodes) are stored here
@@ -82,7 +97,7 @@ class ForwardAnalysis:
         return self._should_abort
 
     @property
-    def graph(self):
+    def graph(self) -> networkx.DiGraph:
         return self._graph
 
     @property
@@ -102,7 +117,7 @@ class ForwardAnalysis:
 
         self._should_abort = True
 
-    def has_job(self, job):
+    def has_job(self, job: JobType) -> bool:
         """
         Checks whether there exists another job which has the same job key.
         :param job: The job to check.
@@ -112,43 +127,59 @@ class ForwardAnalysis:
         job_key = self._job_key(job)
         return job_key in self._job_map
 
+    def downsize(self):
+        self._input_states = defaultdict(list)
+        self._output_state = {}
+
     #
     # Abstract interfaces
     #
 
     # Common interfaces
 
-    def _pre_analysis(self):
-        raise NotImplementedError('_pre_analysis() is not implemented.')
+    def _pre_analysis(self) -> None:
+        raise NotImplementedError("_pre_analysis() is not implemented.")
 
-    def _intra_analysis(self):
-        raise NotImplementedError('_intra_analysis() is not implemented.')
+    def _intra_analysis(self) -> None:
+        raise NotImplementedError("_intra_analysis() is not implemented.")
 
-    def _post_analysis(self):
-        raise NotImplementedError('_post_analysis() is not implemented.')
+    def _post_analysis(self) -> None:
+        raise NotImplementedError("_post_analysis() is not implemented.")
 
-    def _job_key(self, job):
-        raise NotImplementedError('_job_key() is not implemented.')
+    def _job_key(self, job: JobType) -> JobKey:
+        raise NotImplementedError("_job_key() is not implemented.")
 
-    def _get_successors(self, job):
-        raise NotImplementedError('_get_successors() is not implemented.')
+    def _get_successors(self, job: JobType) -> Union[List[SimState], List[JobType]]:
+        raise NotImplementedError("_get_successors() is not implemented.")
 
-    def _pre_job_handling(self, job):
-        raise NotImplementedError('_pre_job_handling() is not implemented.')
+    def _pre_job_handling(self, job: JobType) -> None:
+        raise NotImplementedError("_pre_job_handling() is not implemented.")
 
-    def _post_job_handling(self, job, new_jobs, successors):
-        raise NotImplementedError('_post_job_handling() is not implemented.')
+    def _post_job_handling(self, job: JobType, new_jobs, successors: List[SimState]) -> None:
+        raise NotImplementedError("_post_job_handling() is not implemented.")
 
-    def _handle_successor(self, job, successor, successors):
-        raise NotImplementedError('_handle_successor() is not implemented.')
+    def _handle_successor(self, job: JobType, successor: SimState, successors: List[SimState]) -> List[JobType]:
+        raise NotImplementedError("_handle_successor() is not implemented.")
 
-    def _job_queue_empty(self):
-        raise NotImplementedError('_job_queue_empty() is not implemented.')
+    def _job_queue_empty(self) -> None:
+        raise NotImplementedError("_job_queue_empty() is not implemented.")
 
-    def _initial_abstract_state(self, node):
-        raise NotImplementedError('_initial_abstract_state() is not implemented.')
+    def _initial_abstract_state(self, node: NodeType) -> AnalysisState:
+        raise NotImplementedError("_initial_abstract_state() is not implemented.")
 
-    def _run_on_node(self, node, state):
+    def _process_input_state_for_successor(
+        self, node: NodeType, successor: NodeType, input_state: AnalysisState
+    ) -> AnalysisState:
+        # can be overwritten
+        return input_state
+
+    def _node_key(self, node: NodeType) -> NodeType:  # pylint:disable=no-self-use
+        """
+        Override this method if hash(node) is slow for the type of node that are in use.
+        """
+        return node
+
+    def _run_on_node(self, node: NodeType, state: AnalysisState) -> Tuple[bool, AnalysisState]:
         """
         The analysis routine that runs on each node in the graph.
 
@@ -164,9 +195,9 @@ class ForwardAnalysis:
         :return:        A tuple: (changed, output abstract state)
         """
 
-        raise NotImplementedError('_run_on_node() is not implemented.')
+        raise NotImplementedError("_run_on_node() is not implemented.")
 
-    def _merge_states(self, node, *states):
+    def _merge_states(self, node: NodeType, *states: AnalysisState) -> Tuple[AnalysisState, bool]:
         """
         Merge multiple abstract states into one.
 
@@ -176,30 +207,30 @@ class ForwardAnalysis:
                        i.e., union(state0, state1) == state0), in which case, its successors will not be revisited.
         """
 
-        raise NotImplementedError('_merge_states() is not implemented.')
+        raise NotImplementedError("_merge_states() is not implemented.")
 
-    def _widen_states(self, *states):
-        raise NotImplementedError('_widen_states() is not implemented.')
+    def _widen_states(self, *states: AnalysisState) -> AnalysisState:
+        raise NotImplementedError("_widen_states() is not implemented.")
 
     # Special interfaces for non-graph-traversal mode
 
-    def _merge_jobs(self, *jobs):
-        raise NotImplementedError('_merge_jobs() is not implemented.')
+    def _merge_jobs(self, *jobs: JobType):
+        raise NotImplementedError("_merge_jobs() is not implemented.")
 
-    def _should_widen_jobs(self, *jobs):
-        raise NotImplementedError('_should_widen_jobs() is not implemented.')
+    def _should_widen_jobs(self, *jobs: JobType):
+        raise NotImplementedError("_should_widen_jobs() is not implemented.")
 
-    def _widen_jobs(self, *jobs):
-        raise NotImplementedError('_widen_jobs() is not implemented.')
+    def _widen_jobs(self, *jobs: JobType):
+        raise NotImplementedError("_widen_jobs() is not implemented.")
 
-    def _job_sorting_key(self, job):
-        raise NotImplementedError('_job_sorting_key() is not implemented.')
+    def _job_sorting_key(self, job: JobType) -> int:
+        raise NotImplementedError("_job_sorting_key() is not implemented.")
 
     #
     # Private methods
     #
 
-    def _analyze(self):
+    def _analyze(self) -> None:
         """
         The main analysis routine.
 
@@ -222,72 +253,74 @@ class ForwardAnalysis:
 
         self._post_analysis()
 
-    def _analysis_core_graph(self):
-
+    def _analysis_core_graph(self) -> None:
         while not self.should_abort:
-
             self._intra_analysis()
 
-            n = self._graph_visitor.next_node()
+            n: NodeType = self._graph_visitor.next_node()
 
             if n is None:
                 break
 
-            job_state = self._get_input_state(n)
+            job_state = self._get_and_update_input_state(n)
             if job_state is None:
                 job_state = self._initial_abstract_state(n)
 
             changed, output_state = self._run_on_node(n, job_state)
 
-            # output state of node n is input state for successors to node n
-            successors_to_visit = self._add_input_state(n, output_state)
-
             if changed is False:
                 # no change is detected
+                self._output_state[self._node_key(n)] = output_state
                 continue
-            elif changed is True:
+            if changed is True:
                 # changes detected
+
+                # output state of node n is input state for successors to node n
+                self._add_input_state(n, output_state)
+
                 # revisit all its successors
                 self._graph_visitor.revisit_successors(n, include_self=False)
             else:
                 # the change of states are determined during state merging (_add_input_state()) instead of during
                 # simulated execution (_run_on_node()).
-                # revisit all successors in the `successors_to_visit` list
-                for succ in successors_to_visit:
-                    self._graph_visitor.revisit_node(succ)
 
-    def _add_input_state(self, node, input_state):
+                if self._node_key(n) not in self._output_state:
+                    reached_fixedpoint = False
+                else:
+                    # is the output state the same as the old one?
+                    _, reached_fixedpoint = self._merge_states(n, self._output_state[self._node_key(n)], output_state)
+                self._output_state[self._node_key(n)] = output_state
+
+                if not reached_fixedpoint:
+                    successors_to_visit = self._add_input_state(n, output_state)
+                    # revisit all successors in the `successors_to_visit` list
+                    for succ in successors_to_visit:
+                        self._graph_visitor.revisit_node(succ)
+
+    def _add_input_state(self, node: NodeType, input_state: AnalysisState) -> Set[NodeType]:
         """
         Add the input state to all successors of the given node.
 
         :param node:        The node whose successors' input states will be touched.
         :param input_state: The state that will be added to successors of the node.
-        :return:            None
         """
 
-        successors = self._graph_visitor.successors(node)
-        successors_to_visit = set()  # a collection of successors whose input states did not reach a fixed point
+        successors = set(self._graph_visitor.successors(node))
+        # successors_to_visit = set()  # a collection of successors whose input states did not reach a fixed point
 
         for succ in successors:
-            if succ in self._state_map:
-                to_merge = [ self._state_map[succ], input_state ]
-                r = self._merge_states(succ, *to_merge)
-                if type(r) is tuple and len(r) == 2:
-                    merged_state, reached_fixedpoint = r
-                else:
-                    # compatibility concerns
-                    merged_state, reached_fixedpoint = r, False
-                self._state_map[succ] = merged_state
+            # if a node has only one predecessor, we overwrite existing input states
+            # otherwise, we add the state as a new input state
+            # this is an approximation for removing input states for all nodes that `node` dominates
+            new_input_state = self._process_input_state_for_successor(node, succ, input_state)
+            if sum(1 for _ in self._graph_visitor.predecessors(succ)) == 1:
+                self._input_states[self._node_key(succ)] = [new_input_state]
             else:
-                self._state_map[succ] = input_state
-                reached_fixedpoint = False
+                self._input_states[self._node_key(succ)].append(new_input_state)
 
-            if not reached_fixedpoint:
-                successors_to_visit.add(succ)
+        return successors
 
-        return successors_to_visit
-
-    def _pop_input_state(self, node):
+    def _get_and_update_input_state(self, node: NodeType) -> Optional[AnalysisState]:
         """
         Get the input abstract state for this node, and remove it from the state map.
 
@@ -295,11 +328,13 @@ class ForwardAnalysis:
         :return:     A merged state, or None if there is no input state for this node available.
         """
 
-        if node in self._state_map:
-            return self._state_map.pop(node)
+        if self._node_key(node) in self._input_states:
+            input_state = self._get_input_state(node)
+            self._input_states[self._node_key(node)] = [input_state]
+            return input_state
         return None
 
-    def _get_input_state(self, node):
+    def _get_input_state(self, node: NodeType) -> Optional[AnalysisState]:
         """
         Get the input abstract state for this node.
 
@@ -307,32 +342,20 @@ class ForwardAnalysis:
         :return:        A merged state, or None if there is no input state for this node available.
         """
 
-        return self._state_map.get(node, None)
-
-    def _merge_state_from_predecessors(self, node):
-        """
-        Get abstract states for all predecessors of the node, merge them, and return the merged state.
-
-        :param node: The node in graph.
-        :return:     A merged state, or None if no predecessor is available.
-        """
-
-        preds = self._graph_visitor.predecessors(node)
-
-        states = [ self._state_map[n] for n in preds if n in self._state_map ]
-
-        if not states:
+        if self._node_key(node) not in self._input_states:
             return None
 
-        return reduce(lambda s0, s1: self._merge_states(node, s0, s1), states[1:], states[0])
-
-    def _analysis_core_baremetal(self):
+        all_input_states = self._input_states.get(self._node_key(node))
+        if len(all_input_states) == 1:
+            return all_input_states[0]
+        merged_state, _ = self._merge_states(node, *all_input_states)
+        return merged_state
 
+    def _analysis_core_baremetal(self) -> None:
         if not self._job_info_queue:
             self._job_queue_empty()
 
         while not self.should_abort:
-
             if self._status_callback is not None:
                 self._status_callback(self)
 
@@ -356,14 +379,13 @@ class ForwardAnalysis:
                 continue
             except AngrSkipJobNotice:
                 # consume and skip this job
-                self._job_info_queue = self._job_info_queue[1:]
+                self._job_info_queue.pop(0)
                 self._job_map.pop(self._job_key(job_info.job), None)
                 continue
 
             # remove the job info from the map
             self._job_map.pop(self._job_key(job_info.job), None)
-
-            self._job_info_queue = self._job_info_queue[1:]
+            self._job_info_queue.pop(0)
 
             self._process_job_and_get_successors(job_info)
 
@@ -373,7 +395,7 @@ class ForwardAnalysis:
 
             self._intra_analysis()
 
-    def _process_job_and_get_successors(self, job_info):
+    def _process_job_and_get_successors(self, job_info: JobInfo[JobType, JobKey]) -> None:
         """
         Process a job, get all successors of this job, and call _handle_successor() to handle each successor.
 
@@ -385,7 +407,7 @@ class ForwardAnalysis:
 
         successors = self._get_successors(job)
 
-        all_new_jobs = [ ]
+        all_new_jobs = []
 
         for successor in successors:
             new_jobs = self._handle_successor(job, successor, successors)
@@ -398,7 +420,7 @@ class ForwardAnalysis:
 
         self._post_job_handling(job, all_new_jobs, successors)
 
-    def _insert_job(self, job):
+    def _insert_job(self, job: JobType) -> None:
         """
         Insert a new job into the job queue. If the job queue is ordered, this job will be inserted at the correct
         position.
@@ -452,12 +474,12 @@ class ForwardAnalysis:
             self._job_map[key] = job_info
 
         if self._order_jobs:
-            self._binary_insert(self._job_info_queue, job_info, lambda elem: self._job_sorting_key(elem.job))
+            binary_insert(self._job_info_queue, job_info, lambda elem: self._job_sorting_key(elem.job))
 
         else:
             self._job_info_queue.append(job_info)
 
-    def _peek_job(self, pos):
+    def _peek_job(self, pos: int) -> JobType:
         """
         Return the job currently at position `pos`, but still keep it in the job queue. An IndexError will be raised
         if that position does not currently exist in the job list.
@@ -471,37 +493,20 @@ class ForwardAnalysis:
 
         raise IndexError()
 
-    #
-    # Utils
-    #
-
-    @staticmethod
-    def _binary_insert(lst, elem, key, lo=0, hi=None):
+    def _remove_job(self, predicate: Callable) -> None:
         """
-        Insert an element into a sorted list, and keep the list sorted.
+        Remove jobs that satisfy certain criteria from job_info_queue.
 
-        The major difference from bisect.bisect_left is that this function supports a key method, so user doesn't have
-        to create the key array for each insertion.
-
-        :param list lst: The list. Must be pre-ordered.
-        :param object element: An element to insert into the list.
-        :param func key: A method to get the key for each element in the list.
-        :param int lo: Lower bound of the search.
-        :param int hi: Upper bound of the search.
-        :return: None
+        :param predicate:   A method that determines if a job should be removed or not.
         """
 
-        if lo < 0:
-            raise ValueError("lo must be a non-negative number")
+        to_remove = []
+        for job_info in self._job_info_queue:
+            if predicate(job_info.job):
+                to_remove.append(job_info)
 
-        if hi is None:
-            hi = len(lst)
-
-        while lo < hi:
-            mid = (lo + hi) // 2
-            if key(lst[mid]) < key(elem):
-                lo = mid + 1
-            else:
-                hi = mid
-
-        lst.insert(lo, elem)
+        for job_info in to_remove:
+            self._job_info_queue.remove(job_info)
+            key = self._job_key(job_info.job)
+            if key in self._job_map:
+                del self._job_map[key]
