@@ -1,4 +1,3 @@
-
 import operator
 
 from archinfo.arch_soot import SootNullConstant
@@ -14,10 +13,11 @@ class SimSootExpr_Condition(SimSootExpr):
         v2 = self._translate_expr(self.expr.value2)
         operator_func = SimSootExpr_Condition.condition_str_to_function[self.expr.op]
         if isinstance(v1.expr, SimSootValue_ThisRef) and isinstance(v2.expr, SootNullConstant) and v1.expr.symbolic:
-            #TODO fix this
-            self.expr = operator_func(claripy.BVS('obf_ref', 32), claripy.BVV(0, 32))
-        elif isinstance(v1.expr, (SootNullConstant, SimSootValue_StringRef)) or \
-             isinstance(v2.expr, (SootNullConstant, SimSootValue_StringRef)):
+            # TODO fix this
+            self.expr = operator_func(claripy.BVS("obf_ref", 32), claripy.BVV(0, 32))
+        elif isinstance(v1.expr, (SootNullConstant, SimSootValue_StringRef)) or isinstance(
+            v2.expr, (SootNullConstant, SimSootValue_StringRef)
+        ):
             self.expr = claripy.true if operator_func(v1.expr, v2.expr) else claripy.false
         else:
             self.expr = operator_func(v1.expr, v2.expr)
