@@ -1,10 +1,8 @@
-
 from .base import SimSootValue
 
 
 class SimSootValue_StringRef(SimSootValue):
-
-    __slots__ = [ 'id', 'type' ]
+    __slots__ = ["id", "type"]
 
     def __init__(self, heap_alloc_id):
         self.id = "%s.string" % heap_alloc_id
@@ -14,9 +12,7 @@ class SimSootValue_StringRef(SimSootValue):
         return self.id
 
     def __eq__(self, other):
-        return isinstance(other, SimSootValue_StringRef) and \
-               self.id == other.id and \
-               self.type == other.type
+        return isinstance(other, SimSootValue_StringRef) and self.id == other.id and self.type == other.type
 
     @classmethod
     def from_sootvalue(cls, soot_value, state):
@@ -39,4 +35,3 @@ class SimSootValue_StringRef(SimSootValue):
         str_ref = SimSootValue_StringRef(state.memory.get_new_uuid())
         state.memory.store(str_ref, value)
         return str_ref
-
