@@ -1,18 +1,28 @@
-
+from archinfo.arch_arm import is_arm_arch
 import cle
 
 from .plugin import KnowledgeBasePlugin
 
 
 class Labels(KnowledgeBasePlugin):
-
     def __init__(self, kb):
         self._kb = kb
         self._labels = {}
         self._reverse_labels = {}
+
+        is_arm = is_arm_arch(kb._project.arch)
         for obj in kb._project.loader.all_objects:
             for v in obj.symbols:
-                if v.name and not v.is_import and v.type not in {cle.SymbolType.TYPE_OTHER, }:
+                if is_arm and v.name in {"$d", "$t", "$a"}:
+                    continue
+                if (
+                    v.name
+                    and not v.is_import
+                    and v.type
+                    not in {
+                        cle.SymbolType.TYPE_OTHER,
+                    }
+                ):
                     self._labels[v.rebased_addr] = v.name
                     self._reverse_labels[v.name] = v.rebased_addr
             try:
@@ -54,6 +64,9 @@ class Labels(KnowledgeBasePlugin):
     def __contains__(self, k):
         return k in self._labels
 
+    def items(self):
+        return self._labels.items()
+
     def get(self, addr):
         """
         Get a label as string for a given address
@@ -93,4 +106,4 @@ class Labels(KnowledgeBasePlugin):
             i += 1
 
 
-KnowledgeBasePlugin.register_default('labels', Labels)
+KnowledgeBasePlugin.register_default("labels", Labels)
