@@ -1,9 +1,10 @@
 import angr
 import time as _time
 
+
 class time(angr.SimProcedure):
-    #pylint: disable=arguments-differ
-    KEY = 'sys_last_time'
+    # pylint: disable=arguments-differ
+    KEY = "sys_last_time"
 
     @property
     def last_time(self):
@@ -14,11 +15,12 @@ class time(angr.SimProcedure):
         self.state.globals[self.KEY] = v
 
     def run(self, pointer):
+        # TODO lord have mercy. how big is time_t?
         if angr.options.USE_SYSTEM_TIMES in self.state.options:
             ts = int(_time.time())
             result = self.state.solver.BVV(ts, self.state.arch.bits)
         else:
-            result = self.state.solver.BVS('sys_time', self.state.arch.bits, key=('api', 'time'))
+            result = self.state.solver.BVS("sys_time", self.state.arch.bits, key=("api", "time"))
             if self.last_time is not None:
                 self.state.add_constraints(result.SGE(self.last_time))
             else:
