@@ -1,5 +1,6 @@
 import angr
 
+
 class UserHook(angr.SimProcedure):
     NO_RET = True
 
@@ -7,8 +8,8 @@ class UserHook(angr.SimProcedure):
     def run(self, user_func=None, length=None):
         result = user_func(self.state)
         if result is None:
-            jumpkind = 'Ijk_NoHook' if length == 0 else 'Ijk_Boring'
-            self.successors.add_successor(self.state, self.state.addr+length, self.state.solver.true, jumpkind)
+            jumpkind = "Ijk_NoHook" if length == 0 else "Ijk_Boring"
+            self.successors.add_successor(self.state, self.state.addr + length, self.state.solver.true, jumpkind)
         else:
             for state in result:
                 self.successors.add_successor(state, state.addr, state.scratch.guard, state.history.jumpkind)
