@@ -6,6 +6,7 @@ import logging
 
 l = logging.getLogger("angr.state_plugins.heap.heap_freelist")
 
+
 class Chunk:
     """
     The sort of chunk as would typically be found in a freelist-style heap implementation. Provides a representation of
@@ -32,58 +33,49 @@ class Chunk:
         """
         Returns the actual size of a chunk (as opposed to the entire size field, which may include some flags).
         """
-        raise NotImplementedError("%s not implemented for %s" % (self.get_size.__func__.__name__,
-                                                                 self.__class__.__name__))
+        raise NotImplementedError(f"{self.get_size.__func__.__name__} not implemented for {self.__class__.__name__}")
 
     def get_data_size(self):
         """
         Returns the size of the data portion of a chunk.
         """
-        raise NotImplementedError("%s not implemented for %s" % (self.get_size.__func__.__name__,
-                                                                 self.__class__.__name__))
-
+        raise NotImplementedError(f"{self.get_size.__func__.__name__} not implemented for {self.__class__.__name__}")
 
     def set_size(self, size):
         """
         Sets the size of the chunk, preserving any flags.
         """
-        raise NotImplementedError("%s not implemented for %s" % (self.set_size.__func__.__name__,
-                                                                 self.__class__.__name__))
+        raise NotImplementedError(f"{self.set_size.__func__.__name__} not implemented for {self.__class__.__name__}")
 
     def data_ptr(self):
         """
         Returns the address of the payload of the chunk.
         """
-        raise NotImplementedError("%s not implemented for %s" % (self.data_ptr.__func__.__name__,
-                                                                 self.__class__.__name__))
+        raise NotImplementedError(f"{self.data_ptr.__func__.__name__} not implemented for {self.__class__.__name__}")
 
     def is_free(self):
         """
         Returns a concrete determination as to whether the chunk is free.
         """
-        raise NotImplementedError("%s not implemented for %s" % (self.is_free.__func__.__name__,
-                                                                 self.__class__.__name__))
+        raise NotImplementedError(f"{self.is_free.__func__.__name__} not implemented for {self.__class__.__name__}")
 
     def next_chunk(self):
         """
         Returns the chunk immediately following (and adjacent to) this one.
         """
-        raise NotImplementedError("%s not implemented for %s" % (self.next_chunk.__func__.__name__,
-                                                                 self.__class__.__name__))
+        raise NotImplementedError(f"{self.next_chunk.__func__.__name__} not implemented for {self.__class__.__name__}")
 
     def prev_chunk(self):
         """
         Returns the chunk immediately prior (and adjacent) to this one.
         """
-        raise NotImplementedError("%s not implemented for %s" % (self.prev_chunk.__func__.__name__,
-                                                                 self.__class__.__name__))
+        raise NotImplementedError(f"{self.prev_chunk.__func__.__name__} not implemented for {self.__class__.__name__}")
 
     def fwd_chunk(self):
         """
         Returns the chunk following this chunk in the list of free chunks.
         """
-        raise NotImplementedError("%s not implemented for %s" % (self.fwd_chunk.__func__.__name__,
-                                                                 self.__class__.__name__))
+        raise NotImplementedError(f"{self.fwd_chunk.__func__.__name__} not implemented for {self.__class__.__name__}")
 
     def set_fwd_chunk(self, fwd):
         """
@@ -91,15 +83,15 @@ class Chunk:
 
         :param fwd: the chunk to follow this chunk in the list of free chunks
         """
-        raise NotImplementedError("%s not implemented for %s" % (self.set_fwd_chunk.__func__.__name__,
-                                                                 self.__class__.__name__))
+        raise NotImplementedError(
+            f"{self.set_fwd_chunk.__func__.__name__} not implemented for {self.__class__.__name__}"
+        )
 
     def bck_chunk(self):
         """
         Returns the chunk backward from this chunk in the list of free chunks.
         """
-        raise NotImplementedError("%s not implemented for %s" % (self.bck_chunk.__func__.__name__,
-                                                                 self.__class__.__name__))
+        raise NotImplementedError(f"{self.bck_chunk.__func__.__name__} not implemented for {self.__class__.__name__}")
 
     def set_bck_chunk(self, bck):
         """
@@ -107,8 +99,9 @@ class Chunk:
 
         :param bck: the chunk to precede this chunk in the list of free chunks
         """
-        raise NotImplementedError("%s not implemented for %s" % (self.set_bck_chunk.__func__.__name__,
-                                                                 self.__class__.__name__))
+        raise NotImplementedError(
+            f"{self.set_bck_chunk.__func__.__name__} not implemented for {self.__class__.__name__}"
+        )
 
     def _compare(self, other, comparison):
         if self.state is other.state:
@@ -153,7 +146,8 @@ class Chunk:
         return self._compare(other, self.base >= other.base)
 
     def __repr__(self):
-        return "<%s (%s @ 0x%x)>" % (self.__class__.__name__, "free" if self.is_free() else "used", self.base)
+        return "<{} ({} @ 0x{:x})>".format(self.__class__.__name__, "free" if self.is_free() else "used", self.base)
+
 
 class SimHeapFreelist(SimHeapLibc):
     """
@@ -168,22 +162,21 @@ class SimHeapFreelist(SimHeapLibc):
         """
         Returns an iterator over all the chunks in the heap.
         """
-        raise NotImplementedError("%s not implemented for %s" % (self.chunks.__func__.__name__,
-                                                                 self.__class__.__name__))
+        raise NotImplementedError(f"{self.chunks.__func__.__name__} not implemented for {self.__class__.__name__}")
 
     def allocated_chunks(self):
         """
         Returns an iterator over all the allocated chunks in the heap.
         """
-        raise NotImplementedError("%s not implemented for %s" % (self.allocated_chunks.__func__.__name__,
-                                                                 self.__class__.__name__))
+        raise NotImplementedError(
+            f"{self.allocated_chunks.__func__.__name__} not implemented for {self.__class__.__name__}"
+        )
 
     def free_chunks(self):
         """
         Returns an iterator over all the free chunks in the heap.
         """
-        raise NotImplementedError("%s not implemented for %s" % (self.free_chunks.__func__.__name__,
-                                                                 self.__class__.__name__))
+        raise NotImplementedError(f"{self.free_chunks.__func__.__name__} not implemented for {self.__class__.__name__}")
 
     def chunk_from_mem(self, ptr):
         """
@@ -192,25 +185,26 @@ class SimHeapFreelist(SimHeapLibc):
         :param ptr: a pointer to the base of a user payload in the heap
         :returns: the associated heap chunk
         """
-        raise NotImplementedError("%s not implemented for %s" % (self.chunk_from_mem.__func__.__name__,
-                                                                 self.__class__.__name__))
+        raise NotImplementedError(
+            f"{self.chunk_from_mem.__func__.__name__} not implemented for {self.__class__.__name__}"
+        )
 
     def print_heap_state(self):
-        print("┌───────────────────────────────┐")
-        print("├───────── HEAP CHUNKS ─────────┤")
+        print("|-------------------------------|")
+        print("|--------- HEAP CHUNKS ---------|")
         for ck in self.chunks():
             print("│ " + str(ck) + " │")
-        print("├───────── USED CHUNKS ─────────┤")
+        print("|--------- USED CHUNKS ---------|")
         for ck in self.allocated_chunks():
             print("│ " + str(ck) + " │")
-        print("├───────── FREE CHUNKS ─────────┤")
+        print("|--------- FREE CHUNKS ---------|")
         for ck in self.free_chunks():
             print("│ " + str(ck) + " │")
-        print("└───────────────────────────────┘")
+        print("|-------------------------------|")
 
     def print_all_chunks(self):
-        print("┌───────────────────────────────┐")
-        print("├───────── HEAP CHUNKS ─────────┤")
+        print("|-------------------------------|")
+        print("|--------- HEAP CHUNKS ---------|")
         for ck in self.chunks():
             print("│ " + str(ck) + " │")
-        print("└───────────────────────────────┘")
+        print("|-------------------------------|")
