@@ -26,11 +26,12 @@ class SimStateHistory(SimStatePlugin):
 
         # attributes handling the progeny of this history object
         self.parent = parent if clone is None else clone.parent
-        self.merged_from = [ ] if clone is None else list(clone.merged_from)
-        self.merge_conditions = [ ] if clone is None else list(clone.merge_conditions)
+        self.merged_from = [] if clone is None else list(clone.merged_from)
+        self.merge_conditions = [] if clone is None else list(clone.merge_conditions)
         self.depth = (0 if parent is None else parent.depth + 1) if clone is None else clone.depth
-        self.previous_block_count = (0 if parent is None else parent.block_count) if clone is None else \
-            clone.previous_block_count
+        self.previous_block_count = (
+            (0 if parent is None else parent.block_count) if clone is None else clone.previous_block_count
+        )
 
         # a string description of this history
         self.recent_description = None if clone is None else clone.recent_description
@@ -39,14 +40,14 @@ class SimStateHistory(SimStatePlugin):
         self.jump_target = None if clone is None else clone.jump_target
         self.jump_source = None if clone is None else clone.jump_source
         self.jump_avoidable = None if clone is None else clone.jump_avoidable
-        self.jump_guard = None if clone is None else clone.jump_guard # type: Optional[BV]
+        self.jump_guard: Optional[BV] = None if clone is None else clone.jump_guard
         self.jumpkind = None if clone is None else clone.jumpkind
 
         # the execution log for this history
-        self.recent_events = [ ] if clone is None else list(clone.recent_events)
-        self.recent_bbl_addrs = [ ] if clone is None else list(clone.recent_bbl_addrs)
-        self.recent_ins_addrs = [ ] if clone is None else list(clone.recent_ins_addrs)
-        self.recent_stack_actions = [ ] if clone is None else list(clone.recent_stack_actions)
+        self.recent_events = [] if clone is None else list(clone.recent_events)
+        self.recent_bbl_addrs = [] if clone is None else list(clone.recent_bbl_addrs)
+        self.recent_ins_addrs = [] if clone is None else list(clone.recent_ins_addrs)
+        self.recent_stack_actions = [] if clone is None else list(clone.recent_stack_actions)
         self.last_stmt_idx = None if clone is None else clone.last_stmt_idx
 
         # numbers of blocks, syscalls, and instructions that were executed in this step
@@ -80,10 +81,10 @@ class SimStateHistory(SimStatePlugin):
             ancestry[-1].parent = None
 
         rev_ancestry = list(reversed(ancestry))
-        d = super(SimStateHistory, self).__getstate__()
-        d['strongref_state'] = None
-        d['rev_ancestry'] = rev_ancestry
-        d['successor_ip'] = self.successor_ip
+        d = super().__getstate__()
+        d["strongref_state"] = None
+        d["rev_ancestry"] = rev_ancestry
+        d["successor_ip"] = self.successor_ip
 
         # reconstruct chain
         child = self
@@ -91,14 +92,14 @@ class SimStateHistory(SimStatePlugin):
             child.parent = parent
             child = parent
 
-        d.pop('parent')
+        d.pop("parent")
         return d
 
     def __setstate__(self, d):
         child = self
-        ancestry = list(reversed(d.pop('rev_ancestry')))
+        ancestry = list(reversed(d.pop("rev_ancestry")))
         for parent in ancestry:
-            if hasattr(child, 'parent'):
+            if hasattr(child, "parent"):
                 break
             child.parent = parent
             child = parent
@@ -110,8 +111,10 @@ class SimStateHistory(SimStatePlugin):
         addr = self.addr
         if addr is None:
             addr_str = "Unknown"
-        else:
+        elif isinstance(addr, int):
             addr_str = "%#x" % addr
+        else:
+            addr_str = repr(addr)
 
         return "<StateHistory @ %s>" % addr_str
 
@@ -126,7 +129,6 @@ class SimStateHistory(SimStatePlugin):
         return self.recent_bbl_addrs[-1]
 
     def merge(self, others, merge_conditions, common_ancestor=None):
-
         if not others:
             return False
 
@@ -137,30 +139,37 @@ class SimStateHistory(SimStatePlugin):
         # correct results when using constraints_since()
         self.parent = common_ancestor if common_ancestor is not None else self.parent
 
-        self.recent_events = [e.recent_events for e in itertools.chain([self], others)
-                              if not isinstance(e, SimActionConstraint)
-                              ]
-
         # rebuild recent constraints
-        recent_constraints = [ h.constraints_since(common_ancestor) for h in itertools.chain([self], others) ]
-        combined_constraint = self.state.solver.Or(
-            *[ self.state.solver.simplify(self.state.solver.And(*history_constraints)) for history_constraints in recent_constraints ]
-        )
+        recent_constraints = [h.constraints_since(common_ancestor) for h in itertools.chain([self], others)]
+        if sim_options.SIMPLIFY_MERGED_CONSTRAINTS in self.state.options:
+            combined_constraint = self.state.solver.Or(
+                *[
+                    self.state.solver.simplify(self.state.solver.And(*history_constraints))
+                    for history_constraints in recent_constraints
+                ]
+            )
+        else:
+            combined_constraint = self.state.solver.Or(
+                *[self.state.solver.And(*history_constraints) for history_constraints in recent_constraints]
+            )
+        self.recent_events = [
+            e.recent_events for e in itertools.chain([self], others) if not isinstance(e, SimActionConstraint)
+        ]
         self.recent_events.append(SimActionConstraint(self.state, combined_constraint))
 
         # hard to say what we should do with these others list of things...
-        #self.recent_bbl_addrs = [e.recent_bbl_addrs for e in itertools.chain([self], others)]
-        #self.recent_ins_addrs = [e.recent_ins_addrs for e in itertools.chain([self], others)]
-        #self.recent_stack_actions = [e.recent_stack_actions for e in itertools.chain([self], others)]
+        # self.recent_bbl_addrs = [e.recent_bbl_addrs for e in itertools.chain([self], others)]
+        # self.recent_ins_addrs = [e.recent_ins_addrs for e in itertools.chain([self], others)]
+        # self.recent_stack_actions = [e.recent_stack_actions for e in itertools.chain([self], others)]
 
         return True
 
-    def widen(self, others): # pylint: disable=unused-argument
+    def widen(self, others):  # pylint: disable=unused-argument
         l.warning("history widening is not implemented!")
-        return # TODO
+        return  # TODO
 
     @SimStatePlugin.memo
-    def copy(self, memo): # pylint: disable=unused-argument
+    def copy(self, memo):  # pylint: disable=unused-argument
         return SimStateHistory(clone=self)
 
     def trim(self):
@@ -169,13 +178,18 @@ class SimStateHistory(SimStatePlugin):
         """
         new_hist = self.copy({})
         new_hist.parent = None
-        self.state.register_plugin('history', new_hist)
+        self.state.register_plugin("history", new_hist)
 
-    def filter_actions(self, block_addr=None, block_stmt=None, insn_addr=None, read_from=None, write_to=None):
+    def filter_actions(
+        self, start_block_addr=None, end_block_addr=None, block_stmt=None, insn_addr=None, read_from=None, write_to=None
+    ):
         """
         Filter self.actions based on some common parameters.
 
-        :param block_addr:  Only return actions generated in blocks starting at this address.
+        [start_block_addr, end_block_addr]
+
+        :param start_block_addr:  Only return actions generated in blocks starting at this address.
+        :param end_block_addr: Only return actions generated in blocks ending at this address.
         :param block_stmt:  Only return actions generated in the nth statement of each block.
         :param insn_addr:   Only return actions generated in the assembly instruction at this address.
         :param read_from:   Only return actions that perform a read from the specified location.
@@ -195,27 +209,27 @@ class SimStateHistory(SimStatePlugin):
         if read_from is not None:
             if write_to is not None:
                 raise ValueError("Can't handle read_from and write_to at the same time!")
-            if read_from in ('reg', 'mem'):
+            if read_from in ("reg", "mem"):
                 read_type = read_from
                 read_offset = None
             elif isinstance(read_from, str):
-                read_type = 'reg'
+                read_type = "reg"
                 read_offset = self.state.project.arch.registers[read_from][0]
             else:
-                read_type = 'mem'
+                read_type = "mem"
                 read_offset = read_from
         if write_to is not None:
-            if write_to in ('reg', 'mem'):
+            if write_to in ("reg", "mem"):
                 write_type = write_to
                 write_offset = None
             elif isinstance(write_to, str):
-                write_type = 'reg'
+                write_type = "reg"
                 write_offset = self.state.project.arch.registers[write_to][0]
             else:
-                write_type = 'mem'
+                write_type = "mem"
                 write_offset = write_to
 
-        #def addr_of_stmt(bbl_addr, stmt_idx):
+        # def addr_of_stmt(bbl_addr, stmt_idx):
         #    if stmt_idx is None:
         #        return None
         #    stmts = self.state.project.factory.block(bbl_addr).vex.statements
@@ -229,7 +243,7 @@ class SimStateHistory(SimStatePlugin):
         def action_reads(action):
             if action.type != read_type:
                 return False
-            if action.action != 'read':
+            if action.action != "read":
                 return False
             if read_offset is None:
                 return True
@@ -247,7 +261,7 @@ class SimStateHistory(SimStatePlugin):
         def action_writes(action):
             if action.type != write_type:
                 return False
-            if action.action != 'write':
+            if action.action != "write":
                 return False
             if write_offset is None:
                 return True
@@ -262,46 +276,17 @@ class SimStateHistory(SimStatePlugin):
                 return False
             return True
 
-        return [x for x in reversed(self.actions) if
-                    (block_addr is None or x.bbl_addr == block_addr) and
-                    (block_stmt is None or x.stmt_idx == block_stmt) and
-                    (read_from is None or action_reads(x)) and
-                    (write_to is None or action_writes(x)) and
-                    (insn_addr is None or (x.sim_procedure is None and x.ins_addr == insn_addr))
-                    #(insn_addr is None or (x.sim_procedure is None and addr_of_stmt(x.bbl_addr, x.stmt_idx) == insn_addr))
-            ]
-
-    #def _record_state(self, state, strong_reference=True):
-    #   else:
-    #       # state.scratch.bbl_addr may not be initialized as final states from the "flat_successors" list. We need to get
-    #       # the value from _target in that case.
-    #       if self.addr is None and not self._target.symbolic:
-    #           self._addrs = [ self._target._model_concrete.value ]
-    #       else:
-    #           # FIXME: redesign so this does not happen
-    #           l.warning("Encountered a path to a SimProcedure with a symbolic target address.")
-    #
-    #   if o.UNICORN in state.options:
-    #       self.extra_length += state.scratch.executed_block_count - 1
-    #
-    #   if o.TRACK_ACTION_HISTORY in state.options:
-    #       self._events = state.history.events
-    #
-    #   # record constraints, added constraints, and satisfiability
-    #   self._all_constraints = state.solver.constraints
-    #   self._fresh_constraints = state.history.fresh_constraints
-    #
-    #   if isinstance(state.solver._solver, claripy.frontend_mixins.SatCacheMixin):
-    #       self._satisfiable = state.solver._solver._cached_satness
-    #   else:
-    #       self._satisfiable = None
-    #
-    #   # record the state as a weak reference
-    #   self._state_weak_ref = weakref.ref(state)
-    #
-    #   # and as a strong ref
-    #   if strong_reference:
-    #       self._state_strong_ref = state
+        return [
+            x
+            for x in reversed(self.actions)
+            if (start_block_addr is None or x.bbl_addr >= start_block_addr)
+            and (end_block_addr is None or x.bbl_addr <= end_block_addr)
+            and (block_stmt is None or x.stmt_idx == block_stmt)
+            and (read_from is None or action_reads(x))
+            and (write_to is None or action_writes(x))
+            and (insn_addr is None or (x.sim_procedure is None and x.ins_addr == insn_addr))
+            # (insn_addr is None or (x.sim_procedure is None and addr_of_stmt(x.bbl_addr, x.stmt_idx) == insn_addr))
+        ]
 
     def demote(self):
         """
@@ -311,13 +296,18 @@ class SimStateHistory(SimStatePlugin):
 
     def reachable(self):
         if self._satisfiable is not None:
-            pass
-        elif self.state is not None:
-            self._satisfiable = self.state.solver.satisfiable()
-        else:
-            solver = claripy.Solver()
-            solver.add(self._all_constraints)
-            self._satisfiable = solver.satisfiable()
+            return self._satisfiable
+
+        if self.state is not None:
+            try:
+                self._satisfiable = self.state.solver.satisfiable()
+                return self._satisfiable
+            except ReferenceError:
+                pass
+
+        solver = claripy.Solver()
+        solver.add(self._all_constraints)
+        self._satisfiable = solver.satisfiable()
 
         return self._satisfiable
 
@@ -349,10 +339,11 @@ class SimStateHistory(SimStatePlugin):
     @property
     def recent_constraints(self):
         # this and the below MUST be lists, not generators, because we need to reverse them
-        return [ ev.constraint for ev in self.recent_events if isinstance(ev, SimActionConstraint) ]
+        return [ev.constraint for ev in self.recent_events if isinstance(ev, SimActionConstraint)]
+
     @property
     def recent_actions(self):
-        return [ ev for ev in self.recent_events if isinstance(ev, SimAction) ]
+        return [ev for ev in self.recent_events if isinstance(ev, SimAction)]
 
     @property
     def block_count(self):
@@ -361,41 +352,51 @@ class SimStateHistory(SimStatePlugin):
     @property
     def lineage(self):
         return HistoryIter(self)
+
     @property
     def parents(self):
         if self.parent:
-            for p in self.parent.lineage:
-                yield p
+            yield from self.parent.lineage
+
     @property
     def events(self):
-        return LambdaIterIter(self, operator.attrgetter('recent_events'))
+        return LambdaIterIter(self, operator.attrgetter("recent_events"))
+
     @property
     def actions(self):
-        return LambdaIterIter(self, operator.attrgetter('recent_actions'))
+        return LambdaIterIter(self, operator.attrgetter("recent_actions"))
+
     @property
     def jumpkinds(self):
-        return LambdaAttrIter(self, operator.attrgetter('jumpkind'))
+        return LambdaAttrIter(self, operator.attrgetter("jumpkind"))
+
     @property
     def jump_guards(self):
-        return LambdaAttrIter(self, operator.attrgetter('jump_guard'))
+        return LambdaAttrIter(self, operator.attrgetter("jump_guard"))
+
     @property
     def jump_targets(self):
-        return LambdaAttrIter(self, operator.attrgetter('jump_target'))
+        return LambdaAttrIter(self, operator.attrgetter("jump_target"))
+
     @property
     def jump_sources(self):
-        return LambdaAttrIter(self, operator.attrgetter('jump_source'))
+        return LambdaAttrIter(self, operator.attrgetter("jump_source"))
+
     @property
     def descriptions(self):
-        return LambdaAttrIter(self, operator.attrgetter('recent_description'))
+        return LambdaAttrIter(self, operator.attrgetter("recent_description"))
+
     @property
     def bbl_addrs(self):
-        return LambdaIterIter(self, operator.attrgetter('recent_bbl_addrs'))
+        return LambdaIterIter(self, operator.attrgetter("recent_bbl_addrs"))
+
     @property
     def ins_addrs(self):
-        return LambdaIterIter(self, operator.attrgetter('recent_ins_addrs'))
+        return LambdaIterIter(self, operator.attrgetter("recent_ins_addrs"))
+
     @property
     def stack_actions(self):
-        return LambdaIterIter(self, operator.attrgetter('recent_stack_actions'))
+        return LambdaIterIter(self, operator.attrgetter("recent_stack_actions"))
 
     #
     # Merging support
@@ -448,7 +449,7 @@ class SimStateHistory(SimStatePlugin):
         :returns: a list of constraints
         """
 
-        constraints = [ ]
+        constraints = []
         cur = self
         while cur is not other and cur is not None:
             constraints.extend(cur.recent_constraints)
@@ -458,7 +459,8 @@ class SimStateHistory(SimStatePlugin):
     def make_child(self):
         return SimStateHistory(parent=self)
 
-class TreeIter(object):
+
+class TreeIter:
     def __init__(self, start, end=None):
         self._start = start
         self._end = end
@@ -470,8 +472,7 @@ class TreeIter(object):
             n = n.parent
 
     def __iter__(self):
-        for i in self.hardcopy:
-            yield i
+        yield from self.hardcopy
 
     def __reversed__(self):
         raise NotImplementedError("Why are you using this class")
@@ -514,8 +515,7 @@ class TreeIter(object):
 
 class HistoryIter(TreeIter):
     def __reversed__(self):
-        for hist in self._iter_nodes():
-            yield hist
+        yield from self._iter_nodes()
 
 
 class LambdaAttrIter(TreeIter):
@@ -538,13 +538,12 @@ class LambdaIterIter(LambdaAttrIter):
 
     def __reversed__(self):
         for hist in self._iter_nodes():
-            for a in reversed(self._f(hist)) if self._reverse else self._f(hist):
-                yield a
+            yield from reversed(self._f(hist)) if self._reverse else self._f(hist)
 
 
 from angr.sim_state import SimState
-SimState.register_default('history', SimStateHistory)
 
+SimState.register_default("history", SimStateHistory)
 
 from .sim_action import SimAction, SimActionConstraint
 from .sim_event import SimEvent
